@@ -1,5 +1,4 @@
 import numpy as np
-from scipy.stats import norm
 
 from autoarray import exc
 
@@ -40,7 +39,7 @@ def array_eps_to_counts(array_eps, exposure_time_map):
     exposure_time_map
         The exposure time at every data-point of the array.
     """
-    return np.multiply(array_eps, exposure_time_map)
+    return array_eps * exposure_time_map
 
 
 def array_counts_to_eps(array_counts, exposure_time_map):
@@ -59,7 +58,7 @@ def array_counts_to_eps(array_counts, exposure_time_map):
     exposure_time_map
         The exposure time at every data-point of the array.
     """
-    return np.divide(array_counts, exposure_time_map)
+    return array_counts / exposure_time_map
 
 
 def array_eps_to_adus(array_eps, exposure_time_map, gain):
@@ -80,7 +79,7 @@ def array_eps_to_adus(array_eps, exposure_time_map, gain):
     gain
         The gain of the instrument used in the conversion to / from counts and ADUs.
     """
-    return np.multiply(array_eps, exposure_time_map) / gain
+    return (array_eps * exposure_time_map) / gain
 
 
 def array_adus_to_eps(array_adus, exposure_time_map, gain):
@@ -101,11 +100,10 @@ def array_adus_to_eps(array_adus, exposure_time_map, gain):
     gain
         The gain of the instrument used in the conversion to / from counts and ADUs.
     """
-    return np.divide(gain * array_adus, exposure_time_map)
+    return (gain * array_adus) / exposure_time_map
 
 
 def array_counts_to_counts_per_second(array_counts, exposure_time):
-
     if exposure_time is None:
         raise exc.ArrayException(
             "Cannot convert a Frame2D to units counts per second without an exposure time attribute (exposure_time = None)."
@@ -149,7 +147,10 @@ def noise_map_via_data_eps_and_exposure_time_map_from(data_eps, exposure_time_ma
     exposure_time_map
         The exposure time at every data-point of the data.
     """
-    return np.sqrt(np.abs(data_eps * exposure_time_map)) / exposure_time_map
+    return data_eps.with_new_array(
+        np.abs(data_eps.array * exposure_time_map.array) ** 0.5
+        / exposure_time_map.array
+    )
 
 
 def noise_map_via_weight_map_from(weight_map):
@@ -167,12 +168,13 @@ def noise_map_via_weight_map_from(weight_map):
     Parameters
     ----------
     pixel_scales
-        The size of each pixel in scaled units.
+        The (y,x) arcsecond-to-pixel units conversion factor of every pixel. If this is input as a `float`,
+            it is converted to a (float, float).
     weight_map
         The weight-value of each pixel which is converted to a variance.
     """
     np.seterr(divide="ignore")
-    noise_map = 1.0 / np.sqrt(weight_map)
+    noise_map = 1.0 / weight_map**0.5
     noise_map[noise_map > 1.0e8] = 1.0e8
     return noise_map
 
@@ -213,14 +215,12 @@ def noise_map_via_data_eps_exposure_time_map_and_background_noise_map_from(
         of electrons per second.
     """
     return (
-        np.sqrt(
-            (
-                np.abs(data_eps * exposure_time_map)
-                + np.square(background_noise_map * exposure_time_map)
-            )
+        (
+            abs(data_eps * exposure_time_map)
+            + (background_noise_map * exposure_time_map) ** 2
         )
-        / exposure_time_map
-    )
+        ** 0.5
+    ) / exposure_time_map
 
 
 def noise_map_via_data_eps_exposure_time_map_and_background_variances_from(
@@ -241,14 +241,9 @@ def noise_map_via_data_eps_exposure_time_map_and_background_variances_from(
         of electrons per second.
     """
     return (
-        np.sqrt(
-            (
-                np.abs(data_eps * exposure_time_map)
-                + (background_variances * exposure_time_map)
-            )
-        )
-        / exposure_time_map
-    )
+        (abs(data_eps * exposure_time_map) + (background_variances * exposure_time_map))
+        ** 0.5
+    ) / exposure_time_map
 
 
 def edges_from(image, no_edges):
@@ -268,15 +263,17 @@ def edges_from(image, no_edges):
     edges = []
 
     for edge_no in range(no_edges):
-        top_edge = image.native[edge_no, edge_no : image.shape_native[1] - edge_no]
-        bottom_edge = image.native[
+        top_edge = image.native.array[
+            edge_no, edge_no : image.shape_native[1] - edge_no
+        ]
+        bottom_edge = image.native.array[
             image.shape_native[0] - 1 - edge_no,
             edge_no : image.shape_native[1] - edge_no,
         ]
-        left_edge = image.native[
+        left_edge = image.native.array[
             edge_no + 1 : image.shape_native[0] - 1 - edge_no, edge_no
         ]
-        right_edge = image.native[
+        right_edge = image.native.array[
             edge_no + 1 : image.shape_native[0] - 1 - edge_no,
             image.shape_native[1] - 1 - edge_no,
         ]
@@ -286,7 +283,7 @@ def edges_from(image, no_edges):
     return edges
 
 
-def background_sky_level_via_edges_of_image_from(image, no_edges):
+def background_sky_level_via_edges_from(image, no_edges):
     """
     Estimate the background sky level in an image using the data values at its edges. These edge values are extracted
     and their median is used to calculate the bakcground sky level.
@@ -303,7 +300,7 @@ def background_sky_level_via_edges_of_image_from(image, no_edges):
     return np.median(edges)
 
 
-def background_noise_map_via_edges_of_image_from(image, no_edges):
+def background_noise_map_via_edges_from(image, no_edges):
     """
     Estimate the background noise level in an image using the data values at its edges. These edge values are binned
     into a histogram, with a Gaussian profile fitted to this histogram, such that its standard deviation (sigma) gives
@@ -318,6 +315,7 @@ def background_noise_map_via_edges_of_image_from(image, no_edges):
     no_edges
         Number of edges used to estimate the background level.
     """
+    from scipy.stats import norm
 
     from autoarray.structures.arrays.uniform_2d import Array2D
 
@@ -333,7 +331,7 @@ def background_noise_map_via_edges_of_image_from(image, no_edges):
 def psf_with_odd_dimensions_from(psf):
     """
     If the PSF kernel has one or two even-sized dimensions, return a PSF object where the kernel has odd-sized
-    dimensions (odd-sized dimensions are required by a *Convolver*).
+    dimensions (odd-sized dimensions are required for 2D convolution).
 
     Kernels are rescaled using the scikit-image routine rescale, which performs rescaling via an interpolation
     routine. This may lead to loss of accuracy in the PSF kernel and it is advised that users, where possible,
@@ -371,7 +369,7 @@ def exposure_time_map_via_exposure_time_and_background_noise_map_from(
     relative_background_noise_map = inverse_background_noise_map / np.max(
         inverse_background_noise_map
     )
-    return np.abs(exposure_time * (relative_background_noise_map))
+    return abs(exposure_time * relative_background_noise_map)
 
 
 def setup_random_seed(seed):
@@ -411,9 +409,10 @@ def poisson_noise_via_data_eps_from(data_eps, exposure_time_map, seed=-1):
         An array describing simulated poisson noise_maps
     """
     setup_random_seed(seed)
-    image_counts = np.multiply(data_eps, exposure_time_map)
+
+    image_counts = np.multiply(data_eps.array, exposure_time_map.array)
     return data_eps - np.divide(
-        np.random.poisson(image_counts, data_eps.shape), exposure_time_map
+        np.random.poisson(image_counts, data_eps.shape), exposure_time_map.array
     )
 
 
@@ -469,7 +468,6 @@ def data_with_gaussian_noise_added(data, sigma, seed=-1):
 
 
 def data_with_complex_gaussian_noise_added(data, sigma, seed=-1):
-
     gaussian_noise = gaussian_noise_via_shape_and_sigma_from(
         shape=(data.shape[0], 2), sigma=sigma, seed=seed
     )
@@ -484,7 +482,7 @@ def noise_map_with_signal_to_noise_limit_from(
     Given data and its noise map, increase the noise-values of all data points which signal to noise is above an input
     `signal_to_noise_limit`, such that the signal to noise values do not exceed this limit.
 
-    This may be performed for imaging data with extremely high signal-to-noise regions in the data which are poorly
+    This may be performed for dataset data with extremely high signal-to-noise regions in the data which are poorly
     fit my the model. By downweighting their signal to noise values, the model-fit with focus on other parts of the
     data with low S/N.
 
@@ -512,9 +510,7 @@ def noise_map_with_signal_to_noise_limit_from(
     from autoarray.structures.arrays.uniform_1d import Array1D
     from autoarray.structures.arrays.uniform_2d import Array2D
 
-    # TODO : Refacotr into a util
-
-    signal_to_noise_map = np.divide(data, noise_map)
+    signal_to_noise_map = data / noise_map
     signal_to_noise_map[signal_to_noise_map < 0] = 0
 
     if noise_limit_mask is None:
@@ -523,12 +519,14 @@ def noise_map_with_signal_to_noise_limit_from(
     noise_map_limit = np.where(
         (signal_to_noise_map.native > signal_to_noise_limit)
         & (noise_limit_mask == False),
-        np.abs(data.native) / signal_to_noise_limit,
-        noise_map.native,
+        np.abs(data.native.array) / signal_to_noise_limit,
+        noise_map.native.array,
     )
 
     mask = Mask2D.all_false(
-        shape_native=data.shape_native, pixel_scales=data.pixel_scales
+        shape_native=data.shape_native,
+        pixel_scales=data.pixel_scales,
+        origin=data.origin,
     )
 
     if len(noise_map.native) == 1:
@@ -539,7 +537,6 @@ def noise_map_with_signal_to_noise_limit_from(
 def visibilities_noise_map_with_signal_to_noise_limit_from(
     data, noise_map, signal_to_noise_limit
 ):
-
     from autoarray.structures.visibilities import VisibilitiesNoiseMap
 
     # TODO : Refacotr into a util
