@@ -1,7 +1,9 @@
 import numpy as np
-from typing import Dict, Optional, Union
+from typing import Dict, Optional
 
-from autoarray.dataset.interferometer.interferometer import Interferometer
+from autoarray.dataset.interferometer.dataset import Interferometer
+
+from autoarray.dataset.dataset_model import DatasetModel
 from autoarray.structures.arrays.uniform_2d import Array2D
 from autoarray.structures.visibilities import Visibilities
 from autoarray.fit.fit_dataset import FitDataset
@@ -14,16 +16,19 @@ class FitInterferometer(FitDataset):
     def __init__(
         self,
         dataset: Interferometer,
+        dataset_model: DatasetModel = None,
         use_mask_in_fit: bool = False,
-        profiling_dict: Optional[Dict] = None,
     ):
-        """Class to fit a masked interferometer dataset.
+        """
+        Class to fit a masked interferometer dataset.
 
         Parameters
         ----------
         dataset : MaskedInterferometer
             The masked interferometer dataset that is fitted.
-        model_visibilities : Visibilities
+        dataset_model
+            Attributes which allow for parts of a dataset to be treated as a model (e.g. the background sky level).
+        model_data : Visibilities
             The model visibilities the masked imaging is fitted with.
         inversion : Inversion
             If the fit uses an `Inversion` this is the instance of the object used to perform the fit. This determines
@@ -51,29 +56,17 @@ class FitInterferometer(FitDataset):
 
         super().__init__(
             dataset=dataset,
+            dataset_model=dataset_model,
             use_mask_in_fit=use_mask_in_fit,
-            profiling_dict=profiling_dict,
         )
 
     @property
     def mask(self) -> np.ndarray:
-        return np.full(shape=self.visibilities.shape, fill_value=False)
-
-    @property
-    def interferometer(self) -> Interferometer:
-        return self.dataset
+        return np.full(shape=self.data.shape, fill_value=False)
 
     @property
     def transformer(self) -> ty.Transformer:
-        return self.interferometer.transformer
-
-    @property
-    def visibilities(self) -> Visibilities:
-        return self.data
-
-    @property
-    def model_visibilities(self) -> Visibilities:
-        return self.model_data
+        return self.dataset.transformer
 
     @property
     def normalized_residual_map(self) -> np.ndarray:
@@ -82,8 +75,9 @@ class FitInterferometer(FitDataset):
 
         Normalized_Residual = (Data - Model_Data) / Noise
         """
-        return fit_util.normalized_residual_map_complex_with_mask_from(
-            residual_map=self.residual_map, noise_map=self.noise_map, mask=self.mask
+        return fit_util.normalized_residual_map_complex_from(
+            residual_map=self.residual_map,
+            noise_map=self.noise_map,
         )
 
     @property
@@ -93,36 +87,31 @@ class FitInterferometer(FitDataset):
 
         Chi_Squared = ((Residuals) / (Noise)) ** 2.0 = ((Data - Model)**2.0)/(Variances)
         """
-        return fit_util.chi_squared_map_complex_with_mask_from(
-            residual_map=self.residual_map, noise_map=self.noise_map, mask=self.mask
+        return fit_util.chi_squared_map_complex_from(
+            residual_map=self.residual_map,
+            noise_map=self.noise_map,
         )
 
     @property
     def signal_to_noise_map(self) -> np.ndarray:
-        """The signal-to-noise_map of the dataset and noise-map which are fitted."""
-        signal_to_noise_map_real = np.divide(
-            np.real(self.data), np.real(self.noise_map)
-        )
+        """
+        The signal-to-noise_map of the dataset and noise-map which are fitted."""
+        signal_to_noise_map_real = self.data.real / self.noise_map.real
+
         signal_to_noise_map_real[signal_to_noise_map_real < 0] = 0.0
-        signal_to_noise_map_imag = np.divide(
-            np.imag(self.data), np.imag(self.noise_map)
-        )
+        signal_to_noise_map_imag = self.data.imag / self.noise_map.imag
+
         signal_to_noise_map_imag[signal_to_noise_map_imag < 0] = 0.0
 
         return signal_to_noise_map_real + 1.0j * signal_to_noise_map_imag
 
-    @property
-    def potential_chi_squared_map(self) -> np.ndarray:
-        """The signal-to-noise_map of the dataset and noise-map which are fitted."""
-        return self.signal_to_noise_map
-
     @property
     def chi_squared(self) -> float:
         """
         Returns the chi-squared terms of the model data's fit to an dataset, by summing the chi-squared-map.
         """
-        return fit_util.chi_squared_complex_with_mask_from(
-            chi_squared_map=self.chi_squared_map, mask=self.mask
+        return fit_util.chi_squared_complex_from(
+            chi_squared_map=self.chi_squared_map.array,
         )
 
     @property
@@ -132,13 +121,13 @@ class FitInterferometer(FitDataset):
 
         [Noise_Term] = sum(log(2*pi*[Noise]**2.0))
         """
-        return fit_util.noise_normalization_complex_with_mask_from(
-            noise_map=self.noise_map, mask=self.mask
+        return fit_util.noise_normalization_complex_from(
+            noise_map=self.noise_map.array,
         )
 
     @property
     def dirty_image(self) -> Array2D:
-        return self.transformer.image_from(visibilities=self.visibilities)
+        return self.transformer.image_from(visibilities=self.data)
 
     @property
     def dirty_noise_map(self) -> Array2D:
@@ -150,7 +139,7 @@ class FitInterferometer(FitDataset):
 
     @property
     def dirty_model_image(self) -> Array2D:
-        return self.transformer.image_from(visibilities=self.model_visibilities)
+        return self.transformer.image_from(visibilities=self.model_data)
 
     @property
     def dirty_residual_map(self) -> Array2D:
