@@ -1,10 +1,25 @@
+from functools import wraps
+import jax.numpy as jnp
 import numpy as np
 
 from autoarray.mask.abstract_mask import Mask
-from autoarray.structures.abstract_structure import Structure
+
+from autoarray import type as ty
 
 
-def residual_map_from(*, data: Structure, model_data: Structure) -> Structure:
+def to_new_array(func):
+    @wraps(func)
+    def wrapper(**kwargs):
+        result = func(**kwargs)
+        try:
+            return list(kwargs.values())[0].with_new_array(result)
+        except AttributeError:
+            return result
+
+    return wrapper
+
+
+def residual_map_from(*, data: ty.DataLike, model_data: ty.DataLike) -> ty.DataLike:
     """
     Returns the residual-map of the fit of model-data to a masked dataset, where:
 
@@ -19,12 +34,12 @@ def residual_map_from(*, data: Structure, model_data: Structure) -> Structure:
     model_data
         The model data used to fit the data.
     """
-    return np.subtract(data, model_data, out=np.zeros_like(data))
+    return data - model_data
 
 
 def normalized_residual_map_from(
-    *, residual_map: Structure, noise_map: Structure
-) -> Structure:
+    *, residual_map: ty.DataLike, noise_map: ty.DataLike
+) -> ty.DataLike:
     """
     Returns the normalized residual-map of the fit of model-data to a masked dataset, where:
 
@@ -39,10 +54,12 @@ def normalized_residual_map_from(
     mask
         The mask applied to the residual-map, where `False` entries are included in the calculation.
     """
-    return np.divide(residual_map, noise_map, out=np.zeros_like(residual_map))
+    return residual_map / noise_map
 
 
-def chi_squared_map_from(*, residual_map: Structure, noise_map: Structure) -> Structure:
+def chi_squared_map_from(
+    *, residual_map: ty.DataLike, noise_map: ty.DataLike
+) -> ty.DataLike:
     """
     Returns the chi-squared-map of the fit of model-data to a masked dataset, where:
 
@@ -55,12 +72,10 @@ def chi_squared_map_from(*, residual_map: Structure, noise_map: Structure) -> St
     noise_map
         The noise-map of the dataset.
     """
-    return np.square(
-        np.divide(residual_map, noise_map, out=np.zeros_like(residual_map))
-    )
+    return (residual_map / noise_map) ** 2.0
 
 
-def chi_squared_from(*, chi_squared_map: Structure) -> float:
+def chi_squared_from(*, chi_squared_map: ty.DataLike) -> float:
     """
     Returns the chi-squared terms of a model data's fit to an dataset, by summing the chi-squared-map.
 
@@ -69,10 +84,10 @@ def chi_squared_from(*, chi_squared_map: Structure) -> float:
     chi_squared_map
         The chi-squared-map of values of the model-data fit to the dataset.
     """
-    return float(np.sum(chi_squared_map))
+    return jnp.sum(chi_squared_map)
 
 
-def noise_normalization_from(*, noise_map: Structure) -> float:
+def noise_normalization_from(*, noise_map: ty.DataLike) -> float:
     """
     Returns the noise-map normalization term of the noise-map, summing the noise_map value in every pixel as:
 
@@ -83,12 +98,12 @@ def noise_normalization_from(*, noise_map: Structure) -> float:
     noise_map
         The masked noise-map of the dataset.
     """
-    return float(np.sum(np.log(2 * np.pi * noise_map**2.0)))
+    return jnp.sum(jnp.log(2 * jnp.pi * noise_map**2.0))
 
 
 def normalized_residual_map_complex_from(
-    *, residual_map: Structure, noise_map: Structure
-) -> Structure:
+    *, residual_map: jnp.ndarray, noise_map: jnp.ndarray
+) -> jnp.ndarray:
     """
     Returns the normalized residual-map of the fit of complex model-data to a dataset, where:
 
@@ -101,24 +116,19 @@ def normalized_residual_map_complex_from(
     noise_map
         The noise-map of the dataset.
     """
-    normalized_residual_map_real = np.divide(
-        residual_map.real,
-        noise_map.real,
-        out=np.zeros_like(residual_map, dtype="complex128"),
+    normalized_residual_map_real = (residual_map.real / noise_map.real).astype(
+        "complex128"
     )
-
-    normalized_residual_map_imag = np.divide(
-        residual_map.imag,
-        noise_map.imag,
-        out=np.zeros_like(residual_map, dtype="complex128"),
+    normalized_residual_map_imag = (residual_map.imag / noise_map.imag).astype(
+        "complex128"
     )
 
     return normalized_residual_map_real + 1j * normalized_residual_map_imag
 
 
 def chi_squared_map_complex_from(
-    *, residual_map: Structure, noise_map: Structure
-) -> Structure:
+    *, residual_map: jnp.ndarray, noise_map: jnp.ndarray
+) -> jnp.ndarray:
     """
     Returnss the chi-squared-map of the fit of complex model-data to a dataset, where:
 
@@ -131,16 +141,12 @@ def chi_squared_map_complex_from(
     noise_map
         The noise-map of the dataset.
     """
-    chi_squared_map_real = np.square(
-        np.divide(residual_map.real, noise_map.real, out=np.zeros_like(residual_map))
-    )
-    chi_squared_map_imag = np.square(
-        np.divide(residual_map.imag, noise_map.imag, out=np.zeros_like(residual_map))
-    )
+    chi_squared_map_real = (residual_map.real / noise_map.real) ** 2
+    chi_squared_map_imag = (residual_map.imag / noise_map.imag) ** 2
     return chi_squared_map_real + 1j * chi_squared_map_imag
 
 
-def chi_squared_complex_from(*, chi_squared_map: Structure) -> float:
+def chi_squared_complex_from(*, chi_squared_map: jnp.ndarray) -> float:
     """
     Returns the chi-squared terms of each complex model data's fit to a masked dataset, by summing the masked
     chi-squared-map of the fit.
@@ -152,12 +158,12 @@ def chi_squared_complex_from(*, chi_squared_map: Structure) -> float:
     chi_squared_map
         The chi-squared-map of values of the model-data fit to the dataset.
     """
-    chi_squared_real = float(np.sum(chi_squared_map.real))
-    chi_squared_imag = float(np.sum(chi_squared_map.imag))
+    chi_squared_real = jnp.sum(chi_squared_map.real)
+    chi_squared_imag = jnp.sum(chi_squared_map.imag)
     return chi_squared_real + chi_squared_imag
 
 
-def noise_normalization_complex_from(*, noise_map: Structure) -> float:
+def noise_normalization_complex_from(*, noise_map: jnp.ndarray) -> float:
     """
     Returns the noise-map normalization terms of a complex noise-map, summing the noise_map value in every pixel as:
 
@@ -168,14 +174,15 @@ def noise_normalization_complex_from(*, noise_map: Structure) -> float:
     noise_map
         The masked noise-map of the dataset.
     """
-    noise_normalization_real = float(np.sum(np.log(2 * np.pi * noise_map.real**2.0)))
-    noise_normalization_imag = float(np.sum(np.log(2 * np.pi * noise_map.imag**2.0)))
+    noise_normalization_real = jnp.sum(jnp.log(2 * jnp.pi * noise_map.real**2.0))
+    noise_normalization_imag = jnp.sum(jnp.log(2 * jnp.pi * noise_map.imag**2.0))
     return noise_normalization_real + noise_normalization_imag
 
 
+@to_new_array
 def residual_map_with_mask_from(
-    *, data: Structure, mask: Mask, model_data: Structure
-) -> Structure:
+    *, data: ty.DataLike, mask: Mask, model_data: ty.DataLike
+) -> ty.DataLike:
     """
     Returns the residual-map of the fit of model-data to a masked dataset, where:
 
@@ -192,14 +199,13 @@ def residual_map_with_mask_from(
     model_data
         The model data used to fit the data.
     """
-    return np.subtract(
-        data, model_data, out=np.zeros_like(data), where=np.asarray(mask) == 0
-    )
+    return jnp.where(jnp.asarray(mask) == 0, jnp.subtract(data, model_data), 0)
 
 
+@to_new_array
 def normalized_residual_map_with_mask_from(
-    *, residual_map: Structure, noise_map: Structure, mask: Mask
-) -> Structure:
+    *, residual_map: ty.DataLike, noise_map: ty.DataLike, mask: Mask
+) -> ty.DataLike:
     """
     Returns the normalized residual-map of the fit of model-data to a masked dataset, where:
 
@@ -216,17 +222,13 @@ def normalized_residual_map_with_mask_from(
     mask
         The mask applied to the residual-map, where `False` entries are included in the calculation.
     """
-    return np.divide(
-        residual_map,
-        noise_map,
-        out=np.zeros_like(residual_map),
-        where=np.asarray(mask) == 0,
-    )
+    return jnp.where(jnp.asarray(mask) == 0, jnp.divide(residual_map, noise_map), 0)
 
 
+@to_new_array
 def chi_squared_map_with_mask_from(
-    *, residual_map: Structure, noise_map: Structure, mask: Mask
-) -> Structure:
+    *, residual_map: ty.DataLike, noise_map: ty.DataLike, mask: Mask
+) -> ty.DataLike:
     """
     Returnss the chi-squared-map of the fit of model-data to a masked dataset, where:
 
@@ -243,17 +245,10 @@ def chi_squared_map_with_mask_from(
     mask
         The mask applied to the residual-map, where `False` entries are included in the calculation.
     """
-    return np.square(
-        np.divide(
-            residual_map,
-            noise_map,
-            out=np.zeros_like(residual_map),
-            where=np.asarray(mask) == 0,
-        )
-    )
+    return jnp.where(jnp.asarray(mask) == 0, jnp.square(residual_map / noise_map), 0)
 
 
-def chi_squared_with_mask_from(*, chi_squared_map: Structure, mask: Mask) -> float:
+def chi_squared_with_mask_from(*, chi_squared_map: ty.DataLike, mask: Mask) -> float:
     """
     Returns the chi-squared terms of each model data's fit to a masked dataset, by summing the masked
     chi-squared-map of the fit.
@@ -267,11 +262,11 @@ def chi_squared_with_mask_from(*, chi_squared_map: Structure, mask: Mask) -> flo
     mask
         The mask applied to the chi-squared-map, where `False` entries are included in the calculation.
     """
-    return float(np.sum(chi_squared_map[np.asarray(mask) == 0]))
+    return float(jnp.sum(chi_squared_map[jnp.asarray(mask) == 0]))
 
 
 def chi_squared_with_mask_fast_from(
-    *, data: Structure, mask: Mask, model_data: Structure, noise_map: Structure
+    *, data: ty.DataLike, mask: Mask, model_data: ty.DataLike, noise_map: ty.DataLike
 ) -> float:
     """
     Returns the chi-squared terms of each model data's fit to a masked dataset, by summing the masked
@@ -293,23 +288,22 @@ def chi_squared_with_mask_fast_from(
     mask
         The mask applied to the chi-squared-map, where `False` entries are included in the calculation.
     """
-
     return float(
-        np.sum(
-            np.square(
-                np.divide(
-                    np.subtract(
+        jnp.sum(
+            jnp.square(
+                jnp.divide(
+                    jnp.subtract(
                         data,
                         model_data,
-                    ),
-                    noise_map,
+                    )[jnp.asarray(mask) == 0],
+                    noise_map[jnp.asarray(mask) == 0],
                 )
-            )[np.asarray(mask) == 0]
+            )
         )
     )
 
 
-def noise_normalization_with_mask_from(*, noise_map: Structure, mask: Mask) -> float:
+def noise_normalization_with_mask_from(*, noise_map: ty.DataLike, mask: Mask) -> float:
     """
     Returns the noise-map normalization terms of masked noise-map, summing the noise_map value in every pixel as:
 
@@ -324,107 +318,13 @@ def noise_normalization_with_mask_from(*, noise_map: Structure, mask: Mask) -> f
     mask
         The mask applied to the noise-map, where `False` entries are included in the calculation.
     """
-    return float(np.sum(np.log(2 * np.pi * noise_map[np.asarray(mask) == 0] ** 2.0)))
-
-
-def normalized_residual_map_complex_with_mask_from(
-    *, residual_map: Structure, noise_map: Structure, mask: Mask
-) -> Structure:
-    """
-    Returns the normalized residual-map of the fit of complex model-data to a masked dataset, where:
-
-    Normalized_Residual = (Data - Model_Data) / Noise
-
-    The normalized residual-map values in masked pixels are returned as zero.
-
-    Parameters
-    ----------
-    residual_map
-        The residual-map of the model-data fit to the dataset.
-    noise_map
-        The noise-map of the dataset.
-    mask
-        The mask applied to the residual-map, where `False` entries are included in the calculation.
-    """
-    normalized_residual_map_real = np.divide(
-        residual_map.real,
-        noise_map.real,
-        out=np.zeros_like(residual_map.real),
-        where=np.asarray(mask) == 0,
+    return float(
+        jnp.sum(jnp.log(2 * jnp.pi * noise_map[jnp.asarray(mask) == 0] ** 2.0))
     )
 
-    normalized_residual_map_imag = np.divide(
-        residual_map.imag,
-        noise_map.imag,
-        out=np.zeros_like(residual_map.imag),
-        where=np.asarray(mask) == 0,
-    )
-
-    return normalized_residual_map_real + 1j * normalized_residual_map_imag
-
-
-def chi_squared_map_complex_with_mask_from(
-    *, residual_map: Structure, noise_map: Structure, mask: Mask
-) -> Structure:
-    """
-    Returnss the chi-squared-map of the fit of complex model-data to a masked dataset, where:
-
-    Chi_Squared = ((Residuals) / (Noise)) ** 2.0 = ((Data - Model)**2.0)/(Variances)
-
-    The chi-squared-map values in masked pixels are returned as zero.
-
-    Parameters
-    ----------
-    residual_map
-        The residual-map of the model-data fit to the dataset.
-    noise_map
-        The noise-map of the dataset.
-    mask
-        The mask applied to the residual-map, where `False` entries are included in the calculation.
-    """
-
-    chi_squared_map_real = np.square(
-        np.divide(
-            residual_map.real,
-            noise_map.real,
-            out=np.zeros_like(residual_map.real),
-            where=np.asarray(mask) == 0,
-        )
-    )
-    chi_squared_map_imag = np.square(
-        np.divide(
-            residual_map.imag,
-            noise_map.imag,
-            out=np.zeros_like(residual_map.imag),
-            where=np.asarray(mask) == 0,
-        )
-    )
-    return chi_squared_map_real + 1j * chi_squared_map_imag
-
-
-def chi_squared_complex_with_mask_from(
-    *, chi_squared_map: Structure, mask: Mask
-) -> float:
-    """
-    Returns the chi-squared terms of each complex model data's fit to a masked dataset, by summing the masked
-    chi-squared-map of the fit.
-
-    The chi-squared values in masked pixels are omitted from the calculation.
-
-    Parameters
-    ----------
-    chi_squared_map
-        The chi-squared-map of values of the model-data fit to the dataset.
-    mask
-        The mask applied to the chi-squared-map, where `False` entries are included in the calculation.
-    """
-    chi_squared_real = float(np.sum(chi_squared_map[np.asarray(mask) == 0].real))
-    chi_squared_imag = float(np.sum(chi_squared_map[np.asarray(mask) == 0].imag))
-    return chi_squared_real + chi_squared_imag
-
 
 def chi_squared_with_noise_covariance_from(
-    *, residual_map: Structure, noise_covariance_matrix_inv: np.ndarray
+    *, residual_map: ty.DataLike, noise_covariance_matrix_inv: jnp.ndarray
 ) -> float:
     """
     Returns the chi-squared value of the fit of model-data to a masked dataset, where
@@ -440,33 +340,7 @@ def chi_squared_with_noise_covariance_from(
         The inverse of the noise covariance matrix.
     """
 
-    return residual_map @ noise_covariance_matrix_inv @ residual_map
-
-
-def noise_normalization_complex_with_mask_from(
-    *, noise_map: Structure, mask: Mask
-) -> float:
-    """
-    Returns the noise-map normalization terms of a complex masked noise-map, summing the noise_map value in every pixel as:
-
-    [Noise_Term] = sum(log(2*pi*[Noise]**2.0))
-
-    The noise-map values in masked pixels are omitted from the calculation.
-
-    Parameters
-    ----------
-    noise_map
-        The masked noise-map of the dataset.
-    mask
-        The mask applied to the noise-map, where `False` entries are included in the calculation.
-    """
-    noise_normalization_real = float(
-        np.sum(np.log(2 * np.pi * noise_map[np.asarray(mask) == 0].real ** 2.0))
-    )
-    noise_normalization_imag = float(
-        np.sum(np.log(2 * np.pi * noise_map[np.asarray(mask) == 0].imag ** 2.0))
-    )
-    return noise_normalization_real + noise_normalization_imag
+    return residual_map.array @ noise_covariance_matrix_inv @ residual_map.array
 
 
 def log_likelihood_from(*, chi_squared: float, noise_normalization: float) -> float:
@@ -482,7 +356,7 @@ def log_likelihood_from(*, chi_squared: float, noise_normalization: float) -> fl
     noise_normalization
         The normalization noise_map-term for the dataset's noise-map.
     """
-    return float(-0.5 * (chi_squared + noise_normalization))
+    return -0.5 * (chi_squared + noise_normalization)
 
 
 def log_likelihood_with_regularization_from(
@@ -504,7 +378,7 @@ def log_likelihood_with_regularization_from(
     noise_normalization
         The normalization noise_map-term for the dataset's noise-map.
     """
-    return float(-0.5 * (chi_squared + regularization_term + noise_normalization))
+    return -0.5 * (chi_squared + regularization_term + noise_normalization)
 
 
 def log_evidence_from(
@@ -536,13 +410,50 @@ def log_evidence_from(
     noise_normalization
         The normalization noise_map-term for the dataset's noise-map.
     """
-    return float(
-        -0.5
-        * (
-            chi_squared
-            + regularization_term
-            + log_curvature_regularization_term
-            - log_regularization_term
-            + noise_normalization
-        )
+    return -0.5 * (
+        chi_squared
+        + regularization_term
+        + log_curvature_regularization_term
+        - log_regularization_term
+        + noise_normalization
     )
+
+
+def residual_flux_fraction_map_from(
+    *, residual_map: jnp.ndarray, data: jnp.ndarray
+) -> jnp.ndarray:
+    """
+    Returns the residual flux fraction map of the fit of model-data to a masked dataset, where:
+
+    Residual_Flux_Fraction = Residuals / Data = (Data - Model)/Data
+
+    Parameters
+    ----------
+    residual_map
+        The residual-map of the model-data fit to the dataset.
+    data
+        The data of the dataset.
+    """
+    return jnp.where(data != 0, residual_map / data, 0)
+
+
+def residual_flux_fraction_map_with_mask_from(
+    *, residual_map: jnp.ndarray, data: jnp.ndarray, mask: Mask
+) -> jnp.ndarray:
+    """
+    Returnss the residual flux fraction map of the fit of model-data to a masked dataset, where:
+
+    Residual_Flux_Fraction = Residuals / Data = (Data - Model)/Data
+
+    The residual flux fraction map values in masked pixels are returned as zero.
+
+    Parameters
+    ----------
+    residual_map
+        The residual-map of the model-data fit to the dataset.
+    data
+        The data of the dataset.
+    mask
+        The mask applied to the residual-map, where `False` entries are included in the calculation.
+    """
+    return jnp.where(mask == 0, residual_map / data, 0)
