@@ -4,21 +4,12 @@ import autoarray as aa
 
 
 def make_mask_1d_7():
-
     mask = np.array([True, True, False, False, False, True, True])
 
-    return aa.Mask1D(mask=mask, pixel_scales=(1.0,), sub_size=1)
-
-
-def make_sub_mask_1d_7():
-
-    mask = np.array([True, True, False, False, False, True, True])
-
-    return aa.Mask1D(mask=mask, pixel_scales=(1.0,), sub_size=2)
+    return aa.Mask1D(mask=mask, pixel_scales=(1.0,))
 
 
 def make_mask_2d_7x7():
-
     mask = np.array(
         [
             [True, True, True, True, True, True, True],
@@ -31,23 +22,7 @@ def make_mask_2d_7x7():
         ]
     )
 
-    return aa.Mask2D(mask=mask, pixel_scales=(1.0, 1.0), sub_size=1)
-
-
-def make_sub_mask_2d_7x7():
-    mask = np.array(
-        [
-            [True, True, True, True, True, True, True],
-            [True, True, True, True, True, True, True],
-            [True, True, False, False, False, True, True],
-            [True, True, False, False, False, True, True],
-            [True, True, False, False, False, True, True],
-            [True, True, True, True, True, True, True],
-            [True, True, True, True, True, True, True],
-        ]
-    )
-
-    return aa.Mask2D(mask=mask, sub_size=2, pixel_scales=(1.0, 1.0))
+    return aa.Mask2D(mask=mask, pixel_scales=(1.0, 1.0))
 
 
 def make_mask_2d_7x7_1_pix():
@@ -93,6 +68,10 @@ def make_array_2d_7x7():
     return aa.Array2D.ones(shape_native=(7, 7), pixel_scales=(1.0, 1.0))
 
 
+def make_array_2d_rgb_7x7():
+    return aa.Array2DRGB(values=np.ones((7, 7, 3)), mask=make_mask_2d_7x7())
+
+
 def make_layout_2d_7x7():
     return aa.Layout2D(
         shape_2d=(7, 7),
@@ -110,50 +89,36 @@ def make_grid_1d_7():
     return aa.Grid1D.from_mask(mask=make_mask_1d_7())
 
 
-def make_sub_grid_1d_7():
-    return aa.Grid1D.from_mask(mask=make_sub_mask_1d_7())
-
-
 def make_grid_2d_7x7():
     return aa.Grid2D.from_mask(mask=make_mask_2d_7x7())
 
 
-def make_sub_grid_2d_7x7():
-    return aa.Grid2D.from_mask(mask=make_sub_mask_2d_7x7())
+def make_grid_2d_sub_1_7x7():
+    return aa.Grid2D.from_mask(mask=make_mask_2d_7x7(), over_sample_size=1)
 
 
-def make_grid_2d_iterate_7x7():
-    return aa.Grid2DIterate.from_mask(
-        mask=make_mask_2d_7x7(), fractional_accuracy=0.9999, sub_steps=[2, 4, 8, 16]
-    )
+def make_grid_2d_sub_2_7x7():
+    return aa.Grid2D.from_mask(mask=make_mask_2d_7x7(), over_sample_size=2)
 
 
-def make_sub_grid_2d_7x7_simple():
-    sub_grid_2d_7x7 = make_sub_grid_2d_7x7()
-    sub_grid_2d_7x7[0] = np.array([1.0, 1.0])
-    sub_grid_2d_7x7[1] = np.array([1.0, 0.0])
-    sub_grid_2d_7x7[2] = np.array([1.0, 1.0])
-    sub_grid_2d_7x7[3] = np.array([1.0, 0.0])
-    return sub_grid_2d_7x7
+def make_grid_2d_7x7_simple():
+    grid_2d_7x7 = make_grid_2d_7x7()
+    grid_2d_7x7[0] = np.array([1.0, 1.0])
+    grid_2d_7x7[1] = np.array([1.0, 0.0])
+    grid_2d_7x7[2] = np.array([1.0, 1.0])
+    grid_2d_7x7[3] = np.array([1.0, 0.0])
+    return grid_2d_7x7
 
 
 def make_blurring_grid_2d_7x7():
     return aa.Grid2D.from_mask(mask=make_blurring_mask_2d_7x7())
 
 
-# CONVOLVERS #
-
-
-def make_convolver_7x7():
-    return aa.Convolver(mask=make_mask_2d_7x7(), kernel=make_psf_3x3())
-
-
 def make_image_7x7():
     return aa.Array2D.ones(shape_native=(7, 7), pixel_scales=(1.0, 1.0))
 
 
 def make_psf_3x3():
-
     psf = np.array([[0.0, 0.5, 0.0], [0.5, 1.0, 0.5], [0.0, 0.5, 0.0]])
 
     return aa.Kernel2D.no_mask(values=psf, pixel_scales=(1.0, 1.0))
@@ -168,7 +133,6 @@ def make_noise_map_7x7():
 
 
 def make_noise_covariance_matrix_7x7():
-
     noise_covariance_matrix_7x7 = np.eye(N=49, M=49)
 
     noise_covariance_matrix_7x7[:, 24] = 1.0
@@ -190,25 +154,46 @@ def make_grid_2d_irregular_7x7_list():
 
 def make_imaging_7x7():
     return aa.Imaging(
-        image=make_image_7x7(),
+        data=make_image_7x7(),
         psf=make_psf_3x3(),
         noise_map=make_noise_map_7x7(),
+        over_sample_size_lp=1,
+    )
+
+
+def make_imaging_7x7_sub_2():
+    return aa.Imaging(
+        data=make_image_7x7(),
+        psf=make_psf_3x3(),
+        noise_map=make_noise_map_7x7(),
+        over_sample_size_lp=2,
     )
 
 
 def make_imaging_covariance_7x7():
     return aa.Imaging(
-        image=make_image_7x7(),
+        data=make_image_7x7(),
         psf=make_psf_3x3(),
         noise_covariance_matrix=make_noise_covariance_matrix_7x7(),
+        over_sample_size_lp=1,
     )
 
 
 def make_imaging_7x7_no_blur():
     return aa.Imaging(
-        image=make_image_7x7(),
+        data=make_image_7x7(),
         psf=make_psf_3x3_no_blur(),
         noise_map=make_noise_map_7x7(),
+        over_sample_size_lp=1,
+    )
+
+
+def make_imaging_7x7_no_blur_sub_2():
+    return aa.Imaging(
+        data=make_image_7x7(),
+        psf=make_psf_3x3_no_blur(),
+        noise_map=make_noise_map_7x7(),
+        over_sample_size_lp=2,
     )
 
 
@@ -240,50 +225,41 @@ def make_uv_wavelengths_7x2_no_fft():
 
 def make_interferometer_7():
     return aa.Interferometer(
-        visibilities=make_visibilities_7(),
+        data=make_visibilities_7(),
         noise_map=make_visibilities_noise_map_7(),
         uv_wavelengths=make_uv_wavelengths_7x2(),
-        real_space_mask=make_sub_mask_2d_7x7(),
-        settings=aa.SettingsInterferometer(
-            grid_class=aa.Grid2D, sub_size=1, transformer_class=aa.TransformerDFT
-        ),
+        real_space_mask=make_mask_2d_7x7(),
+        transformer_class=aa.TransformerDFT,
     )
 
 
 def make_interferometer_7_no_fft():
     return aa.Interferometer(
-        visibilities=make_visibilities_7(),
+        data=make_visibilities_7(),
         noise_map=make_visibilities_noise_map_7(),
         uv_wavelengths=make_uv_wavelengths_7x2_no_fft(),
-        real_space_mask=make_sub_mask_2d_7x7(),
-        settings=aa.SettingsInterferometer(
-            grid_class=aa.Grid2D, sub_size=1, transformer_class=aa.TransformerDFT
-        ),
+        real_space_mask=make_mask_2d_7x7(),
+        transformer_class=aa.TransformerDFT,
     )
 
 
 def make_interferometer_7_grid():
     return aa.Interferometer(
-        visibilities=make_visibilities_7(),
+        data=make_visibilities_7(),
         noise_map=make_visibilities_noise_map_7(),
         uv_wavelengths=make_uv_wavelengths_7x2(),
-        real_space_mask=make_sub_mask_2d_7x7(),
-        settings=aa.SettingsInterferometer(
-            sub_size=1, transformer_class=aa.TransformerDFT
-        ),
+        real_space_mask=make_mask_2d_7x7(),
+        transformer_class=aa.TransformerDFT,
     )
 
 
 def make_interferometer_7_lop():
-
     return aa.Interferometer(
-        visibilities=make_visibilities_7(),
+        data=make_visibilities_7(),
         noise_map=make_visibilities_noise_map_7(),
         uv_wavelengths=make_uv_wavelengths_7x2(),
         real_space_mask=make_mask_2d_7x7(),
-        settings=aa.SettingsInterferometer(
-            sub_size_pixelization=1, transformer_class=aa.TransformerNUFFT
-        ),
+        transformer_class=aa.TransformerNUFFT,
     )
 
 
@@ -297,38 +273,39 @@ def make_transformer_7x7_7():
 
 
 def make_masked_imaging_7x7():
-
     imaging_7x7 = make_imaging_7x7()
 
     return imaging_7x7.apply_mask(mask=make_mask_2d_7x7())
 
 
 def make_masked_imaging_covariance_7x7():
-
     imaging_7x7 = make_imaging_covariance_7x7()
 
     return imaging_7x7.apply_mask(mask=make_mask_2d_7x7())
 
 
 def make_masked_imaging_7x7_no_blur():
-
     imaging_7x7 = make_imaging_7x7_no_blur()
 
     return imaging_7x7.apply_mask(mask=make_mask_2d_7x7())
 
 
-def make_model_image_7x7():
+def make_masked_imaging_7x7_no_blur_sub_2():
+    imaging_7x7 = make_imaging_7x7_no_blur_sub_2()
 
+    return imaging_7x7.apply_mask(mask=make_mask_2d_7x7())
+
+
+def make_model_image_7x7():
     imaging_7x7 = make_masked_imaging_7x7()
 
-    return 5.0 * imaging_7x7.image
+    return 5.0 * imaging_7x7.data
 
 
 def make_imaging_fit_x1_plane_7x7():
-
     imaging_7x7 = make_masked_imaging_7x7()
 
-    model_data = 5.0 * imaging_7x7.image
+    model_data = 5.0 * imaging_7x7.data
 
     return aa.m.MockFitImaging(
         dataset=imaging_7x7, use_mask_in_fit=False, model_data=model_data
@@ -336,10 +313,9 @@ def make_imaging_fit_x1_plane_7x7():
 
 
 def make_fit_interferometer_7():
-
     interferometer_7 = make_interferometer_7()
 
-    model_data = 5.0 * interferometer_7.visibilities
+    model_data = 5.0 * interferometer_7.data
 
     return aa.m.MockFitInterferometer(
         dataset=interferometer_7, use_mask_in_fit=False, model_data=model_data
@@ -366,14 +342,25 @@ def make_regularization_adaptive_brightness_split():
     )
 
 
+def make_regularization_gaussian_kernel():
+    return aa.reg.GaussianKernel(coefficient=1.0, scale=0.5)
+
+
+def make_regularization_exponential_kernel():
+    return aa.reg.ExponentialKernel(coefficient=1.0, scale=0.5)
+
+
+def make_regularization_matern_kernel():
+    return aa.reg.MaternKernel(coefficient=1.0, scale=0.5, nu=0.7)
+
+
 def make_rectangular_mesh_grid_3x3():
     return aa.Mesh2DRectangular.overlay_grid(
-        grid=make_sub_grid_2d_7x7(), shape_native=(3, 3)
+        grid=make_grid_2d_sub_2_7x7().over_sampled, shape_native=(3, 3)
     )
 
 
 def make_delaunay_mesh_grid_9():
-
     grid_9 = aa.Grid2D.no_mask(
         values=[
             [0.6, -0.3],
@@ -394,7 +381,6 @@ def make_delaunay_mesh_grid_9():
 
 
 def make_voronoi_mesh_grid_9():
-
     grid_9 = aa.Grid2D.no_mask(
         values=[
             [0.6, -0.3],
@@ -413,66 +399,68 @@ def make_voronoi_mesh_grid_9():
 
     return aa.Mesh2DVoronoi(
         values=grid_9,
-        nearest_pixelization_index_for_slim_index=np.zeros(
-            shape=make_grid_2d_7x7().shape_slim, dtype="int"
-        ),
+    )
+
+
+def make_over_sampler_2d_7x7():
+    return aa.OverSampler(mask=make_mask_2d_7x7(), sub_size=2)
+
+
+def make_border_relocator_2d_7x7():
+    return aa.BorderRelocator(
+        mask=make_mask_2d_7x7(), sub_size=np.array([2, 2, 2, 2, 2, 2, 2, 2, 2])
     )
 
 
 def make_rectangular_mapper_7x7_3x3():
-
     mapper_grids = aa.MapperGrids(
-        source_plane_data_grid=make_sub_grid_2d_7x7(),
+        mask=make_mask_2d_7x7(),
+        source_plane_data_grid=make_grid_2d_sub_2_7x7(),
         source_plane_mesh_grid=make_rectangular_mesh_grid_3x3(),
         image_plane_mesh_grid=None,
-        hyper_data=aa.Array2D.ones(shape_native=(3, 3), pixel_scales=0.1),
+        adapt_data=aa.Array2D.ones(shape_native=(3, 3), pixel_scales=0.1),
     )
 
-    return aa.MapperRectangularNoInterp(
-        mapper_grids=mapper_grids, regularization=make_regularization_constant()
+    return aa.MapperRectangularUniform(
+        mapper_grids=mapper_grids,
+        border_relocator=make_border_relocator_2d_7x7(),
+        regularization=make_regularization_constant(),
     )
 
 
 def make_delaunay_mapper_9_3x3():
-
     mapper_grids = aa.MapperGrids(
-        source_plane_data_grid=make_sub_grid_2d_7x7(),
+        mask=make_mask_2d_7x7(),
+        source_plane_data_grid=make_grid_2d_sub_2_7x7(),
         source_plane_mesh_grid=make_delaunay_mesh_grid_9(),
         image_plane_mesh_grid=aa.Grid2D.uniform(shape_native=(3, 3), pixel_scales=0.1),
-        hyper_data=aa.Array2D.ones(shape_native=(3, 3), pixel_scales=0.1),
+        adapt_data=aa.Array2D.ones(shape_native=(3, 3), pixel_scales=0.1),
     )
 
     return aa.MapperDelaunay(
-        mapper_grids=mapper_grids, regularization=make_regularization_constant()
+        mapper_grids=mapper_grids,
+        border_relocator=make_border_relocator_2d_7x7(),
+        regularization=make_regularization_constant(),
     )
 
 
 def make_voronoi_mapper_9_3x3():
     mapper_grids = aa.MapperGrids(
-        source_plane_data_grid=make_sub_grid_2d_7x7(),
+        mask=make_mask_2d_7x7(),
+        source_plane_data_grid=make_grid_2d_sub_2_7x7(),
         source_plane_mesh_grid=make_voronoi_mesh_grid_9(),
         image_plane_mesh_grid=aa.Grid2D.uniform(shape_native=(3, 3), pixel_scales=0.1),
-        hyper_data=aa.Array2D.ones(shape_native=(3, 3), pixel_scales=0.1),
+        adapt_data=aa.Array2D.ones(shape_native=(3, 3), pixel_scales=0.1),
     )
 
-    return aa.MapperVoronoiNoInterp(
-        mapper_grids=mapper_grids, regularization=make_regularization_constant()
+    return aa.MapperVoronoi(
+        mapper_grids=mapper_grids,
+        border_relocator=make_border_relocator_2d_7x7(),
+        regularization=make_regularization_constant(),
     )
 
 
-def make_voronoi_mapper_nn_9_3x3():
-    mapper_grids = aa.MapperGrids(
-        source_plane_data_grid=make_sub_grid_2d_7x7(),
-        source_plane_mesh_grid=make_voronoi_mesh_grid_9(),
-        image_plane_mesh_grid=aa.Grid2D.uniform(shape_native=(3, 3), pixel_scales=0.1),
-        hyper_data=aa.Array2D.ones(shape_native=(3, 3), pixel_scales=0.1),
-    )
-
-    return aa.MapperVoronoi(mapper_grids=mapper_grids, regularization=None)
-
-
 def make_rectangular_inversion_7x7_3x3():
-
     return aa.Inversion(
         dataset=make_masked_imaging_7x7(),
         linear_obj_list=[make_rectangular_mapper_7x7_3x3()],
@@ -480,7 +468,6 @@ def make_rectangular_inversion_7x7_3x3():
 
 
 def make_delaunay_inversion_9_3x3():
-
     return aa.Inversion(
         dataset=make_masked_imaging_7x7(),
         linear_obj_list=[make_delaunay_mapper_9_3x3()],
@@ -488,7 +475,6 @@ def make_delaunay_inversion_9_3x3():
 
 
 def make_voronoi_inversion_9_3x3():
-
     return aa.Inversion(
         dataset=make_masked_imaging_7x7(), linear_obj_list=[make_voronoi_mapper_9_3x3()]
     )
