@@ -1,7 +1,7 @@
-from typing import Tuple, Union
+import jax.numpy as jnp
 import numpy as np
+from typing import Tuple, Union
 
-from autoarray import numba_util
 from autoarray import type as ty
 
 
@@ -55,7 +55,6 @@ def convert_pixel_scales_1d(pixel_scales: ty.PixelScales) -> Tuple[float]:
     return pixel_scales
 
 
-@numba_util.jit()
 def central_pixel_coordinates_1d_from(
     shape_slim: Tuple[int],
 ) -> Union[Tuple[float], Tuple[float]]:
@@ -83,7 +82,6 @@ def central_pixel_coordinates_1d_from(
     return (float(shape_slim[0] - 1) / 2,)
 
 
-@numba_util.jit()
 def central_scaled_coordinate_1d_from(
     shape_slim: Tuple[float],
     pixel_scales: ty.PixelScales,
@@ -119,14 +117,12 @@ def central_scaled_coordinate_1d_from(
     return (x_pixel,)
 
 
-@numba_util.jit()
 def pixel_coordinates_1d_from(
     scaled_coordinates_1d: Tuple[float],
     shape_slim: Tuple[int],
     pixel_scales: ty.PixelScales,
     origins: Tuple[float] = (0.0, 0.0),
 ) -> Union[Tuple[float], Tuple[float]]:
-
     central_pixel_coordinates = central_pixel_coordinates_1d_from(shape_slim=shape_slim)
 
     x_pixel = int(
@@ -138,14 +134,12 @@ def pixel_coordinates_1d_from(
     return (x_pixel,)
 
 
-@numba_util.jit()
 def scaled_coordinates_1d_from(
     pixel_coordinates_1d: Tuple[float],
     shape_slim: Tuple[int],
     pixel_scales: ty.PixelScales,
     origins: Tuple[float] = (0.0, 0.0),
 ) -> Union[Tuple[float], Tuple[float]]:
-
     central_scaled_coordinates = central_scaled_coordinate_1d_from(
         shape_slim=shape_slim, pixel_scales=pixel_scales, origin=origins
     )
@@ -182,9 +176,8 @@ def convert_pixel_scales_2d(pixel_scales: ty.PixelScales) -> Tuple[float, float]
     return pixel_scales
 
 
-@numba_util.jit()
 def central_pixel_coordinates_2d_from(
-    shape_native: Tuple[int, int]
+    shape_native: Tuple[int, int],
 ) -> Tuple[float, float]:
     """
     Returns the central pixel coordinates of a 2D geometry (and therefore a 2D data structure like an ``Array2D``)
@@ -207,7 +200,6 @@ def central_pixel_coordinates_2d_from(
     return (float(shape_native[0] - 1) / 2, float(shape_native[1] - 1) / 2)
 
 
-@numba_util.jit()
 def central_scaled_coordinate_2d_from(
     shape_native: Tuple[int, int],
     pixel_scales: ty.PixelScales,
@@ -246,7 +238,6 @@ def central_scaled_coordinate_2d_from(
     return (y_pixel, x_pixel)
 
 
-@numba_util.jit()
 def pixel_coordinates_2d_from(
     scaled_coordinates_2d: Tuple[float, float],
     shape_native: Tuple[int, int],
@@ -310,7 +301,6 @@ def pixel_coordinates_2d_from(
     return (y_pixel, x_pixel)
 
 
-@numba_util.jit()
 def scaled_coordinates_2d_from(
     pixel_coordinates_2d: Tuple[float, float],
     shape_native: Tuple[int, int],
@@ -384,14 +374,19 @@ def transform_grid_2d_to_reference_frame(
     grid
         The 2d grid of (y, x) coordinates which are transformed to a new reference frame.
     """
-    shifted_grid_2d = np.subtract(grid_2d, centre)
-    radius = np.sqrt(np.sum(shifted_grid_2d**2.0, 1))
-    theta_coordinate_to_profile = np.arctan2(
+
+    shifted_grid_2d = grid_2d - jnp.array(centre)
+
+    radius = jnp.sqrt(jnp.sum(jnp.square(shifted_grid_2d), axis=1))
+    theta_coordinate_to_profile = jnp.arctan2(
         shifted_grid_2d[:, 0], shifted_grid_2d[:, 1]
-    ) - np.radians(angle)
-    return np.vstack(
-        radius
-        * (np.sin(theta_coordinate_to_profile), np.cos(theta_coordinate_to_profile))
+    ) - jnp.radians(angle)
+
+    return jnp.vstack(
+        [
+            radius * jnp.sin(theta_coordinate_to_profile),
+            radius * jnp.cos(theta_coordinate_to_profile),
+        ]
     ).T
 
 
@@ -413,26 +408,26 @@ def transform_grid_2d_from_reference_frame(
         The 2d grid of (y, x) coordinates which are transformed to a new reference frame.
     """
 
-    cos_angle = np.cos(np.radians(angle))
-    sin_angle = np.sin(np.radians(angle))
+    cos_angle = jnp.cos(jnp.radians(angle))
+    sin_angle = jnp.sin(jnp.radians(angle))
 
-    y = np.add(
-        np.add(
-            np.multiply(grid_2d[:, 1], sin_angle), np.multiply(grid_2d[:, 0], cos_angle)
+    y = jnp.add(
+        jnp.add(
+            jnp.multiply(grid_2d[:, 1], sin_angle),
+            jnp.multiply(grid_2d[:, 0], cos_angle),
         ),
         centre[0],
     )
-    x = np.add(
-        np.add(
-            np.multiply(grid_2d[:, 1], cos_angle),
-            -np.multiply(grid_2d[:, 0], sin_angle),
+    x = jnp.add(
+        jnp.add(
+            jnp.multiply(grid_2d[:, 1], cos_angle),
+            -jnp.multiply(grid_2d[:, 0], sin_angle),
         ),
         centre[1],
     )
-    return np.vstack((y, x)).T
+    return jnp.vstack((y, x)).T
 
 
-@numba_util.jit()
 def grid_pixels_2d_slim_from(
     grid_scaled_2d_slim: np.ndarray,
     shape_native: Tuple[int, int],
@@ -473,30 +468,15 @@ def grid_pixels_2d_slim_from(
     grid_pixels_2d_slim = grid_scaled_2d_slim_from(grid_scaled_2d_slim=grid_scaled_2d_slim, shape=(2,2),
                                                            pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
     """
-
-    grid_pixels_2d_slim = np.zeros((grid_scaled_2d_slim.shape[0], 2))
-
     centres_scaled = central_scaled_coordinate_2d_from(
         shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
     )
-
-    for slim_index in range(grid_scaled_2d_slim.shape[0]):
-
-        grid_pixels_2d_slim[slim_index, 0] = (
-            (-grid_scaled_2d_slim[slim_index, 0] / pixel_scales[0])
-            + centres_scaled[0]
-            + 0.5
-        )
-        grid_pixels_2d_slim[slim_index, 1] = (
-            (grid_scaled_2d_slim[slim_index, 1] / pixel_scales[1])
-            + centres_scaled[1]
-            + 0.5
-        )
-
-    return grid_pixels_2d_slim
+    centres_scaled = centres_scaled
+    pixel_scales = pixel_scales
+    sign = np.array([-1, 1])
+    return (sign * grid_scaled_2d_slim / pixel_scales) + centres_scaled + 0.5
 
 
-@numba_util.jit()
 def grid_pixel_centres_2d_slim_from(
     grid_scaled_2d_slim: np.ndarray,
     shape_native: Tuple[int, int],
@@ -537,29 +517,18 @@ def grid_pixel_centres_2d_slim_from(
                                                            pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
     """
 
-    grid_pixels_2d_slim = np.zeros((grid_scaled_2d_slim.shape[0], 2))
-
     centres_scaled = central_scaled_coordinate_2d_from(
         shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
     )
 
-    for slim_index in range(grid_scaled_2d_slim.shape[0]):
-
-        grid_pixels_2d_slim[slim_index, 0] = int(
-            (-grid_scaled_2d_slim[slim_index, 0] / pixel_scales[0])
-            + centres_scaled[0]
-            + 0.5
-        )
-        grid_pixels_2d_slim[slim_index, 1] = int(
-            (grid_scaled_2d_slim[slim_index, 1] / pixel_scales[1])
-            + centres_scaled[1]
-            + 0.5
-        )
-
-    return grid_pixels_2d_slim
+    centres_scaled = np.array(centres_scaled)
+    pixel_scales = np.array(pixel_scales)
+    sign = np.array([-1.0, 1.0])
+    return ((sign * grid_scaled_2d_slim / pixel_scales) + centres_scaled + 0.5).astype(
+        int
+    )
 
 
-@numba_util.jit()
 def grid_pixel_indexes_2d_slim_from(
     grid_scaled_2d_slim: np.ndarray,
     shape_native: Tuple[int, int],
@@ -612,19 +581,11 @@ def grid_pixel_indexes_2d_slim_from(
         origin=origin,
     )
 
-    grid_pixel_indexes_2d_slim = np.zeros(grid_pixels_2d_slim.shape[0])
-
-    for slim_index in range(grid_pixels_2d_slim.shape[0]):
-
-        grid_pixel_indexes_2d_slim[slim_index] = int(
-            grid_pixels_2d_slim[slim_index, 0] * shape_native[1]
-            + grid_pixels_2d_slim[slim_index, 1]
-        )
-
-    return grid_pixel_indexes_2d_slim
+    return (
+        (grid_pixels_2d_slim * np.array([shape_native[1], 1])).sum(axis=1).astype(int)
+    )
 
 
-@numba_util.jit()
 def grid_scaled_2d_slim_from(
     grid_pixels_2d_slim: np.ndarray,
     shape_native: Tuple[int, int],
@@ -663,27 +624,16 @@ def grid_scaled_2d_slim_from(
     grid_pixels_2d_slim = grid_scaled_2d_slim_from(grid_pixels_2d_slim=grid_pixels_2d_slim, shape=(2,2),
                                                            pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
     """
-
-    grid_scaled_2d_slim = np.zeros((grid_pixels_2d_slim.shape[0], 2))
-
     centres_scaled = central_scaled_coordinate_2d_from(
         shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
     )
 
-    for slim_index in range(grid_scaled_2d_slim.shape[0]):
-
-        grid_scaled_2d_slim[slim_index, 0] = (
-            -(grid_pixels_2d_slim[slim_index, 0] - centres_scaled[0] - 0.5)
-            * pixel_scales[0]
-        )
-        grid_scaled_2d_slim[slim_index, 1] = (
-            grid_pixels_2d_slim[slim_index, 1] - centres_scaled[1] - 0.5
-        ) * pixel_scales[1]
-
-    return grid_scaled_2d_slim
+    centres_scaled = np.array(centres_scaled)
+    pixel_scales = np.array(pixel_scales)
+    sign = np.array([-1, 1])
+    return (grid_pixels_2d_slim - centres_scaled - 0.5) * pixel_scales * sign
 
 
-@numba_util.jit()
 def grid_pixel_centres_2d_from(
     grid_scaled_2d: np.ndarray,
     shape_native: Tuple[int, int],
@@ -724,19 +674,49 @@ def grid_pixel_centres_2d_from(
                                                            pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
     """
 
-    grid_pixels_2d = np.zeros((grid_scaled_2d.shape[0], grid_scaled_2d.shape[1], 2))
-
     centres_scaled = central_scaled_coordinate_2d_from(
         shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
     )
 
-    for y in range(grid_scaled_2d.shape[0]):
-        for x in range(grid_scaled_2d.shape[1]):
-            grid_pixels_2d[y, x, 0] = int(
-                (-grid_scaled_2d[y, x, 0] / pixel_scales[0]) + centres_scaled[0] + 0.5
-            )
-            grid_pixels_2d[y, x, 1] = int(
-                (grid_scaled_2d[y, x, 1] / pixel_scales[1]) + centres_scaled[1] + 0.5
-            )
+    centres_scaled = np.array(centres_scaled)
+    pixel_scales = np.array(pixel_scales)
+    sign = np.array([-1.0, 1.0])
+    return ((sign * grid_scaled_2d / pixel_scales) + centres_scaled + 0.5).astype(int)
 
-    return grid_pixels_2d
+
+def extent_symmetric_from(
+    extent: Tuple[float, float, float, float],
+) -> Tuple[float, float, float, float]:
+    """
+    Given an input extent of the form (x_min, x_max, y_min, y_max), this function returns an extent which is
+    symmetric about the origin.
+
+    For example, if the sepration from x_min to x_max is 2.0 and the separation from y_min to y_max is 1.0, the
+    returned extent expands the y-axis to also have a separation of 2.0.
+
+    Parameters
+    ----------
+    extent
+        The extent which is to be made symmetric about the origin.
+
+    Returns
+    -------
+    The new extent which is symmetric about the origin.
+    """
+
+    y_min = extent[2]
+    y_max = extent[3]
+    x_min = extent[0]
+    x_max = extent[1]
+
+    y_sep = y_max - y_min
+    x_sep = x_max - x_min
+
+    if y_sep > x_sep:
+        x_min -= (y_sep - x_sep) / 2
+        x_max += (y_sep - x_sep) / 2
+    elif x_sep > y_sep:
+        y_min -= (x_sep - y_sep) / 2
+        y_max += (x_sep - y_sep) / 2
+
+    return (x_min, x_max, y_min, y_max)
