@@ -1,20 +1,22 @@
 import copy
-
+import jax.numpy as jnp
+from jax.scipy.linalg import block_diag
 import numpy as np
-from scipy.linalg import block_diag
-from scipy.sparse import csc_matrix
-from scipy.sparse.linalg import splu
-from typing import Dict, List, Optional, Tuple, Type, Union
+
+from typing import Dict, List, Optional, Type, Union
 
 from autoconf import cached_property
-from autoarray.numba_util import profile_func
 
+from autoarray.dataset.imaging.dataset import Imaging
+from autoarray.dataset.interferometer.dataset import Interferometer
+from autoarray.inversion.inversion.dataset_interface import DatasetInterface
 from autoarray.inversion.linear_obj.linear_obj import LinearObj
+from autoarray.inversion.linear_obj.func_list import AbstractLinearObjFuncList
 from autoarray.inversion.pixelization.mappers.abstract import AbstractMapper
 from autoarray.inversion.regularization.abstract import AbstractRegularization
 from autoarray.inversion.inversion.settings import SettingsInversion
+from autoarray.preloads import Preloads
 from autoarray.structures.arrays.uniform_2d import Array2D
-from autoarray.structures.grids.irregular_2d import Grid2DIrregular
 from autoarray.structures.visibilities import Visibilities
 
 from autoarray import exc
@@ -25,12 +27,10 @@ from autoarray.inversion.inversion import inversion_util
 class AbstractInversion:
     def __init__(
         self,
-        data: Union[Visibilities, Array2D],
-        noise_map: Union[Visibilities, Array2D],
+        dataset: Union[Imaging, Interferometer, DatasetInterface],
         linear_obj_list: List[LinearObj],
         settings: SettingsInversion = SettingsInversion(),
-        preloads: Optional["Preloads"] = None,
-        profiling_dict: Optional[Dict] = None,
+        preloads: Preloads = None,
     ):
         """
         An `Inversion` reconstructs an input dataset using a list of linear objects (e.g. a list of analytic functions
@@ -68,37 +68,23 @@ class AbstractInversion:
             input dataset's data and whose values are solved for via the inversion.
         settings
             Settings controlling how an inversion is fitted for example which linear algebra formalism is used.
-        preloads
-            Preloads in memory certain arrays which may be known beforehand in order to speed up the calculation,
-            for example certain matrices used by the linear algebra could be preloaded.
-        profiling_dict
-            A dictionary which contains timing of certain functions calls which is used for profiling.
         """
 
-        from autoarray.preloads import Preloads
-
-        preloads = preloads or Preloads()
-
-        try:
-            import numba
-        except ModuleNotFoundError:
-            raise exc.InversionException(
-                "Inversion functionality (linear light profiles, pixelized reconstructions) is "
-                "disabled if numba is not installed.\n\n"
-                "This is because the run-times without numba are too slow.\n\n"
-                "Please install numba, which is described at the following web page:\n\n"
-                "https://pyautolens.readthedocs.io/en/latest/installation/overview.html"
-            )
-
-        self.data = data
-        self.noise_map = noise_map
+        self.dataset = dataset
 
         self.linear_obj_list = linear_obj_list
 
         self.settings = settings
 
-        self.preloads = preloads
-        self.profiling_dict = profiling_dict
+        self.preloads = preloads or Preloads()
+
+    @property
+    def data(self):
+        return self.dataset.data
+
+    @property
+    def noise_map(self):
+        return self.dataset.noise_map
 
     def has(self, cls: Type) -> bool:
         """
@@ -161,19 +147,9 @@ class AbstractInversion:
         -------
         A list of the index range of the parameters of each linear object in the inversion of the input cls type.
         """
-        index_list = []
-
-        pixel_count = 0
-
-        for linear_obj in self.linear_obj_list:
-
-            if isinstance(linear_obj, cls):
-
-                index_list.append([pixel_count, pixel_count + linear_obj.params])
-
-            pixel_count += linear_obj.params
-
-        return index_list
+        return inversion_util.param_range_list_from(
+            cls=cls, linear_obj_list=self.linear_obj_list
+        )
 
     def cls_list_from(self, cls: Type, cls_filtered: Optional[Type] = None) -> List:
         """
@@ -260,33 +236,41 @@ class AbstractInversion:
 
     @property
     def no_regularization_index_list(self) -> List[int]:
-
         # TODO : Needs to be range based on pixels.
 
         no_regularization_index_list = []
 
-        pixel_count = 0
+        param_range_list = self.param_range_list_from(cls=LinearObj)
 
-        for linear_obj, regularization in zip(
-            self.linear_obj_list, self.regularization_list
+        for linear_obj, regularization, param_range in zip(
+            self.linear_obj_list, self.regularization_list, param_range_list
         ):
-
             if regularization is None:
-
-                for pixel in range(pixel_count, pixel_count + linear_obj.params):
-
-                    no_regularization_index_list.append(pixel)
-
-            pixel_count += linear_obj.params
+                no_regularization_index_list += range(param_range[0], param_range[1])
 
         return no_regularization_index_list
 
+    @property
+    def mapper_indices(self) -> np.ndarray:
+
+        if self.preloads.mapper_indices is not None:
+            return self.preloads.mapper_indices
+
+        mapper_indices = []
+
+        param_range_list = self.param_range_list_from(cls=AbstractMapper)
+
+        for param_range in param_range_list:
+
+            mapper_indices += range(param_range[0], param_range[1])
+
+        return np.array(mapper_indices)
+
     @property
     def mask(self) -> Array2D:
         return self.data.mask
 
     @cached_property
-    @profile_func
     def mapping_matrix(self) -> np.ndarray:
         """
         The `mapping_matrix` of a linear object describes the mappings between the observed data's data-points / pixels
@@ -302,7 +286,7 @@ class AbstractInversion:
         If there are multiple linear objects, the mapping matrices are stacked such that their simultaneous linear
         equations are solved simultaneously. This property returns the stacked mapping matrix.
         """
-        return np.hstack(
+        return jnp.hstack(
             [linear_obj.mapping_matrix for linear_obj in self.linear_obj_list]
         )
 
@@ -311,7 +295,6 @@ class AbstractInversion:
         raise NotImplementedError
 
     @cached_property
-    @profile_func
     def operated_mapping_matrix(self) -> np.ndarray:
         """
         The `operated_mapping_matrix` of a linear object describes the mappings between the observed data's values and
@@ -322,24 +305,17 @@ class AbstractInversion:
         If there are multiple linear objects, the blurred mapping matrices are stacked such that their simultaneous
         linear equations are solved simultaneously.
         """
-
-        if self.preloads.operated_mapping_matrix is not None:
-            return self.preloads.operated_mapping_matrix
-
-        return np.hstack(self.operated_mapping_matrix_list)
+        return jnp.hstack(self.operated_mapping_matrix_list)
 
     @cached_property
-    @profile_func
     def data_vector(self) -> np.ndarray:
         raise NotImplementedError
 
     @cached_property
-    @profile_func
     def curvature_matrix(self) -> np.ndarray:
         raise NotImplementedError
 
     @cached_property
-    @profile_func
     def regularization_matrix(self) -> Optional[np.ndarray]:
         """
         The regularization matrix H is used to impose smoothness on our inversion's reconstruction. This enters the
@@ -356,15 +332,11 @@ class AbstractInversion:
         If the `settings.force_edge_pixels_to_zeros` is `True`, the edge pixels of each mapper in the inversion
         are regularized so high their value is forced to zero.
         """
-        if self.preloads.regularization_matrix is not None:
-            return self.preloads.regularization_matrix
-
         return block_diag(
             *[linear_obj.regularization_matrix for linear_obj in self.linear_obj_list]
         )
 
     @cached_property
-    @profile_func
     def regularization_matrix_reduced(self) -> Optional[np.ndarray]:
         """
         The regularization matrix H is used to impose smoothness on our inversion's reconstruction. This enters the
@@ -379,22 +351,16 @@ class AbstractInversion:
         regularization it is bypassed.
         """
 
-        regularization_matrix = self.regularization_matrix
-
         if self.all_linear_obj_have_regularization:
-            return regularization_matrix
+            return self.regularization_matrix
 
-        regularization_matrix = np.delete(
-            regularization_matrix, self.no_regularization_index_list, 0
-        )
-        regularization_matrix = np.delete(
-            regularization_matrix, self.no_regularization_index_list, 1
-        )
+        # ids of values which are on edge so zero-d and not solved for.
+        ids_to_keep = self.mapper_indices
 
-        return regularization_matrix
+        # Zero rows and columns in the matrix we want to ignore
+        return self.regularization_matrix[ids_to_keep][:, ids_to_keep]
 
     @cached_property
-    @profile_func
     def curvature_reg_matrix(self) -> np.ndarray:
         """
         The linear system of equations solves for F + regularization_coefficient*H, which is computed below.
@@ -407,80 +373,99 @@ class AbstractInversion:
         if not self.has(cls=AbstractRegularization):
             return self.curvature_matrix
 
-        if len(self.regularization_list) == 1:
-
-            curvature_matrix = self.curvature_matrix
-            curvature_matrix += self.regularization_matrix
-
-            del self.__dict__["curvature_matrix"]
-
-            return curvature_matrix
-
-        return np.add(self.curvature_matrix, self.regularization_matrix)
+        return jnp.add(self.curvature_matrix, self.regularization_matrix)
 
     @cached_property
-    def curvature_reg_matrix_solver(self):
-
-        if self.settings.force_edge_pixels_to_zeros:
-
-            curvature_reg_matrix_solver = copy.copy(self.curvature_reg_matrix)
-
-            curvature_reg_matrix_solver[:, self.mapper_edge_pixel_list] = 0.0
-
-            return curvature_reg_matrix_solver
-
-        return self.curvature_reg_matrix
-
-    @cached_property
-    @profile_func
-    def curvature_reg_matrix_reduced(self) -> np.ndarray:
+    def curvature_reg_matrix_reduced(self) -> Optional[np.ndarray]:
         """
-        The linear system of equations solves for F + regularization_coefficient*H, which is computed below.
+        The regularization matrix H is used to impose smoothness on our inversion's reconstruction. This enters the
+        linear algebra system we solve for using D and F above and is given by
+        equation (12) in https://arxiv.org/pdf/astro-ph/0302587.pdf.
 
-        This is the curvature reg matrix for only the mappers, which is necessary for computing the log det
-        term without the linear light profiles included.
+        A complete description of regularization is given in the `regularization.py` and `regularization_util.py`
+        modules.
+
+        For multiple mappers, the regularization matrix is computed as the block diagonal of each individual mapper.
+        The scipy function `block_diag` has an overhead associated with it and if there is only one mapper and
+        regularization it is bypassed.
         """
+
         if self.all_linear_obj_have_regularization:
             return self.curvature_reg_matrix
 
-        curvature_reg_matrix = self.curvature_reg_matrix
+        # ids of values which are on edge so zero-d and not solved for.
+        ids_to_keep = self.mapper_indices
 
-        curvature_reg_matrix = np.delete(
-            curvature_reg_matrix, self.no_regularization_index_list, 0
-        )
-        curvature_reg_matrix = np.delete(
-            curvature_reg_matrix, self.no_regularization_index_list, 1
-        )
-
-        return curvature_reg_matrix
+        # Zero rows and columns in the matrix we want to ignore
+        return self.curvature_reg_matrix[ids_to_keep][:, ids_to_keep]
 
     @cached_property
-    @profile_func
     def reconstruction(self) -> np.ndarray:
         """
         Solve the linear system [F + reg_coeff*H] S = D -> S = [F + reg_coeff*H]^-1 D given by equation (12)
-        of https://arxiv.org/pdf/astro-ph/0302587.pdf
+        of https://arxiv.org/pdf/astro-ph/0302587.pdf (Positive-Negative solution)
 
-        S is the vector of reconstructed inversion values.
+        ============================================================================================
+
+        Solve the Eq.(2) of https://arxiv.org/pdf/astro-ph/0302587.pdf (Non-negative solution)
+        Find non-negative solution that minimizes |Z * S - x|^2.
+
+        We use fnnls (https://github.com/jvendrow/fnnls) to optimize the quadratic value. Two commonly used
+        variables in the code are defined as follows:
+            ZTZ := np.dot(Z.T, Z)
+            ZTx := np.dot(Z.T, x)
         """
         if self.settings.use_positive_only_solver:
 
-            return inversion_util.reconstruction_positive_only_from(
-                data_vector=self.data_vector,
-                curvature_reg_matrix=self.curvature_reg_matrix_solver,
-                settings=self.settings,
-            )
+            if (
+                self.preloads.source_pixel_zeroed_indices is not None
+                and self.settings.force_edge_pixels_to_zeros
+            ):
 
-        mapper_param_range_list = self.param_range_list_from(cls=AbstractMapper)
+                # ids of values which are not zeroed and therefore kept in soluiton, which is computed in preloads.
+                ids_to_keep = self.preloads.source_pixel_zeroed_indices_to_keep
+
+                total_linear_light_profiles = self.cls_list_from(
+                    cls=AbstractLinearObjFuncList
+                )
+
+                # Use advanced indexing to select rows/columns
+                data_vector = self.data_vector[ids_to_keep]
+                curvature_reg_matrix = self.curvature_reg_matrix[ids_to_keep][
+                    :, ids_to_keep
+                ]
+
+                # Perform reconstruction via fnnls
+                reconstruction_partial = (
+                    inversion_util.reconstruction_positive_only_from(
+                        data_vector=data_vector,
+                        curvature_reg_matrix=curvature_reg_matrix,
+                    )
+                )
+
+                # Allocate full solution array
+                reconstruction = jnp.zeros(self.data_vector.shape[0])
+
+                # Scatter the partial solution back to the full shape
+                reconstruction = reconstruction.at[ids_to_keep].set(
+                    reconstruction_partial
+                )
+
+                return reconstruction
+
+            else:
+
+                return inversion_util.reconstruction_positive_only_from(
+                    data_vector=self.data_vector,
+                    curvature_reg_matrix=self.curvature_reg_matrix,
+                )
 
         return inversion_util.reconstruction_positive_negative_from(
             data_vector=self.data_vector,
-            curvature_reg_matrix=self.curvature_reg_matrix_solver,
-            mapper_param_range_list=mapper_param_range_list,
+            curvature_reg_matrix=self.curvature_reg_matrix,
         )
 
     @cached_property
-    @profile_func
     def reconstruction_reduced(self) -> np.ndarray:
         """
         Solve the linear system [F + reg_coeff*H] S = D -> S = [F + reg_coeff*H]^-1 D given by equation (12)
@@ -492,7 +477,11 @@ class AbstractInversion:
         if self.all_linear_obj_have_regularization:
             return self.reconstruction
 
-        return np.delete(self.reconstruction, self.no_regularization_index_list, axis=0)
+        # ids of values which are on edge so zero-d and not solved for.
+        ids_to_keep = self.mapper_indices
+
+        # Zero rows and columns in the matrix we want to ignore
+        return self.reconstruction[ids_to_keep]
 
     @property
     def reconstruction_dict(self) -> Dict[LinearObj, np.ndarray]:
@@ -528,7 +517,6 @@ class AbstractInversion:
         index = 0
 
         for linear_obj in self.linear_obj_list:
-
             source_quantity_dict[linear_obj] = source_quantity[
                 index : index + linear_obj.params
             ]
@@ -538,7 +526,6 @@ class AbstractInversion:
         return source_quantity_dict
 
     @property
-    @profile_func
     def mapped_reconstructed_data_dict(self) -> Dict[LinearObj, Array2D]:
         raise NotImplementedError
 
@@ -559,7 +546,6 @@ class AbstractInversion:
         return self.mapped_reconstructed_data_dict
 
     @cached_property
-    @profile_func
     def mapped_reconstructed_data(self) -> Union[Array2D, Visibilities]:
         """
         Using the reconstructed source pixel fluxes we map each source pixel flux back to the image plane and
@@ -592,7 +578,34 @@ class AbstractInversion:
         return sum(self.mapped_reconstructed_image_dict.values())
 
     @cached_property
-    @profile_func
+    def data_subtracted_dict(self) -> Dict[LinearObj, Array2D]:
+        """
+        Returns a dictionary of the data subtracted by the reconstructed images of combinations of all but one of the
+        linear objects the inversion.
+
+        This produces images of the data showing what each linear object is actually fitted to, after accounting for
+        the signal in the other linear objects.
+
+        Returns
+        -------
+        A dictionary of the data subtracted by the reconstructed images of combinations of all but one of the
+        linear objects the inversion.
+        """
+
+        data_subtracted_dict = {}
+
+        for linear_obj in self.linear_obj_list:
+            data_subtracted_dict[linear_obj] = copy.copy(self.data)
+
+            for linear_obj_other in self.linear_obj_list:
+                if linear_obj != linear_obj_other:
+                    data_subtracted_dict[
+                        linear_obj
+                    ] -= self.mapped_reconstructed_image_dict[linear_obj_other]
+
+        return data_subtracted_dict
+
+    @cached_property
     def regularization_term(self) -> float:
         """
         Returns the regularization term of an inversion. This term represents the sum of the difference in flux
@@ -611,13 +624,12 @@ class AbstractInversion:
         if not self.has(cls=AbstractRegularization):
             return 0.0
 
-        return np.matmul(
+        return jnp.matmul(
             self.reconstruction_reduced.T,
-            np.matmul(self.regularization_matrix_reduced, self.reconstruction_reduced),
+            jnp.matmul(self.regularization_matrix_reduced, self.reconstruction_reduced),
         )
 
     @cached_property
-    @profile_func
     def log_det_curvature_reg_matrix_term(self) -> float:
         """
         The log determinant of [F + reg_coeff*H] is used to determine the Bayesian evidence of the solution.
@@ -629,13 +641,14 @@ class AbstractInversion:
 
         try:
             return 2.0 * np.sum(
-                np.log(np.diag(np.linalg.cholesky(self.curvature_reg_matrix_reduced)))
+                jnp.log(
+                    jnp.diag(jnp.linalg.cholesky(self.curvature_reg_matrix_reduced))
+                )
             )
         except np.linalg.LinAlgError as e:
             raise exc.InversionException() from e
 
     @cached_property
-    @profile_func
     def log_det_regularization_matrix_term(self) -> float:
         """
         The Bayesian evidence of an inversion which quantifies its overall goodness-of-fit uses the log determinant
@@ -649,183 +662,85 @@ class AbstractInversion:
         float
             The log determinant of the regularization matrix.
         """
-
         if not self.has(cls=AbstractRegularization):
             return 0.0
 
-        if self.preloads.log_det_regularization_matrix_term is not None:
-            return self.preloads.log_det_regularization_matrix_term
-
         try:
-
-            lu = splu(csc_matrix(self.regularization_matrix_reduced))
-            diagL = lu.L.diagonal()
-            diagU = lu.U.diagonal()
-            diagL = diagL.astype(np.complex128)
-            diagU = diagU.astype(np.complex128)
-
-            return np.real(np.log(diagL).sum() + np.log(diagU).sum())
-
-        except RuntimeError:
-
-            try:
-                return 2.0 * np.sum(
-                    np.log(
-                        np.diag(np.linalg.cholesky(self.regularization_matrix_reduced))
-                    )
+            return 2.0 * np.sum(
+                jnp.log(
+                    jnp.diag(jnp.linalg.cholesky(self.regularization_matrix_reduced))
                 )
-            except np.linalg.LinAlgError as e:
-                raise exc.InversionException() from e
-
-    @property
-    def errors_with_covariance(self) -> np.ndarray:
-        return np.linalg.inv(self.curvature_reg_matrix)
-
-    def interpolated_reconstruction_list_from(
-        self,
-        shape_native: Tuple[int, int] = (401, 401),
-        extent: Optional[Tuple[float, float, float, float]] = None,
-    ) -> List[Array2D]:
-        """
-        The reconstruction of an `Inversion` can be on an irregular pixelization (e.g. a Delaunay triangulation,
-        Voronoi mesh).
-
-        Analysing the reconstruction can therefore be difficult and require specific functionality tailored to using
-        this irregular grid.
-
-        This function therefore interpolates the irregular reconstruction on to a regular grid of square pixels.
-        The routine that performs the interpolation is specific to each pixelization and contained in its
-        corresponding `Mapper` and `Grid2DMesh` objects, which are called by this function.
-
-        The output interpolated reconstruction cis by default returned on a grid of 401 x 401 square pixels. This
-        can be customized by changing the `shape_native` input, and a rectangular grid with rectangular pixels can
-        be returned by instead inputting the optional `shape_scaled` tuple.
-
-        Parameters
-        ----------
-        shape_native
-            The 2D shape in pixels of the interpolated reconstruction, which is always returned using square pixels.
-        extent
-            The (x0, x1, y0, y1) extent of the grid in scaled coordinates over which the grid is created if it
-            is input.
-        """
-        return [
-            mapper.interpolated_array_from(
-                values=self.reconstruction_dict[mapper],
-                shape_native=shape_native,
-                extent=extent,
             )
-            for mapper in self.cls_list_from(cls=AbstractMapper)
-        ]
+        except np.linalg.LinAlgError as e:
+            raise exc.InversionException() from e
 
     @property
-    def errors(self):
-        return np.diagonal(self.errors_with_covariance)
-
-    @property
-    def errors_dict(self) -> Dict[LinearObj, np.ndarray]:
-        return self.source_quantity_dict_from(source_quantity=self.errors)
-
-    def interpolated_errors_list_from(
-        self,
-        shape_native: Tuple[int, int] = (401, 401),
-        extent: Optional[Tuple[float, float, float, float]] = None,
-    ) -> List[Array2D]:
+    def reconstruction_noise_map_with_covariance(self) -> np.ndarray:
         """
-        Analogous to the function `interpolated_reconstruction_list_from()` but for the error in every reconstructed
-        pixel.
+        Returns the noise-map of the reconstruction as a two dimension matrix which accounts for the covariance
+        of the noise between pixels.
 
-        See this function for a full description.
+        The diagonal of this matrix is the noise-map of the reconstruction, which can be used for analysing the
+        reconstruction with noise properties that are representative of the fit and therefore should be used
+        for any scientific analysis (e.g. source reconstructions of strong lenses).
 
-        Parameters
-        ----------
-        shape_native
-            The 2D shape in pixels of the interpolated errors, which is always returned using square pixels.
-        extent
-            The (x0, x1, y0, y1) extent of the grid in scaled coordinates over which the grid is created if it
-            is input.
+        This noise-map is defined as the RMS standard deviation of the noise in every pixel of the reconstruction.
+        This definition is identical to the `noise_map` attributes of dataset objects.
+
+        It is computed as the square root of the inverse of the curvature matrix with regularization, which is the
+        same matrix used to solve for the reconstruction via the linear inversion.
+
+        Returns
+        -------
+        The noise-map of the reconstruction as a two dimension matrix which accounts for the covariance of the noise
+        between pixels.
         """
-        return [
-            mapper.interpolated_array_from(
-                values=self.errors_dict[mapper],
-                shape_native=shape_native,
-                extent=extent,
-            )
-            for mapper in self.cls_list_from(cls=AbstractMapper)
-        ]
+        return np.sqrt(np.linalg.inv(self.curvature_reg_matrix))
 
     @property
-    def magnification_list(self) -> List[float]:
+    def reconstruction_noise_map(self):
+        """
+        Returns the noise-map of the reconstruction as a one dimensional ndarray, which does not account for the
+        covariance of the noise between pixels.
 
-        magnification_list = []
+        This matrix is representative of the noise properties of the fit and should be used for any scientific
+        analysis (e.g. source reconstructions of strong lenses).
 
-        interpolated_reconstruction_list = self.interpolated_reconstruction_list_from(
-            shape_native=(401, 401)
+        The noise-map of the reconstruction is the RMS standard deviation of the noise in every pixel of the
+        reconstruction. This definition is identical to the `noise_map` attributes of dataset objects.
+
+        It is computed as the square root of the diagonal of the `reconstruction_noise_map_with_covariance` matrix,
+        which is the same matrix used to solve for the reconstruction via the linear inversion.
+
+        Returns
+        -------
+        The noise-map of the reconstruction as a one dimensional ndarray, which does not account for the covariance
+        of the noise between pixels.
+        """
+        return np.diagonal(self.reconstruction_noise_map_with_covariance)
+
+    @property
+    def reconstruction_noise_map_dict(self) -> Dict[LinearObj, np.ndarray]:
+        return self.source_quantity_dict_from(
+            source_quantity=self.reconstruction_noise_map
         )
 
-        for i, linear_obj in enumerate(self.linear_obj_list):
-
-            mapped_reconstructed_image = self.mapped_reconstructed_image_dict[
-                linear_obj
-            ]
-            interpolated_reconstruction = interpolated_reconstruction_list[i]
-
-            magnification_list.append(
-                np.sum(mapped_reconstructed_image) / np.sum(interpolated_reconstruction)
-            )
-
-        return magnification_list
-
-    @property
-    def brightest_reconstruction_pixel_list(self):
-
-        brightest_reconstruction_pixel_list = []
-
-        for mapper in self.cls_list_from(cls=AbstractMapper):
-
-            brightest_reconstruction_pixel_list.append(
-                np.argmax(self.reconstruction_dict[mapper])
-            )
-
-        return brightest_reconstruction_pixel_list
-
-    @property
-    def brightest_reconstruction_pixel_centre_list(self):
-
-        brightest_reconstruction_pixel_centre_list = []
-
-        for mapper in self.cls_list_from(cls=AbstractMapper):
-
-            brightest_reconstruction_pixel = np.argmax(self.reconstruction_dict[mapper])
-
-            centre = Grid2DIrregular(
-                values=[mapper.source_plane_mesh_grid[brightest_reconstruction_pixel]]
-            )
-
-            brightest_reconstruction_pixel_centre_list.append(centre)
-
-        return brightest_reconstruction_pixel_centre_list
-
     def regularization_weights_from(self, index: int) -> np.ndarray:
-
         linear_obj = self.linear_obj_list[index]
         regularization = self.regularization_list[index]
 
         if regularization is None:
-
             pixels = linear_obj.params
 
-            return np.zero((pixels,))
+            return np.zeros((pixels,))
 
         return regularization.regularization_weights_from(linear_obj=linear_obj)
 
     @property
     def regularization_weights_mapper_dict(self) -> Dict[LinearObj, np.ndarray]:
-
         regularization_weights_dict = {}
 
         for index, mapper in enumerate(self.cls_list_from(cls=AbstractMapper)):
-
             regularization_weights_dict[mapper] = self.regularization_weights_from(
                 index=index
             )
@@ -833,64 +748,21 @@ class AbstractInversion:
         return regularization_weights_dict
 
     @property
-    def residual_map_mapper_dict(self) -> Dict[LinearObj, np.ndarray]:
-
-        return {
-            mapper: inversion_util.inversion_residual_map_from(
-                reconstruction=self.reconstruction_dict[mapper],
-                data=self.data,
-                slim_index_for_sub_slim_index=mapper.source_plane_data_grid.mask.derive_indexes.slim_for_sub_slim,
-                sub_slim_indexes_for_pix_index=mapper.sub_slim_indexes_for_pix_index,
-            )
-            for mapper in self.cls_list_from(cls=AbstractMapper)
-        }
+    def _data_vector_mapper(self) -> np.ndarray:
+        raise NotImplementedError
 
     @property
-    def normalized_residual_map_mapper_dict(self) -> Dict[LinearObj, np.ndarray]:
-
-        return {
-            mapper: inversion_util.inversion_normalized_residual_map_from(
-                reconstruction=self.reconstruction_dict[mapper],
-                data=self.data,
-                noise_map_1d=self.noise_map.slim,
-                slim_index_for_sub_slim_index=mapper.source_plane_data_grid.mask.derive_indexes.slim_for_sub_slim,
-                sub_slim_indexes_for_pix_index=mapper.sub_slim_indexes_for_pix_index,
-            )
-            for mapper in self.cls_list_from(cls=AbstractMapper)
-        }
+    def _curvature_matrix_mapper_diag(self) -> Optional[np.ndarray]:
+        raise NotImplementedError
 
     @property
-    def chi_squared_map_mapper_dict(self) -> Dict[LinearObj, np.ndarray]:
-
-        return {
-            mapper: inversion_util.inversion_chi_squared_map_from(
-                reconstruction=self.reconstruction_dict[mapper],
-                data=self.data,
-                noise_map_1d=self.noise_map.slim,
-                slim_index_for_sub_slim_index=mapper.source_plane_data_grid.mask.derive_indexes.slim_for_sub_slim,
-                sub_slim_indexes_for_pix_index=mapper.sub_slim_indexes_for_pix_index,
-            )
-            for mapper in self.cls_list_from(cls=AbstractMapper)
-        }
+    def linear_func_operated_mapping_matrix_dict(self) -> Dict:
+        raise NotImplementedError
 
     @property
-    def curvature_matrix_preload(self) -> np.ndarray:
-        (
-            curvature_matrix_preload,
-            curvature_matrix_counts,
-        ) = inversion_util.curvature_matrix_preload_from(
-            mapping_matrix=self.operated_mapping_matrix
-        )
-
-        return curvature_matrix_preload
+    def data_linear_func_matrix_dict(self):
+        raise NotImplementedError
 
     @property
-    def curvature_matrix_counts(self) -> np.ndarray:
-        (
-            curvature_matrix_preload,
-            curvature_matrix_counts,
-        ) = inversion_util.curvature_matrix_preload_from(
-            mapping_matrix=self.operated_mapping_matrix
-        )
-
-        return curvature_matrix_counts
+    def mapper_operated_mapping_matrix_dict(self) -> Dict:
+        raise NotImplementedError
