@@ -1,35 +1,30 @@
+import jax.numpy as jnp
 import numpy as np
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Union
 
 from autoconf import cached_property
 
-from autoarray.numba_util import profile_func
-
+from autoarray.dataset.imaging.dataset import Imaging
+from autoarray.dataset.imaging.w_tilde import WTildeImaging
+from autoarray.inversion.inversion.dataset_interface import DatasetInterface
 from autoarray.inversion.inversion.imaging.abstract import AbstractInversionImaging
 from autoarray.inversion.linear_obj.linear_obj import LinearObj
 from autoarray.inversion.inversion.settings import SettingsInversion
 from autoarray.inversion.linear_obj.func_list import AbstractLinearObjFuncList
 from autoarray.inversion.pixelization.mappers.abstract import AbstractMapper
-from autoarray.preloads import Preloads
 from autoarray.structures.arrays.uniform_2d import Array2D
-from autoarray.operators.convolver import Convolver
-from autoarray.dataset.imaging.w_tilde import WTildeImaging
 
-from autoarray.inversion.inversion import inversion_util
-from autoarray.inversion.inversion.imaging import inversion_imaging_util
+from autoarray import exc
+from autoarray.inversion.inversion.imaging import inversion_imaging_numba_util
 
 
 class InversionImagingWTilde(AbstractInversionImaging):
     def __init__(
         self,
-        data: Array2D,
-        noise_map: Array2D,
-        convolver: Convolver,
+        dataset: Union[Imaging, DatasetInterface],
         w_tilde: WTildeImaging,
         linear_obj_list: List[LinearObj],
         settings: SettingsInversion = SettingsInversion(),
-        preloads: Preloads = Preloads(),
-        profiling_dict: Optional[Dict] = None,
     ):
         """
         Constructs linear equations (via vectors and matrices) which allow for sets of simultaneous linear equations
@@ -44,38 +39,86 @@ class InversionImagingWTilde(AbstractInversionImaging):
 
         Parameters
         ----------
-        noise_map
-            The noise-map of the observed imaging data which values are solved for.
-        convolver
-            The convolver used to include 2D convolution of the mapping matrix with the imaigng data's PSF.
+        dataset
+            The dataset containing the image data, noise-map and psf which is fitted by the inversion.
         w_tilde
             An object containing matrices that construct the linear equations via the w-tilde formalism which bypasses
             the mapping matrix.
         linear_obj_list
             The linear objects used to reconstruct the data's observed values. If multiple linear objects are passed
             the simultaneous linear equations are combined and solved simultaneously.
-        profiling_dict
-            A dictionary which contains timing of certain functions calls which is used for profiling.
         """
 
+        try:
+            import numba
+        except ModuleNotFoundError:
+            raise exc.InversionException(
+                "Inversion w-tilde functionality (pixelized reconstructions) is "
+                "disabled if numba is not installed.\n\n"
+                "This is because the run-times without numba are too slow.\n\n"
+                "Please install numba, which is described at the following web page:\n\n"
+                "https://pyautolens.readthedocs.io/en/latest/installation/overview.html"
+            )
+
         super().__init__(
-            data=data,
-            noise_map=noise_map,
-            convolver=convolver,
+            dataset=dataset,
             linear_obj_list=linear_obj_list,
             settings=settings,
-            preloads=preloads,
-            profiling_dict=profiling_dict,
         )
 
         if self.settings.use_w_tilde:
             self.w_tilde = w_tilde
-            self.w_tilde.check_noise_map(noise_map=noise_map)
+            self.w_tilde.check_noise_map(noise_map=dataset.noise_map)
         else:
             self.w_tilde = None
 
     @cached_property
-    @profile_func
+    def w_tilde_data(self):
+
+        return inversion_imaging_numba_util.w_tilde_data_imaging_from(
+            image_native=np.array(self.data.native.array),
+            noise_map_native=self.noise_map.native.array,
+            kernel_native=self.psf.native.array,
+            native_index_for_slim_index=self.data.mask.derive_indexes.native_for_slim,
+        )
+
+    @property
+    def _data_vector_mapper(self) -> np.ndarray:
+        """
+        Returns the `data_vector` of all mappers, a 1D vector whose values are solved for by the simultaneous
+        linear equations constructed by this object. The object is described in full in the method `data_vector`.
+
+        This method is used to compute part of the `data_vector` if there are also linear function list objects
+        in the inversion, and is separated into a separate method to enable preloading of the mapper `data_vector`.
+        """
+
+        if not self.has(cls=AbstractMapper):
+            return None
+
+        data_vector = np.zeros(self.total_params)
+
+        mapper_list = self.cls_list_from(cls=AbstractMapper)
+        mapper_param_range = self.param_range_list_from(cls=AbstractMapper)
+
+        for mapper_index, mapper in enumerate(mapper_list):
+            data_vector_mapper = (
+                inversion_imaging_numba_util.data_vector_via_w_tilde_data_imaging_from(
+                    w_tilde_data=self.w_tilde_data,
+                    data_to_pix_unique=np.array(
+                        mapper.unique_mappings.data_to_pix_unique
+                    ),
+                    data_weights=np.array(mapper.unique_mappings.data_weights),
+                    pix_lengths=np.array(mapper.unique_mappings.pix_lengths),
+                    pix_pixels=mapper.params,
+                )
+            )
+            param_range = mapper_param_range[mapper_index]
+
+            data_vector[param_range[0] : param_range[1],] = data_vector_mapper
+
+        return data_vector
+
+    @cached_property
     def data_vector(self) -> np.ndarray:
         """
         Returns the `data_vector`, a 1D vector whose values are solved for by the simultaneous linear equations
@@ -95,18 +138,7 @@ class InversionImagingWTilde(AbstractInversionImaging):
             return self._data_vector_x1_mapper
         return self._data_vector_multi_mapper
 
-    @cached_property
-    @profile_func
-    def w_tilde_data(self):
-        return inversion_imaging_util.w_tilde_data_imaging_from(
-            image_native=self.data.native,
-            noise_map_native=self.noise_map.native,
-            kernel_native=self.convolver.kernel.native,
-            native_index_for_slim_index=self.data.mask.derive_indexes.native_for_slim,
-        )
-
     @property
-    @profile_func
     def _data_vector_x1_mapper(self) -> np.ndarray:
         """
         Returns the `data_vector`, a 1D vector whose values are solved for by the simultaneous linear equations
@@ -117,7 +149,7 @@ class InversionImagingWTilde(AbstractInversionImaging):
         """
         linear_obj = self.linear_obj_list[0]
 
-        return inversion_imaging_util.data_vector_via_w_tilde_data_imaging_from(
+        return inversion_imaging_numba_util.data_vector_via_w_tilde_data_imaging_from(
             w_tilde_data=self.w_tilde_data,
             data_to_pix_unique=linear_obj.unique_mappings.data_to_pix_unique,
             data_weights=linear_obj.unique_mappings.data_weights,
@@ -126,7 +158,6 @@ class InversionImagingWTilde(AbstractInversionImaging):
         )
 
     @property
-    @profile_func
     def _data_vector_multi_mapper(self) -> np.ndarray:
         """
         Returns the `data_vector`, a 1D vector whose values are solved for by the simultaneous linear equations
@@ -135,9 +166,10 @@ class InversionImagingWTilde(AbstractInversionImaging):
         This method computes the `data_vector` when there are multiple mapper objects in the `Inversion`,
         which computes the `data_vector` of each object and concatenates them.
         """
+
         return np.concatenate(
             [
-                inversion_imaging_util.data_vector_via_w_tilde_data_imaging_from(
+                inversion_imaging_numba_util.data_vector_via_w_tilde_data_imaging_from(
                     w_tilde_data=self.w_tilde_data,
                     data_to_pix_unique=linear_obj.unique_mappings.data_to_pix_unique,
                     data_weights=linear_obj.unique_mappings.data_weights,
@@ -149,7 +181,6 @@ class InversionImagingWTilde(AbstractInversionImaging):
         )
 
     @property
-    @profile_func
     def _data_vector_func_list_and_mapper(self) -> np.ndarray:
         """
         Returns the `data_vector`, a 1D vector whose values are solved for by the simultaneous linear equations
@@ -157,60 +188,37 @@ class InversionImagingWTilde(AbstractInversionImaging):
 
         This method computes the `data_vector` when there are one or more mapper objects in the `Inversion`,
         which are combined with linear function list objects.
+
+        The `data_vector` corresponding to all mapper objects is computed first, in a separate function. This
+        separation of functions enables the `data_vector` to be preloaded in certain circumstances.
         """
 
-        data_vector = np.zeros(self.total_params)
-
-        mapper_list = self.cls_list_from(cls=AbstractMapper)
-        mapper_param_range = self.param_range_list_from(cls=AbstractMapper)
+        data_vector = np.array(self._data_vector_mapper)
 
         linear_func_param_range = self.param_range_list_from(
             cls=AbstractLinearObjFuncList
         )
 
-        for mapper_index, mapper in enumerate(mapper_list):
+        for linear_func_index, linear_func in enumerate(
+            self.cls_list_from(cls=AbstractLinearObjFuncList)
+        ):
+            operated_mapping_matrix = self.linear_func_operated_mapping_matrix_dict[
+                linear_func
+            ]
 
-            data_vector_mapper = (
-                inversion_imaging_util.data_vector_via_w_tilde_data_imaging_from(
-                    w_tilde_data=self.w_tilde_data,
-                    data_to_pix_unique=mapper.unique_mappings.data_to_pix_unique,
-                    data_weights=mapper.unique_mappings.data_weights,
-                    pix_lengths=mapper.unique_mappings.pix_lengths,
-                    pix_pixels=mapper.params,
-                )
+            diag = inversion_imaging_numba_util.data_vector_via_blurred_mapping_matrix_from(
+                blurred_mapping_matrix=np.array(operated_mapping_matrix),
+                image=self.data.array,
+                noise_map=self.noise_map.array,
             )
-            param_range = mapper_param_range[mapper_index]
 
-            data_vector[
-                param_range[0] : param_range[1],
-            ] = data_vector_mapper
+            param_range = linear_func_param_range[linear_func_index]
 
-            for linear_func_index, linear_func in enumerate(
-                self.cls_list_from(cls=AbstractLinearObjFuncList)
-            ):
-
-                operated_mapping_matrix = self.linear_func_operated_mapping_matrix_dict[
-                    linear_func
-                ]
-
-                diag = (
-                    inversion_imaging_util.data_vector_via_blurred_mapping_matrix_from(
-                        blurred_mapping_matrix=operated_mapping_matrix,
-                        image=self.data,
-                        noise_map=self.noise_map,
-                    )
-                )
-
-                param_range = linear_func_param_range[linear_func_index]
-
-                data_vector[
-                    param_range[0] : param_range[1],
-                ] = diag
+            data_vector[param_range[0] : param_range[1],] = diag
 
         return data_vector
 
     @cached_property
-    @profile_func
     def curvature_matrix(self) -> np.ndarray:
         """
         Returns the `curvature_matrix`, a 2D matrix which uses the mappings between the data and the linear objects to
@@ -233,14 +241,29 @@ class InversionImagingWTilde(AbstractInversionImaging):
         """
 
         if self.has(cls=AbstractLinearObjFuncList):
-            return self._curvature_matrix_func_list_and_mapper
+            curvature_matrix = self._curvature_matrix_func_list_and_mapper
         elif self.total(cls=AbstractMapper) == 1:
-            return self._curvature_matrix_x1_mapper
-        return self._curvature_matrix_multi_mapper
+            curvature_matrix = self._curvature_matrix_x1_mapper
+        else:
+            curvature_matrix = self._curvature_matrix_multi_mapper
+
+        curvature_matrix = inversion_imaging_numba_util.curvature_matrix_mirrored_from(
+            curvature_matrix=curvature_matrix
+        )
+
+        if len(self.no_regularization_index_list) > 0:
+            curvature_matrix = (
+                inversion_imaging_numba_util.curvature_matrix_with_added_to_diag_from(
+                    curvature_matrix=curvature_matrix,
+                    value=self.settings.no_regularization_add_to_curvature_diag_value,
+                    no_regularization_index_list=self.no_regularization_index_list,
+                )
+            )
+
+        return curvature_matrix
 
     @property
-    @profile_func
-    def _curvature_matrix_mapper_diag(self) -> np.ndarray:
+    def _curvature_matrix_mapper_diag(self) -> Optional[np.ndarray]:
         """
         Returns the diagonal regions of the `curvature_matrix`, a 2D matrix which uses the mappings between the data
         and the linear objects to construct the simultaneous linear equations. The object is described in full in
@@ -250,8 +273,8 @@ class InversionImagingWTilde(AbstractInversionImaging):
         other calculations to enable preloading of this calculation.
         """
 
-        if self.preloads.curvature_matrix_mapper_diag is not None:
-            return self.preloads.curvature_matrix_mapper_diag
+        if not self.has(cls=AbstractMapper):
+            return None
 
         curvature_matrix = np.zeros((self.total_params, self.total_params))
 
@@ -259,17 +282,18 @@ class InversionImagingWTilde(AbstractInversionImaging):
         mapper_param_range_list = self.param_range_list_from(cls=AbstractMapper)
 
         for i in range(len(mapper_list)):
-
             mapper_i = mapper_list[i]
             mapper_param_range_i = mapper_param_range_list[i]
 
-            diag = inversion_imaging_util.curvature_matrix_via_w_tilde_curvature_preload_imaging_from(
+            diag = inversion_imaging_numba_util.curvature_matrix_via_w_tilde_curvature_preload_imaging_from(
                 curvature_preload=self.w_tilde.curvature_preload,
                 curvature_indexes=self.w_tilde.indexes,
                 curvature_lengths=self.w_tilde.lengths,
-                data_to_pix_unique=mapper_i.unique_mappings.data_to_pix_unique,
-                data_weights=mapper_i.unique_mappings.data_weights,
-                pix_lengths=mapper_i.unique_mappings.pix_lengths,
+                data_to_pix_unique=np.array(
+                    mapper_i.unique_mappings.data_to_pix_unique
+                ),
+                data_weights=np.array(mapper_i.unique_mappings.data_weights),
+                pix_lengths=np.array(mapper_i.unique_mappings.pix_lengths),
                 pix_pixels=mapper_i.params,
             )
 
@@ -281,13 +305,8 @@ class InversionImagingWTilde(AbstractInversionImaging):
             if self.total(cls=AbstractMapper) == 1:
                 return curvature_matrix
 
-        curvature_matrix = inversion_util.curvature_matrix_mirrored_from(
-            curvature_matrix=curvature_matrix
-        )
-
         return curvature_matrix
 
-    @profile_func
     def _curvature_matrix_off_diag_from(
         self, mapper_0: AbstractMapper, mapper_1: AbstractMapper
     ) -> np.ndarray:
@@ -301,7 +320,7 @@ class InversionImagingWTilde(AbstractInversionImaging):
         This function computes the off-diagonal terms of F using the w_tilde formalism.
         """
 
-        curvature_matrix_off_diag_0 = inversion_imaging_util.curvature_matrix_off_diags_via_w_tilde_curvature_preload_imaging_from(
+        curvature_matrix_off_diag_0 = inversion_imaging_numba_util.curvature_matrix_off_diags_via_w_tilde_curvature_preload_imaging_from(
             curvature_preload=self.w_tilde.curvature_preload,
             curvature_indexes=self.w_tilde.indexes,
             curvature_lengths=self.w_tilde.lengths,
@@ -315,7 +334,7 @@ class InversionImagingWTilde(AbstractInversionImaging):
             pix_pixels_1=mapper_1.params,
         )
 
-        curvature_matrix_off_diag_1 = inversion_imaging_util.curvature_matrix_off_diags_via_w_tilde_curvature_preload_imaging_from(
+        curvature_matrix_off_diag_1 = inversion_imaging_numba_util.curvature_matrix_off_diags_via_w_tilde_curvature_preload_imaging_from(
             curvature_preload=self.w_tilde.curvature_preload,
             curvature_indexes=self.w_tilde.indexes,
             curvature_lengths=self.w_tilde.lengths,
@@ -332,7 +351,6 @@ class InversionImagingWTilde(AbstractInversionImaging):
         return curvature_matrix_off_diag_0 + curvature_matrix_off_diag_1.T
 
     @property
-    @profile_func
     def _curvature_matrix_x1_mapper(self) -> np.ndarray:
         """
         Returns the `curvature_matrix`, a 2D matrix which uses the mappings between the data and the linear objects to
@@ -344,7 +362,6 @@ class InversionImagingWTilde(AbstractInversionImaging):
         return self._curvature_matrix_mapper_diag
 
     @property
-    @profile_func
     def _curvature_matrix_multi_mapper(self) -> np.ndarray:
         """
         Returns the `curvature_matrix`, a 2D matrix which uses the mappings between the data and the linear objects to
@@ -363,12 +380,10 @@ class InversionImagingWTilde(AbstractInversionImaging):
         mapper_param_range_list = self.param_range_list_from(cls=AbstractMapper)
 
         for i in range(len(mapper_list)):
-
             mapper_i = mapper_list[i]
             mapper_param_range_i = mapper_param_range_list[i]
 
             for j in range(i + 1, len(mapper_list)):
-
                 mapper_j = mapper_list[j]
                 mapper_param_range_j = mapper_param_range_list[j]
 
@@ -381,14 +396,9 @@ class InversionImagingWTilde(AbstractInversionImaging):
                     mapper_param_range_j[0] : mapper_param_range_j[1],
                 ] = off_diag
 
-        curvature_matrix = inversion_util.curvature_matrix_mirrored_from(
-            curvature_matrix=curvature_matrix
-        )
-
         return curvature_matrix
 
     @property
-    @profile_func
     def _curvature_matrix_func_list_and_mapper(self) -> np.ndarray:
         """
         The `curvature_matrix` is a 2D matrix which uses the mappings between the data and the linear objects to
@@ -411,22 +421,25 @@ class InversionImagingWTilde(AbstractInversionImaging):
         )
 
         for i in range(len(mapper_list)):
-
             mapper = mapper_list[i]
             mapper_param_range = mapper_param_range_list[i]
 
             for func_index, linear_func in enumerate(linear_func_list):
-
                 linear_func_param_range = linear_func_param_range_list[func_index]
 
-                off_diag = inversion_imaging_util.curvature_matrix_off_diags_via_mapper_and_linear_func_curvature_vector_from(
+                curvature_weights = (
+                    self.linear_func_operated_mapping_matrix_dict[linear_func]
+                    / self.noise_map[:, None] ** 2
+                )
+
+                off_diag = inversion_imaging_numba_util.curvature_matrix_off_diags_via_mapper_and_linear_func_curvature_vector_from(
                     data_to_pix_unique=mapper.unique_mappings.data_to_pix_unique,
                     data_weights=mapper.unique_mappings.data_weights,
                     pix_lengths=mapper.unique_mappings.pix_lengths,
                     pix_pixels=mapper.params,
-                    curvature_vector=self.linear_func_curvature_vectors_dict[
-                        linear_func
-                    ],
+                    curvature_weights=np.array(curvature_weights),
+                    mask=self.mask.array,
+                    psf_kernel=self.psf.native.array,
                 )
 
                 curvature_matrix[
@@ -435,16 +448,24 @@ class InversionImagingWTilde(AbstractInversionImaging):
                 ] = off_diag
 
         for index_0, linear_func_0 in enumerate(linear_func_list):
-
             linear_func_param_range_0 = linear_func_param_range_list[index_0]
 
-            for index_1, linear_func_1 in enumerate(linear_func_list):
+            weighted_vector_0 = (
+                self.linear_func_operated_mapping_matrix_dict[linear_func_0]
+                / self.noise_map[:, None]
+            )
 
+            for index_1, linear_func_1 in enumerate(linear_func_list):
                 linear_func_param_range_1 = linear_func_param_range_list[index_1]
 
+                weighted_vector_1 = (
+                    self.linear_func_operated_mapping_matrix_dict[linear_func_1]
+                    / self.noise_map[:, None]
+                )
+
                 diag = np.dot(
-                    self.linear_func_weighted_mapping_vectors_dict[linear_func_0].T,
-                    self.linear_func_weighted_mapping_vectors_dict[linear_func_1],
+                    weighted_vector_0.T,
+                    weighted_vector_1,
                 )
 
                 curvature_matrix[
@@ -452,14 +473,9 @@ class InversionImagingWTilde(AbstractInversionImaging):
                     linear_func_param_range_1[0] : linear_func_param_range_1[1],
                 ] = diag
 
-        curvature_matrix = inversion_util.curvature_matrix_mirrored_from(
-            curvature_matrix=curvature_matrix
-        )
-
         return curvature_matrix
 
     @property
-    @profile_func
     def mapped_reconstructed_data_dict(self) -> Dict[LinearObj, Array2D]:
         """
         When constructing the simultaneous linear equations (via vectors and matrices) the quantities of each individual
@@ -492,25 +508,27 @@ class InversionImagingWTilde(AbstractInversionImaging):
         )
 
         for linear_obj in self.linear_obj_list:
-
             reconstruction = reconstruction_dict[linear_obj]
 
             if isinstance(linear_obj, AbstractMapper):
-
-                mapped_reconstructed_image = inversion_util.mapped_reconstructed_data_via_image_to_pix_unique_from(
+                mapped_reconstructed_image = inversion_imaging_numba_util.mapped_reconstructed_data_via_image_to_pix_unique_from(
                     data_to_pix_unique=linear_obj.unique_mappings.data_to_pix_unique,
                     data_weights=linear_obj.unique_mappings.data_weights,
                     pix_lengths=linear_obj.unique_mappings.pix_lengths,
-                    reconstruction=reconstruction,
+                    reconstruction=np.array(reconstruction),
                 )
 
                 mapped_reconstructed_image = Array2D(
-                    values=mapped_reconstructed_image,
-                    mask=self.mask.derive_mask.sub_1,
+                    values=mapped_reconstructed_image, mask=self.mask
                 )
 
-                mapped_reconstructed_image = self.convolver.convolve_image_no_blurring(
-                    image=mapped_reconstructed_image
+                mapped_reconstructed_image = self.psf.convolved_image_from(
+                    image=mapped_reconstructed_image,
+                    blurring_image=None,
+                ).array
+
+                mapped_reconstructed_image = Array2D(
+                    values=mapped_reconstructed_image, mask=self.mask
                 )
 
             else:
@@ -524,8 +542,7 @@ class InversionImagingWTilde(AbstractInversionImaging):
                 )
 
                 mapped_reconstructed_image = Array2D(
-                    values=mapped_reconstructed_image,
-                    mask=self.mask.derive_mask.sub_1,
+                    values=mapped_reconstructed_image, mask=self.mask
                 )
 
             mapped_reconstructed_data_dict[linear_obj] = mapped_reconstructed_image
