@@ -1,9 +1,9 @@
 import numpy as np
 from typing import List
 
+from autoarray.inversion.inversion.dataset_interface import DatasetInterface
 from autoarray.inversion.inversion.abstract import AbstractInversion
 from autoarray.inversion.inversion.settings import SettingsInversion
-from autoarray.preloads import Preloads
 
 
 class MockInversion(AbstractInversion):
@@ -15,37 +15,42 @@ class MockInversion(AbstractInversion):
         operated_mapping_matrix=None,
         data_vector=None,
         curvature_matrix=None,
+        data_vector_mapper=None,
         curvature_matrix_mapper_diag=None,
+        mapper_operated_mapping_matrix_dict=None,
         regularization_matrix=None,
         curvature_reg_matrix=None,
         reconstruction: np.ndarray = None,
         reconstruction_dict: List[np.ndarray] = None,
         mapped_reconstructed_data_dict=None,
         mapped_reconstructed_image_dict=None,
-        errors: np.ndarray = None,
-        errors_dict: List[np.ndarray] = None,
+        reconstruction_noise_map: np.ndarray = None,
+        reconstruction_noise_map_dict: List[np.ndarray] = None,
         regularization_term=None,
         log_det_curvature_reg_matrix_term=None,
         log_det_regularization_matrix_term=None,
-        curvature_matrix_preload=None,
-        curvature_matrix_counts=None,
-        settings: SettingsInversion = SettingsInversion(),
-        preloads: Preloads = Preloads(),
+        settings: SettingsInversion = None,
     ):
-
-        super().__init__(
+        dataset = DatasetInterface(
             data=data,
             noise_map=noise_map,
+        )
+
+        settings = settings or SettingsInversion()
+
+        super().__init__(
+            dataset=dataset,
             linear_obj_list=linear_obj_list or [],
             settings=settings,
-            preloads=preloads,
         )
 
         self._operated_mapping_matrix = operated_mapping_matrix
         self._data_vector = data_vector
         self._regularization_matrix = regularization_matrix
         self._curvature_matrix = curvature_matrix
-        self._curvature_matrix_mapper_diag = curvature_matrix_mapper_diag
+        self.__data_vector_mapper = data_vector_mapper
+        self.__curvature_matrix_mapper_diag = curvature_matrix_mapper_diag
+        self._mapper_operated_mapping_matrix_dict = mapper_operated_mapping_matrix_dict
         self._curvature_reg_matrix = curvature_reg_matrix
         self._reconstruction = reconstruction
         self._reconstruction_dict = reconstruction_dict
@@ -53,16 +58,13 @@ class MockInversion(AbstractInversion):
         self._mapped_reconstructed_data_dict = mapped_reconstructed_data_dict
         self._mapped_reconstructed_image_dict = mapped_reconstructed_image_dict
 
-        self._errors = errors
-        self._errors_dict = errors_dict
+        self._reconstruction_noise_map = reconstruction_noise_map
+        self._reconstruction_noise_map_dict = reconstruction_noise_map_dict
 
         self._regularization_term = regularization_term
         self._log_det_curvature_reg_matrix_term = log_det_curvature_reg_matrix_term
         self._log_det_regularization_matrix_term = log_det_regularization_matrix_term
 
-        self._curvature_matrix_preload = curvature_matrix_preload
-        self._curvature_matrix_counts = curvature_matrix_counts
-
     @property
     def operated_mapping_matrix(self) -> np.ndarray:
         if self._operated_mapping_matrix is None:
@@ -78,7 +80,6 @@ class MockInversion(AbstractInversion):
 
     @property
     def regularization_matrix(self):
-
         if self._regularization_matrix is None:
             return super().regularization_matrix
 
@@ -86,19 +87,31 @@ class MockInversion(AbstractInversion):
 
     @property
     def curvature_matrix(self):
-
         if self._curvature_matrix is None:
             return super().curvature_matrix
 
         return self._curvature_matrix
 
     @property
-    def curvature_matrix_mapper_diag(self):
+    def _data_vector_mapper(self):
+        if self.__data_vector_mapper is None:
+            return super()._data_vector_mapper
 
-        if self._curvature_matrix_mapper_diag is None:
+        return self.__data_vector_mapper
+
+    @property
+    def _curvature_matrix_mapper_diag(self):
+        if self.__curvature_matrix_mapper_diag is None:
             return super()._curvature_matrix_mapper_diag
 
-        return self._curvature_matrix_mapper_diag
+        return self.__curvature_matrix_mapper_diag
+
+    @property
+    def mapper_operated_mapping_matrix_dict(self):
+        if self._mapper_operated_mapping_matrix_dict is None:
+            return super().mapper_operated_mapping_matrix_dict
+
+        return self._mapper_operated_mapping_matrix_dict
 
     @property
     def curvature_reg_matrix(self):
@@ -106,14 +119,12 @@ class MockInversion(AbstractInversion):
 
     @property
     def reconstruction(self):
-
         if self._reconstruction is None:
             return super().reconstruction
         return self._reconstruction
 
     @property
     def reconstruction_dict(self):
-
         if self._reconstruction_dict is None:
             return super().reconstruction_dict
         return self._reconstruction_dict
@@ -159,22 +170,19 @@ class MockInversion(AbstractInversion):
         return self._mapped_reconstructed_image_dict
 
     @property
-    def errors(self):
-
-        if self._errors is None:
-            return super().errors
-        return self._errors
+    def reconstruction_noise_map(self):
+        if self._reconstruction_noise_map is None:
+            return super().reconstruction_noise_map
+        return self._reconstruction_noise_map
 
     @property
-    def errors_dict(self):
-
-        if self._errors_dict is None:
-            return super().errors_dict
-        return self._errors_dict
+    def reconstruction_noise_map_dict(self):
+        if self._reconstruction_noise_map_dict is None:
+            return super().reconstruction_noise_map_dict
+        return self._reconstruction_noise_map_dict
 
     @property
     def regularization_term(self):
-
         if self._regularization_term is None:
             return super().regularization_term
 
@@ -182,7 +190,6 @@ class MockInversion(AbstractInversion):
 
     @property
     def log_det_curvature_reg_matrix_term(self):
-
         if self._log_det_curvature_reg_matrix_term is None:
             return super().log_det_curvature_reg_matrix_term
 
@@ -190,22 +197,7 @@ class MockInversion(AbstractInversion):
 
     @property
     def log_det_regularization_matrix_term(self):
-
         if self._log_det_regularization_matrix_term is None:
             return super().log_det_regularization_matrix_term
 
         return self._log_det_regularization_matrix_term
-
-    @property
-    def curvature_matrix_preload(self):
-        if self._curvature_matrix_preload is None:
-            return super().curvature_matrix_preload
-
-        return self._curvature_matrix_preload
-
-    @property
-    def curvature_matrix_counts(self):
-        if self._curvature_matrix_counts is None:
-            return super().curvature_matrix_counts
-
-        return self._curvature_matrix_counts
