@@ -1,10 +1,10 @@
 import numpy as np
 
+from autoarray.inversion.inversion.dataset_interface import DatasetInterface
 from autoarray.inversion.inversion.interferometer.mapping import (
     InversionInterferometerMapping,
 )
 from autoarray.inversion.inversion.settings import SettingsInversion
-from autoarray.preloads import Preloads
 
 
 class MockInversionInterferometer(InversionInterferometerMapping):
@@ -15,17 +15,20 @@ class MockInversionInterferometer(InversionInterferometerMapping):
         transformer=None,
         linear_obj_list=None,
         operated_mapping_matrix=None,
-        settings: SettingsInversion = SettingsInversion(),
-        preloads: Preloads = Preloads(),
+        settings: SettingsInversion = None,
     ):
-
-        super().__init__(
+        dataset = DatasetInterface(
             data=data,
             noise_map=noise_map,
             transformer=transformer,
+        )
+
+        settings = settings or SettingsInversion()
+
+        super().__init__(
+            dataset=dataset,
             linear_obj_list=linear_obj_list,
             settings=settings,
-            preloads=preloads,
         )
 
         self._operated_mapping_matrix = operated_mapping_matrix
