@@ -1,12 +1,19 @@
+from autoarray.mask.mask_2d import Mask2D
 from autoarray.inversion.pixelization.pixelization import Pixelization
 
 
 class MockPixelization(Pixelization):
     def __init__(
-        self, mesh=None, regularization=None, mapper=None, image_plane_mesh_grid=None
+        self,
+        mesh=None,
+        regularization=None,
+        image_mesh=None,
+        mapper=None,
+        image_plane_mesh_grid=None,
     ):
-
-        super().__init__(mesh=mesh, regularization=regularization)
+        super().__init__(
+            mesh=mesh, regularization=regularization, image_mesh=image_mesh
+        )
 
         self.mapper = mapper
         self.image_plane_mesh_grid = image_plane_mesh_grid
@@ -14,21 +21,22 @@ class MockPixelization(Pixelization):
     # noinspection PyUnusedLocal,PyShadowingNames
     def mapper_grids_from(
         self,
+        mask,
         source_plane_data_grid,
         source_plane_mesh_grid,
         image_plane_mesh_grid=None,
-        hyper_data=None,
+        adapt_data=None,
         settings=None,
-        preloads=None,
-        profiling_dict=None,
     ):
         return self.mapper
 
     def image_plane_mesh_grid_from(
-        self, image_plane_data_grid, hyper_data, settings=None
+        self,
+        mask: Mask2D,
+        adapt_data,
+        settings=None,
     ):
-
-        if hyper_data is not None and self.image_plane_mesh_grid is not None:
-            return hyper_data * self.image_plane_mesh_grid
+        if adapt_data is not None and self.image_plane_mesh_grid is not None:
+            return adapt_data * self.image_plane_mesh_grid
 
         return self.image_plane_mesh_grid
