@@ -2,19 +2,21 @@ import itertools
 import numpy as np
 from typing import Dict, List, Optional, Tuple
 
+from autoconf import conf
 from autoconf import cached_property
 
 from autoarray.inversion.linear_obj.linear_obj import LinearObj
 from autoarray.inversion.linear_obj.func_list import UniqueMappings
 from autoarray.inversion.linear_obj.neighbors import Neighbors
+from autoarray.inversion.pixelization.border_relocator import BorderRelocator
 from autoarray.inversion.pixelization.mappers.mapper_grids import MapperGrids
 from autoarray.inversion.regularization.abstract import AbstractRegularization
 from autoarray.structures.arrays.uniform_2d import Array2D
 from autoarray.structures.grids.uniform_2d import Grid2D
 from autoarray.structures.mesh.abstract_2d import Abstract2DMesh
 
-from autoarray.numba_util import profile_func
 from autoarray.inversion.pixelization.mappers import mapper_util
+from autoarray.inversion.pixelization.mappers import mapper_numba_util
 
 
 class AbstractMapper(LinearObj):
@@ -22,7 +24,7 @@ class AbstractMapper(LinearObj):
         self,
         mapper_grids: MapperGrids,
         regularization: Optional[AbstractRegularization],
-        profiling_dict: Optional[Dict] = None,
+        border_relocator: BorderRelocator,
     ):
         """
         To understand a `Mapper` one must be familiar `Mesh` objects and the `mesh` and `pixelization` packages, where
@@ -76,12 +78,14 @@ class AbstractMapper(LinearObj):
         regularization
             The regularization scheme which may be applied to this linear object in order to smooth its solution,
             which for a mapper smooths neighboring pixels on the mesh.
-        profiling_dict
-            A dictionary which contains timing of certain functions calls which is used for profiling.
+        border_relocator
+           The border relocator, which relocates coordinates outside the border of the source-plane data grid to its
+           edge.
         """
 
-        super().__init__(regularization=regularization, profiling_dict=profiling_dict)
+        super().__init__(regularization=regularization)
 
+        self.border_relocator = border_relocator
         self.mapper_grids = mapper_grids
 
     @property
@@ -104,13 +108,17 @@ class AbstractMapper(LinearObj):
     def image_plane_mesh_grid(self) -> Grid2D:
         return self.mapper_grids.image_plane_mesh_grid
 
+    @property
+    def over_sampler(self):
+        return self.mapper_grids.source_plane_data_grid.over_sampler
+
     @property
     def edge_pixel_list(self) -> List[int]:
         return self.source_plane_mesh_grid.edge_pixel_list
 
     @property
-    def hyper_data(self) -> np.ndarray:
-        return self.mapper_grids.hyper_data
+    def adapt_data(self) -> np.ndarray:
+        return self.mapper_grids.adapt_data
 
     @property
     def neighbors(self) -> Neighbors:
@@ -175,7 +183,7 @@ class AbstractMapper(LinearObj):
         The mappings between every sub-pixel data point on the sub-gridded data and each data point for a grid which
         does not use sub gridding (e.g. `sub_size=1`).
         """
-        return self.source_plane_data_grid.mask.derive_indexes.slim_for_sub_slim
+        return self.over_sampler.slim_for_sub_slim
 
     @property
     def sub_slim_indexes_for_pix_index(self) -> List[List]:
@@ -200,31 +208,7 @@ class AbstractMapper(LinearObj):
 
         return sub_slim_indexes_for_pix_index
 
-    @property
-    @profile_func
-    def sub_slim_indexes_for_pix_index_arr(
-        self,
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-        """
-        Returns the index mappings between each of the pixelization's pixels and the masked data's sub-pixels.
-
-        Given that even pixelization pixel maps to multiple data sub-pixels, index mappings are returned as a list of
-        lists where the first entries are the pixelization index and second entries store the data sub-pixel indexes.
-
-        For example, if `sub_slim_indexes_for_pix_index[2][4] = 10`, the pixelization pixel with index 2
-        (e.g. `mesh_grid[2,:]`) has a mapping to a data sub-pixel with index 10 (e.g. `grid_slim[10, :]).
-
-        This is effectively a reversal of the array `pix_indexes_for_sub_slim_index`.
-        """
-
-        return mapper_util.sub_slim_indexes_for_pix_index(
-            pix_indexes_for_sub_slim_index=self.pix_indexes_for_sub_slim_index,
-            pix_weights_for_sub_slim_index=self.pix_weights_for_sub_slim_index,
-            pix_pixels=self.pixels,
-        )
-
     @cached_property
-    @profile_func
     def unique_mappings(self) -> UniqueMappings:
         """
         Returns the unique mappings of every unmasked data pixel's (e.g. `grid_slim`) sub-pixels (e.g. `grid_sub_slim`)
@@ -237,16 +221,22 @@ class AbstractMapper(LinearObj):
         A full description of these mappings is given in the
         function `mapper_util.data_slim_to_pixelization_unique_from()`.
         """
+
         (
             data_to_pix_unique,
             data_weights,
             pix_lengths,
-        ) = mapper_util.data_slim_to_pixelization_unique_from(
-            data_pixels=self.source_plane_data_grid.shape_slim,
-            pix_indexes_for_sub_slim_index=self.pix_indexes_for_sub_slim_index,
-            pix_sizes_for_sub_slim_index=self.pix_sizes_for_sub_slim_index,
-            pix_weights_for_sub_slim_index=self.pix_weights_for_sub_slim_index,
-            sub_size=self.source_plane_data_grid.sub_size,
+        ) = mapper_numba_util.data_slim_to_pixelization_unique_from(
+            data_pixels=self.over_sampler.mask.pixels_in_mask,
+            pix_indexes_for_sub_slim_index=np.array(
+                self.pix_indexes_for_sub_slim_index
+            ),
+            pix_sizes_for_sub_slim_index=np.array(self.pix_sizes_for_sub_slim_index),
+            pix_weights_for_sub_slim_index=np.array(
+                self.pix_weights_for_sub_slim_index
+            ),
+            pix_pixels=self.params,
+            sub_size=np.array(self.over_sampler.sub_size).astype("int"),
         )
 
         return UniqueMappings(
@@ -256,7 +246,6 @@ class AbstractMapper(LinearObj):
         )
 
     @cached_property
-    @profile_func
     def mapping_matrix(self) -> np.ndarray:
         """
         The `mapping_matrix` of a linear object describes the mappings between the observed data's data-points / pixels
@@ -269,19 +258,20 @@ class AbstractMapper(LinearObj):
         It is described in the following paper as matrix `f` https://arxiv.org/pdf/astro-ph/0302587.pdf and in more
         detail in the function  `mapper_util.mapping_matrix_from()`.
         """
+
         return mapper_util.mapping_matrix_from(
             pix_indexes_for_sub_slim_index=self.pix_indexes_for_sub_slim_index,
             pix_size_for_sub_slim_index=self.pix_sizes_for_sub_slim_index,
             pix_weights_for_sub_slim_index=self.pix_weights_for_sub_slim_index,
             pixels=self.pixels,
-            total_mask_pixels=self.source_plane_data_grid.mask.pixels_in_mask,
+            total_mask_pixels=self.over_sampler.mask.pixels_in_mask,
             slim_index_for_sub_slim_index=self.slim_index_for_sub_slim_index,
-            sub_fraction=self.source_plane_data_grid.mask.sub_fraction,
+            sub_fraction=self.over_sampler.sub_fraction.array,
         )
 
     def pixel_signals_from(self, signal_scale: float) -> np.ndarray:
         """
-        Returns the (hyper) signal in each pixelization pixel, where this signal is an estimate of the expected signal
+        Returns the signal in each pixelization pixel, where this signal is an estimate of the expected signal
         each pixelization pixel contains given the data pixels it maps too.
 
         A full description of this is given in the function `mapper_util.adaptive_pixel_signals_from().
@@ -298,11 +288,11 @@ class AbstractMapper(LinearObj):
             pixel_weights=self.pix_weights_for_sub_slim_index,
             pix_indexes_for_sub_slim_index=self.pix_indexes_for_sub_slim_index,
             pix_size_for_sub_slim_index=self.pix_sizes_for_sub_slim_index,
-            slim_index_for_sub_slim_index=self.source_plane_data_grid.mask.derive_indexes.slim_for_sub_slim,
-            hyper_data=self.hyper_data,
+            slim_index_for_sub_slim_index=self.over_sampler.slim_for_sub_slim,
+            adapt_data=self.adapt_data.array,
         )
 
-    def pix_indexes_for_slim_indexes(self, pix_indexes: List) -> List[List]:
+    def slim_indexes_for_pix_indexes(self, pix_indexes: List) -> List[List]:
         """
         Returns the index mappings between every masked data-point (not subgridded) on the data and the mapper
         pixels / parameters that it maps too.
@@ -310,7 +300,7 @@ class AbstractMapper(LinearObj):
         The `slim_index` refers to the masked data pixels (without subgridding) and `pix_indexes` the pixelization
         pixel indexes, for example:
 
-        - `pix_indexes_for_slim_indexes[0] = [2, 3]`: The data's first (index 0) pixel maps to the
+        - `slim_indexes_for_pix_indexes[0] = [2, 3]`: The data's first (index 0) pixel maps to the
         pixelization's third (index 2) and fourth (index 3) pixels.
 
         Parameters
@@ -342,6 +332,25 @@ class AbstractMapper(LinearObj):
                 )
             return indexes
 
+    def data_weight_total_for_pix_from(self) -> np.ndarray:
+        """
+        Returns the total weight of every pixelization pixel, which is the sum of the weights of all data-points that
+        map to that pixel.
+        """
+
+        return mapper_util.data_weight_total_for_pix_from(
+            pix_indexes_for_sub_slim_index=np.array(
+                self.pix_indexes_for_sub_slim_index
+            ),
+            pix_weights_for_sub_slim_index=np.array(
+                self.pix_weights_for_sub_slim_index
+            ),
+            pixels=self.pixels,
+        )
+
+    def data_pixel_area_for_pix_from(self) -> np.ndarray:
+        pass
+
     def mapped_to_source_from(self, array: Array2D) -> np.ndarray:
         """
         Map a masked 2d image in the image domain to the source domain and sum up all mappings on the source-pixels.
@@ -349,7 +358,7 @@ class AbstractMapper(LinearObj):
         For example, suppose we have an image and a mapper. We can map every image-pixel to its corresponding mapper's
         source pixel and sum the values based on these mappings.
 
-        This will produce something similar to a `reconstruction`, albeit it bypasses the linear algebra / inversion.
+        This will produce something similar to a `reconstruction`, by passing the linear algebra / inversion.
 
         Parameters
         ----------
@@ -358,7 +367,46 @@ class AbstractMapper(LinearObj):
             source domain in order to compute their average values.
         """
         return mapper_util.mapped_to_source_via_mapping_matrix_from(
-            mapping_matrix=self.mapping_matrix, array_slim=array.binned.slim
+            mapping_matrix=np.array(self.mapping_matrix),
+            array_slim=array.slim,
+        )
+
+    def extent_from(
+        self,
+        values: np.ndarray = None,
+        zoom_to_brightest: bool = True,
+        zoom_percent: Optional[float] = None,
+    ) -> Tuple[float, float, float, float]:
+
+        from autoarray.geometry import geometry_util
+
+        if zoom_to_brightest and values is not None:
+            if zoom_percent is None:
+                zoom_percent = conf.instance["visualize"]["general"]["zoom"][
+                    "inversion_percent"
+                ]
+
+            fractional_value = np.max(values) * zoom_percent
+            fractional_bool = values > fractional_value
+            true_indices = np.argwhere(fractional_bool)
+            true_grid = self.source_plane_mesh_grid[true_indices]
+
+            try:
+                return geometry_util.extent_symmetric_from(
+                    extent=(
+                        np.min(true_grid[:, 0, 1]),
+                        np.max(true_grid[:, 0, 1]),
+                        np.min(true_grid[:, 0, 0]),
+                        np.max(true_grid[:, 0, 0]),
+                    )
+                )
+            except ValueError:
+                return geometry_util.extent_symmetric_from(
+                    extent=self.source_plane_mesh_grid.geometry.extent
+                )
+
+        return geometry_util.extent_symmetric_from(
+            extent=self.source_plane_mesh_grid.geometry.extent
         )
 
     def interpolated_array_from(
