@@ -7,3 +7,6 @@ from .adaptive_brightness import AdaptiveBrightness
 from .adaptive_brightness_split import AdaptiveBrightnessSplit
 from .brightness_zeroth import BrightnessZeroth
 from .adaptive_brightness_split_zeroth import AdaptiveBrightnessSplitZeroth
+from .gaussian_kernel import GaussianKernel
+from .exponential_kernel import ExponentialKernel
+from .matern_kernel import MaternKernel
