@@ -1,10 +1,14 @@
+from __future__ import annotations
+
 from abc import ABC
 import logging
 import numpy as np
+from typing import Dict
+
+from autoconf.fitsable import output_to_fits
 
 from autoarray.abstract_ndarray import AbstractNDArray
 
-from autoarray import exc
 from autoarray import type as ty
 
 logging.basicConfig()
@@ -12,18 +16,16 @@ logger = logging.getLogger(__name__)
 
 
 class Mask(AbstractNDArray, ABC):
-
     pixel_scales = None
 
     # noinspection PyUnusedLocal
-    def __new__(
-        cls,
+    def __init__(
+        self,
         mask: np.ndarray,
         origin: tuple,
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         *args,
-        **kwargs
+        **kwargs,
     ):
         """
         An abstract class for a mask that represents data structure that can be in 1D, 2D or other shapes.
@@ -32,7 +34,7 @@ class Mask(AbstractNDArray, ABC):
         that are (`False` or 0).
 
         The mask also defines the geometry of the data structure it is paired with, for example how its pixels convert
-        to physical units via the pixel_scales and origin parameters and a sub-grid which is used for
+        to physical units via the pixel_scales and origin parameters and a grid which is used for
         perform calculations via super-sampling.
 
         Parameters
@@ -49,20 +51,20 @@ class Mask(AbstractNDArray, ABC):
 
         # noinspection PyArgumentList
         mask = mask.astype("bool")
-        obj = mask.view(cls)
-        obj.sub_size = sub_size
-        obj.pixel_scales = pixel_scales
-        obj.origin = origin
-        return obj
+        super().__init__(mask)
+
+        self.pixel_scales = pixel_scales
+        self.origin = origin
+
+    @property
+    def mask(self):
+        return self._array
 
     def __array_finalize__(self, obj):
-
         if isinstance(obj, Mask):
-            self.sub_size = obj.sub_size
             self.pixel_scales = obj.pixel_scales
             self.origin = obj.origin
         else:
-            self.sub_size = 1
             self.pixel_scales = None
 
     @property
@@ -71,48 +73,64 @@ class Mask(AbstractNDArray, ABC):
         For a mask with dimensions two or above check that are pixel scales are the same, and if so return this
         single value as a float.
         """
-        for pixel_scale in self.pixel_scales:
-            if pixel_scale != self.pixel_scales[0]:
-                raise exc.MaskException(
-                    "Cannot return a pixel_scale for a grid where each dimension has a "
-                    "different pixel scale (e.g. pixel_scales[0] != pixel_scales[1])"
-                )
-
         return self.pixel_scales[0]
 
+    @property
+    def header_dict(self) -> Dict:
+        """
+        Returns the pixel scale of the mask as a header dictionary, which can be written to a .fits file.
+
+        If the array has different pixel scales in 2 dimensions, the header will contain both pixel scales as separate
+        y and x entries.
+
+        Returns
+        -------
+        A dictionary containing the pixel scale of the mask, which can be output to a .fits file.
+        """
+
     @property
     def dimensions(self) -> int:
         return len(self.shape)
 
-    @property
-    def sub_length(self) -> int:
+    def output_to_fits(self, file_path, overwrite=False):
         """
-        The total number of sub-pixels in a give pixel,
+        Write the Mask to a .fits file.
 
-        For example, a sub-size of 3x3 means every pixel has 9 sub-pixels.
-        """
-        return int(self.sub_size**self.dimensions)
+        Before outputting a 2D NumPy array mask, the array may be flipped upside-down using np.flipud depending on
+        the project config files. This is for Astronomy projects so that structures appear the same orientation
+        as `.fits` files loaded in DS9.
 
-    @property
-    def sub_fraction(self) -> float:
-        """
-        The fraction of the area of a pixel every sub-pixel contains.
+        Parameters
+        ----------
+        file_path
+            The full path of the file that is output, including the file name and `.fits` extension.
+        overwrite
+            If `True` and a file already exists with the input file_path the .fits file is overwritten. If `False`, an
+            error is raised.
 
-        For example, a sub-size of 3x3 mean every pixel contains 1/9 the area.
-        """
-        return 1.0 / self.sub_length
+        Returns
+        -------
+        None
 
-    def output_to_fits(self, file_path: str, overwrite: bool = False):
-        """
-        Overwrite with method to output the mask to a `.fits` file.
+        Examples
+        --------
+        mask = Mask2D(mask=np.full(shape=(5,5), fill_value=False))
+        mask.output_to_fits(file_path='/path/to/file/filename.fits', overwrite=True)
         """
+        output_to_fits(
+            values=self.astype("float"),
+            file_path=file_path,
+            overwrite=overwrite,
+            header_dict=self.header_dict,
+            ext_name="mask",
+        )
 
     @property
     def pixels_in_mask(self) -> int:
         """
         The total number of unmasked pixels (values are `False`) in the mask.
         """
-        return int(np.size(self) - np.sum(self))
+        return (np.size(self._array) - np.sum(self._array)).astype(int)
 
     @property
     def is_all_true(self) -> bool:
@@ -126,14 +144,7 @@ class Mask(AbstractNDArray, ABC):
         """
         Returns `False` if all pixels in a mask are `False`, else returns `True`.
         """
-        return self.pixels_in_mask == np.size(self)
-
-    @property
-    def sub_pixels_in_mask(self) -> int:
-        """
-        The total number of unmasked sub-pixels (values are `False`) in the mask.
-        """
-        return self.sub_size**self.dimensions * self.pixels_in_mask
+        return self.pixels_in_mask == np.size(self._array)
 
     @property
     def shape_slim(self) -> int:
@@ -141,21 +152,3 @@ class Mask(AbstractNDArray, ABC):
         The 1D shape of the mask, which is equivalent to the total number of unmasked pixels in the mask.
         """
         return self.pixels_in_mask
-
-    @property
-    def sub_shape_slim(self) -> int:
-        """
-        The 1D shape of the mask's sub-grid, which is equivalent to the total number of unmasked pixels in the mask.
-        """
-        return int(self.pixels_in_mask * self.sub_size**self.dimensions)
-
-    def mask_new_sub_size_from(self, mask, sub_size=1) -> "Mask":
-        """
-        Returns the mask on the same scaled coordinate system but with a sub-grid of an inputsub_size.
-        """
-        return self.__class__(
-            mask=mask,
-            sub_size=sub_size,
-            pixel_scales=self.pixel_scales,
-            origin=self.origin,
-        )
