@@ -1,12 +1,17 @@
 from __future__ import annotations
 import logging
+from enum import Enum
 import numpy as np
-from typing import TYPE_CHECKING, List, Tuple, Union
+from pathlib import Path
+from typing import TYPE_CHECKING, Dict, List, Tuple, Union
+
+from autoarray.structures.abstract_structure import Structure
 
 if TYPE_CHECKING:
     from autoarray.structures.arrays.uniform_2d import Array2D
 
 from autoconf import cached_property
+from autoconf.fitsable import ndarray_via_fits_from
 
 from autoarray.mask.abstract_mask import Mask
 
@@ -16,6 +21,7 @@ from autoarray.geometry.geometry_2d import Geometry2D
 from autoarray.mask.derive.mask_2d import DeriveMask2D
 from autoarray.mask.derive.grid_2d import DeriveGrid2D
 from autoarray.mask.derive.indexes_2d import DeriveIndexes2D
+from autoarray.mask.derive.zoom_2d import Zoom2D
 
 from autoarray.structures.arrays import array_2d_util
 from autoarray.geometry import geometry_util
@@ -26,14 +32,19 @@ logging.basicConfig()
 logger = logging.getLogger(__name__)
 
 
-class Mask2D(Mask):
+class Mask2DKeys(Enum):
+    PIXSCAY = "PIXSCAY"
+    PIXSCAX = "PIXSCAX"
+    ORIGINY = "ORIGINY"
+    ORIGINX = "ORIGINX"
 
+
+class Mask2D(Mask):
     # noinspection PyUnusedLocal
-    def __new__(
-        cls,
+    def __init__(
+        self,
         mask: Union[np.ndarray, List],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         invert: bool = False,
         *args,
@@ -50,17 +61,15 @@ class Mask2D(Mask):
         - Maps data structures between two data representations: `slim`` (all unmasked ``False`` values in
           a 1D ``ndarray``) and ``native`` (all unmasked values in a 2D or 3D ``ndarray``).
 
-        - Has a ``Geometry2D`` object (defined by its (y,x) ``pixel scales``, (y,x) ``origin`` and ``sub_size``)
+        - Has a ``Geometry2D`` object (defined by its (y,x) ``pixel scales`` and (y,x) ``origin``)
           which defines how coordinates are converted from pixel units to scaled units.
 
         - Associates Cartesian ``Grid2D`` objects of (y,x) coordinates with the data structure (e.g.
-          a (y,x) grid of all unmasked pixels) via the ``DeriveGrid2D`` object.
-
-        - This includes sub-grids, which perform calculations higher resolutions which are then binned up.
+          a (y,x) grid of all unmasked pixels).
 
         A detailed description of the 2D mask API is provided below.
 
-        **SLIM DATA REPRESENTATION (sub-size=1)**
+        __Slim__
 
         Below is a visual illustration of a ``Mask2D``, where a total of 10 pixels are unmasked (values are ``False``):
 
@@ -106,10 +115,10 @@ class Mask2D(Mask):
             mask[6] = the 7th unmasked pixel's value.
 
         A Cartesian grid of (y,x) coordinates, corresponding to all ``slim`` values (e.g. unmasked pixels) is given
-        by ``mask.derive_grid.masked.slim``.
+        by:
 
 
-        **NATIVE DATA REPRESENTATION (sub_size=1)**
+        __native__
 
         Masked data represented as an an ``ndarray`` of shape [total_y_values, total_x_values], where all masked
         entries have values of 0.0.
@@ -162,7 +171,7 @@ class Mask2D(Mask):
 
         The ``Mask2D`` has functionality which maps data between the ``slim`` and ``native`` data representations.
 
-        For the example mask above, the 1D ``ndarray`` given by ``mask.derive_indexes.slim_to_native`` is:
+        For the example mask above, the 1D ``ndarray`` given by ``derive_indexes.slim_to_native`` is:
 
         ::
 
@@ -177,92 +186,6 @@ class Mask2D(Mask):
             slim_to_native[8] = [5,5]
             slim_to_native[9] = [5,6]
 
-        **SUB GRIDDING**
-
-        If the ``Mask2D`` ``sub_size`` is > 1, its ``slim`` and ``native`` data representations have entries
-        corresponding to the values at the centre of every sub-pixel of each unmasked pixel.
-
-        The sub-array indexes are ordered such that pixels begin from the first (top-left) sub-pixel in the first
-        unmasked pixel. Indexes then go over the sub-pixels in each unmasked pixel, for every unmasked pixel.
-
-        Therefore, the shapes of the sub-array are as follows:
-
-        - ``slim`` representation: an ``ndarray`` of shape [total_unmasked_pixels*sub_size**2].
-        - ``native`` representation: an ``ndarray`` of shape [total_y_values*sub_size, total_x_values*sub_size].
-
-        Below is a visual illustration of a sub array. Indexing of each sub-pixel goes from the top-left corner. In
-        contrast to the array above, our illustration below restricts the mask to just 2 pixels, to keep the
-        illustration brief.
-
-        ::
-
-             x x x x x x x x x x
-             x x x x x x x x x x     This is an example ``Mask2D``, where:
-             x x x x x x x x x x
-             x x x x x x x x x x     x = `True` (Pixel is masked and excluded from lens)
-             x 0 0 x x x x x x x     O = `False` (Pixel is not masked and included in lens)
-             x x x x x x x x x x
-             x x x x x x x x x x
-             x x x x x x x x x x
-             x x x x x x x x x x
-             x x x x x x x x x x
-
-        If ``sub_size=2``, each unmasked pixel has 4 (2x2) sub-pixel values. For the example above, pixels 0 and 1
-        each have 4 values which map to ``slim`` representation as follows:
-
-        ::
-
-            Pixel 0 - (2x2):
-
-                   slim[0] = value of first sub-pixel in pixel 0.
-             0 1   slim[1] = value of first sub-pixel in pixel 1.
-             2 3   slim[2] = value of first sub-pixel in pixel 2.
-                   slim[3] = value of first sub-pixel in pixel 3.
-
-            Pixel 1 - (2x2):
-
-                   slim[4] = value of first sub-pixel in pixel 0.
-             4 5   slim[5] = value of first sub-pixel in pixel 1.
-             6 7   slim[6] = value of first sub-pixel in pixel 2.
-                   slim[7] = value of first sub-pixel in pixel 3.
-
-        For the ``native`` data representation we get the following mappings:
-
-        ::
-
-            Pixel 0 - (2x2):
-
-                   native[8, 2] = value of first sub-pixel in pixel 0.
-             0 1   native[8, 3] = value of first sub-pixel in pixel 1.
-             2 3   native[9, 2] = value of first sub-pixel in pixel 2.
-                   native[9, 3] = value of first sub-pixel in pixel 3.
-
-            Pixel 1 - (2x2):
-
-                   native[10, 4] = value of first sub-pixel in pixel 0.
-             4 5   native[10, 5] = value of first sub-pixel in pixel 1.
-             6 7   native[11, 4] = value of first sub-pixel in pixel 2.
-                   native[11, 5] = value of first sub-pixel in pixel 3.
-
-            Other entries (all masked sub-pixels are zero):
-
-                   native[0, 0] = 0.0 (it is masked, thus zero)
-                   native[15, 12] = 0.0 (it is masked, thus zero)
-
-        If we used a sub_size of 3, for pixel 0 we we would create a 3x3 sub-array:
-
-        ::
-
-                     slim[0] = value of first sub-pixel in pixel 0.
-                     slim[1] = value of first sub-pixel in pixel 1.
-                     slim[2] = value of first sub-pixel in pixel 2.
-             0 1 2   slim[3] = value of first sub-pixel in pixel 3.
-             3 4 5   slim[4] = value of first sub-pixel in pixel 4.
-             6 7 8   slim[5] = value of first sub-pixel in pixel 5.
-                     slim[6] = value of first sub-pixel in pixel 6.
-                     slim[7] = value of first sub-pixel in pixel 7.
-                     slim[8] = value of first sub-pixel in pixel 8.
-
         Parameters
         ----------
         mask
@@ -279,30 +202,35 @@ class Mask2D(Mask):
             mask = np.asarray(mask).astype("bool")
 
         if invert:
-            mask = np.invert(mask)
+            mask = ~mask
 
         pixel_scales = geometry_util.convert_pixel_scales_2d(pixel_scales=pixel_scales)
 
         if len(mask.shape) != 2:
             raise exc.MaskException("The input mask is not a two dimensional array")
 
-        obj = Mask.__new__(
-            cls=cls,
+        super().__init__(
             mask=mask,
-            pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
+            pixel_scales=pixel_scales,
         )
 
-        return obj
+    @cached_property
+    def native_for_slim(self):
+        return self.derive_indexes.native_for_slim
+
+    __no_flatten__ = ("derive_indexes",)
 
     def __array_finalize__(self, obj):
-
         super().__array_finalize__(obj=obj)
 
         if not isinstance(obj, Mask2D):
             self.origin = (0.0, 0.0)
 
+    @property
+    def native(self) -> Structure:
+        return self
+
     @property
     def geometry(self) -> Geometry2D:
         """
@@ -327,12 +255,15 @@ class Mask2D(Mask):
     def derive_grid(self) -> DeriveGrid2D:
         return DeriveGrid2D(mask=self)
 
+    @property
+    def zoom(self) -> Zoom2D:
+        return Zoom2D(mask=self)
+
     @classmethod
     def all_false(
         cls,
         shape_native: Tuple[int, int],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         invert: bool = False,
     ) -> "Mask2D":
@@ -346,8 +277,6 @@ class Mask2D(Mask):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
         invert
@@ -357,7 +286,6 @@ class Mask2D(Mask):
         return cls(
             mask=np.full(shape=shape_native, fill_value=False),
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             invert=invert,
         )
@@ -368,7 +296,6 @@ class Mask2D(Mask):
         shape_native: Tuple[int, int],
         radius: float,
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         centre: Tuple[float, float] = (0.0, 0.0),
         invert: bool = False,
@@ -387,8 +314,6 @@ class Mask2D(Mask):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
         centre
@@ -410,7 +335,6 @@ class Mask2D(Mask):
         return cls(
             mask=mask,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             invert=invert,
         )
@@ -422,7 +346,6 @@ class Mask2D(Mask):
         inner_radius: float,
         outer_radius: float,
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         centre: Tuple[float, float] = (0.0, 0.0),
         invert: bool = False,
@@ -444,8 +367,6 @@ class Mask2D(Mask):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
         centre
@@ -468,70 +389,6 @@ class Mask2D(Mask):
         return cls(
             mask=mask,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
-            origin=origin,
-            invert=invert,
-        )
-
-    @classmethod
-    def circular_anti_annular(
-        cls,
-        shape_native: Tuple[int, int],
-        inner_radius: float,
-        outer_radius: float,
-        outer_radius_2: float,
-        pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
-        origin: Tuple[float, float] = (0.0, 0.0),
-        centre: Tuple[float, float] = (0.0, 0.0),
-        invert: bool = False,
-    ) -> "Mask2D":
-        """
-        Returns a Mask2D (see *Mask2D.__new__*) where all `False` entries are within an inner circle and second
-        outer circle, forming an inverse annulus.
-
-        The `inner_radius`, `outer_radius`, `outer_radius_2` and `centre` are all input in scaled units.
-
-        Parameters
-        ----------
-        shape_native
-            The (y,x) shape of the mask in units of pixels.
-        inner_radius
-            The inner radius in scaled units of the annulus within which pixels are `False` and unmasked.
-        outer_radius
-            The first outer radius in scaled units of the annulus within which pixels are `True` and masked.
-        outer_radius_2
-            The second outer radius in scaled units of the annulus within which pixels are `False` and unmasked and
-            outside of which all entries are `True` and masked.
-        pixel_scales
-            The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
-            it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
-        origin
-            The (y,x) scaled units origin of the mask's coordinate system.
-        centre
-            The (y,x) scaled units centre of the anti-annulus used to mask pixels.
-        invert
-            If `True`, the `bool`'s of the input `mask` are inverted, for example `False`'s become `True`
-            and visa versa.
-        """
-
-        pixel_scales = geometry_util.convert_pixel_scales_2d(pixel_scales=pixel_scales)
-
-        mask = mask_2d_util.mask_2d_circular_anti_annular_from(
-            shape_native=shape_native,
-            pixel_scales=pixel_scales,
-            inner_radius=inner_radius,
-            outer_radius=outer_radius,
-            outer_radius_2_scaled=outer_radius_2,
-            centre=centre,
-        )
-
-        return cls(
-            mask=mask,
-            pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             invert=invert,
         )
@@ -544,7 +401,6 @@ class Mask2D(Mask):
         axis_ratio: float,
         angle: float,
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         centre: Tuple[float, float] = (0.0, 0.0),
         invert: bool = False,
@@ -568,8 +424,6 @@ class Mask2D(Mask):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
         centre
@@ -592,7 +446,6 @@ class Mask2D(Mask):
         return cls(
             mask=mask,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             invert=invert,
         )
@@ -608,7 +461,6 @@ class Mask2D(Mask):
         outer_axis_ratio: float,
         outer_phi: float,
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         centre: Tuple[float, float] = (0.0, 0.0),
         invert: bool = False,
@@ -639,8 +491,6 @@ class Mask2D(Mask):
         outer_phi
             The rotation angle of the outer ellipse within which pixels are unmasked, (counter-clockwise from the
             positive x-axis).
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
         centre
@@ -666,7 +516,6 @@ class Mask2D(Mask):
         return cls(
             mask=mask,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             invert=invert,
         )
@@ -677,7 +526,6 @@ class Mask2D(Mask):
         shape_native: Tuple[int, int],
         pixel_coordinates: [[int, int]],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         buffer: int = 0,
         invert: bool = False,
@@ -697,8 +545,6 @@ class Mask2D(Mask):
             The input lists of 2D pixel coordinates where `False` entries are created.
         pixel_scales
             The scaled units to pixel units conversion factor of each pixel.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
         buffer
@@ -718,7 +564,6 @@ class Mask2D(Mask):
         return cls(
             mask=mask,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             invert=invert,
         )
@@ -726,12 +571,12 @@ class Mask2D(Mask):
     @classmethod
     def from_fits(
         cls,
-        file_path: str,
+        file_path: Union[Path, str],
         pixel_scales: ty.PixelScales,
         hdu: int = 0,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         resized_mask_shape: Tuple[int, int] = None,
+        invert: bool = False,
     ) -> "Mask2D":
         """
         Loads the image from a .fits file.
@@ -744,24 +589,24 @@ class Mask2D(Mask):
             The HDU number in the fits file containing the image image.
         pixel_scales or (float, float)
             The scaled units to pixel units conversion factor of each pixel.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
         """
         pixel_scales = geometry_util.convert_pixel_scales_2d(pixel_scales=pixel_scales)
 
+        mask = ndarray_via_fits_from(file_path=file_path, hdu=hdu)
+
+        if invert:
+            mask = np.invert(mask.astype("bool"))
+
         mask = Mask2D(
-            mask=array_2d_util.numpy_array_2d_via_fits_from(
-                file_path=file_path, hdu=hdu
-            ),
+            mask=mask,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
         )
 
         if resized_mask_shape is not None:
-            mask = mask.derive_mask.resized_from(new_shape=resized_mask_shape)
+            mask = mask.resized_from(new_shape=resized_mask_shape)
 
         return mask
 
@@ -769,13 +614,6 @@ class Mask2D(Mask):
     def shape_native(self) -> Tuple[int, ...]:
         return self.shape
 
-    @property
-    def sub_shape_native(self) -> Tuple[int, int]:
-        try:
-            return (self.shape[0] * self.sub_size, self.shape[1] * self.sub_size)
-        except AttributeError:
-            print("bleh")
-
     def trimmed_array_from(self, padded_array, image_shape) -> Array2D:
         """
         Map a padded 1D array of values to its original 2D array, trimming all edge values.
@@ -790,14 +628,13 @@ class Mask2D(Mask):
 
         pad_size_0 = self.shape[0] - image_shape[0]
         pad_size_1 = self.shape[1] - image_shape[1]
-        trimmed_array = padded_array.binned.native[
+        trimmed_array = padded_array.native[
             pad_size_0 // 2 : self.shape[0] - pad_size_0 // 2,
             pad_size_1 // 2 : self.shape[1] - pad_size_1 // 2,
         ]
         return Array2D.no_mask(
             values=trimmed_array,
             pixel_scales=self.pixel_scales,
-            sub_size=1,
             origin=self.origin,
         )
 
@@ -816,47 +653,44 @@ class Mask2D(Mask):
             The 1D unmasked image which is blurred.
         """
 
-        blurred_image = psf.convolved_array_from(array=padded_array)
+        blurred_image = psf.convolved_image_from(
+            image=padded_array, blurring_image=None
+        )
 
         return self.trimmed_array_from(
             padded_array=blurred_image, image_shape=image_shape
         )
 
-    def output_to_fits(self, file_path, overwrite=False):
+    @property
+    def header_dict(self) -> Dict:
         """
-        Write the 2D Mask to a .fits file.
+        Returns the pixel scales of the mask as a header dictionary, which can be written to a .fits file.
 
-        Before outputting a NumPy array, the array may be flipped upside-down using np.flipud depending on the project
-        config files. This is for Astronomy projects so that structures appear the same orientation as `.fits` files
-        loaded in DS9.
-
-        Parameters
-        ----------
-        file_path
-            The full path of the file that is output, including the file name and `.fits` extension.
-        overwrite
-            If `True` and a file already exists with the input file_path the .fits file is overwritten. If `False`, an
-            error is raised.
+        A 2D mask has different pixel scale variables for each dimension, the header therefore contain both pixel
+        scales as separate y and x entries.
 
         Returns
         -------
-        None
-
-        Examples
-        --------
-        mask = Mask2D(mask=np.full(shape=(5,5), fill_value=False))
-        mask.output_to_fits(file_path='/path/to/file/filename.fits', overwrite=True)
+        A dictionary containing the pixel scale of the mask, which can be output to a .fits file.
         """
-        array_2d_util.numpy_array_2d_to_fits(
-            array_2d=self.astype("float"), file_path=file_path, overwrite=overwrite
-        )
+
+        return {
+            Mask2DKeys.PIXSCAY: self.pixel_scales[0],
+            Mask2DKeys.PIXSCAX: self.pixel_scales[1],
+            Mask2DKeys.ORIGINY: self.origin[0],
+            Mask2DKeys.ORIGINX: self.origin[1],
+        }
 
     @property
     def mask_centre(self) -> Tuple[float, float]:
-        return grid_2d_util.grid_2d_centre_from(
-            grid_2d_slim=self.derive_grid.unmasked_sub_1
+        grid = grid_2d_util.grid_2d_slim_via_mask_from(
+            mask_2d=self,
+            pixel_scales=self.pixel_scales,
+            origin=self.origin,
         )
 
+        return grid_2d_util.grid_2d_centre_from(grid_2d_slim=grid)
+
     @property
     def shape_native_masked_pixels(self) -> Tuple[int, int]:
         """
@@ -867,93 +701,187 @@ class Mask2D(Mask):
         and 12 False entries going horizontally in the central regions of the mask, then shape_masked_pixels=(15,12).
         """
 
-        where = np.array(np.where(np.invert(self.astype("bool"))))
+        where = np.where(np.invert(self.astype("bool")))
         y0, x0 = np.amin(where, axis=1)
         y1, x1 = np.amax(where, axis=1)
 
         return (y1 - y0 + 1, x1 - x0 + 1)
 
-    @property
-    def zoom_centre(self) -> Tuple[float, float]:
+    def rescaled_from(self, rescale_factor) -> Mask2D:
+        """
+        Returns the ``Mask2D`` rescaled to a bigger or small shape via input ``rescale_factor``.
 
-        extraction_grid_1d = self.geometry.grid_pixels_2d_from(
-            grid_scaled_2d=self.derive_grid.unmasked_sub_1.slim
-        )
-        y_pixels_max = np.max(extraction_grid_1d[:, 0])
-        y_pixels_min = np.min(extraction_grid_1d[:, 0])
-        x_pixels_max = np.max(extraction_grid_1d[:, 1])
-        x_pixels_min = np.min(extraction_grid_1d[:, 1])
+        For example, for a ``rescale_factor=2.0`` the following mask:
 
-        return (
-            ((y_pixels_max + y_pixels_min - 1.0) / 2.0),
-            ((x_pixels_max + x_pixels_min - 1.0) / 2.0),
+        ::
+           [[ True,  True],
+            [False, False]]
+
+        Will double in size and become:
+
+        ::
+            [[True,   True,  True,  True],
+             [True,   True,  True,  True],
+             [False, False, False, False],
+             [False, False, False, False]]
+
+        Parameters
+        ----------
+        rescale_factor
+            The factor by which the ``Mask2D`` is rescaled (less than 1.0 produces a smaller mask, greater than 1.0
+            produces a bigger mask).
+
+        Examples
+        --------
+
+        .. code-block:: python
+
+            import autoarray as aa
+
+            mask_2d = aa.Mask2D(
+                mask=[
+                     [ True,  True],
+                     [False, False]
+                ],
+                pixel_scales=1.0,
+            )
+
+            print(mask_2d.rescaled_from(rescale_factor=2.0)
+        """
+        from autoarray.mask.mask_2d import Mask2D
+
+        rescaled_mask = mask_2d_util.rescaled_mask_2d_from(
+            mask_2d=self.array,
+            rescale_factor=rescale_factor,
         )
 
-    @property
-    def zoom_offset_pixels(self) -> Tuple[float, float]:
-
-        if self.pixel_scales is None:
-            return self.geometry.central_pixel_coordinates
-
-        return (
-            self.zoom_centre[0] - self.geometry.central_pixel_coordinates[0],
-            self.zoom_centre[1] - self.geometry.central_pixel_coordinates[1],
-        )
-
-    @property
-    def zoom_offset_scaled(self) -> Tuple[float, float]:
-
-        return (
-            -self.pixel_scales[0] * self.zoom_offset_pixels[0],
-            self.pixel_scales[1] * self.zoom_offset_pixels[1],
-        )
-
-    @property
-    def zoom_region(self) -> List[int]:
-        """
-        The zoomed rectangular region corresponding to the square encompassing all unmasked values. This zoomed
-        extraction region is a squuare, even if the mask is rectangular.
-
-        This is used to zoom in on the region of an image that is used in an analysis for visualization.
-        """
-
-        where = np.array(np.where(np.invert(self.astype("bool"))))
-        y0, x0 = np.amin(where, axis=1)
-        y1, x1 = np.amax(where, axis=1)
-
-        # Have to convert mask to bool for invert function to work.
-
-        ylength = y1 - y0
-        xlength = x1 - x0
-
-        if ylength > xlength:
-            length_difference = ylength - xlength
-            x1 += int(length_difference / 2.0)
-            x0 -= int(length_difference / 2.0)
-        elif xlength > ylength:
-            length_difference = xlength - ylength
-            y1 += int(length_difference / 2.0)
-            y0 -= int(length_difference / 2.0)
-
-        return [y0, y1 + 1, x0, x1 + 1]
-
-    @property
-    def zoom_shape_native(self) -> Tuple[int, int]:
-        region = self.zoom_region
-        return (region[1] - region[0], region[3] - region[2])
-
-    @property
-    def zoom_mask_unmasked(self) -> "Mask2D":
-        """
-        The scaled-grid of (y,x) coordinates of every pixel.
-
-        This is defined from the top-left corner, such that the first pixel at location [0, 0] will have a negative x
-        value y value in scaled units.
-        """
-
-        return Mask2D.all_false(
-            shape_native=self.zoom_shape_native,
+        return Mask2D(
+            mask=rescaled_mask,
             pixel_scales=self.pixel_scales,
-            sub_size=self.sub_size,
-            origin=self.zoom_offset_scaled,
+            origin=self.origin,
         )
+
+    def resized_from(self, new_shape, pad_value: int = 0.0) -> Mask2D:
+        """
+        Returns the ``Mask2D`` resized to a small or bigger ``ndarraay``, but with the same distribution of
+         ``False`` and ``True`` entries.
+
+        Resizing which increases the ``Mask2D`` shape pads it with values on its edge.
+
+        Resizing which reduces the ``Mask2D`` shape removes entries on its edge.
+
+        For example, for a ``new_shape=(4,4)`` the following mask:
+
+        ::
+           [[ True,  True],
+            [False, False]]
+
+        Will be padded with zeros (``False`` values) and become:
+
+        ::
+          [[True,  True,  True, True]
+           [True,  True,  True, True],
+           [True, False, False, True],
+           [True,  True,  True, True]]
+
+        Parameters
+        ----------
+        new_shape
+            The new two-dimensional shape of the resized ``Mask2D``.
+        pad_value
+            The value new values are padded using if the resized ``Mask2D`` is bigger.
+
+        Examples
+        --------
+
+        .. code-block:: python
+
+            import autoarray as aa
+
+            mask_2d = aa.Mask2D(
+                mask=[
+                     [ True,  True],
+                     [False, False]
+                ],
+                pixel_scales=1.0,
+            )
+
+            print(mask_2d.resized_from(new_shape=(4,4))
+        """
+
+        resized_mask = array_2d_util.resized_array_2d_from(
+            array_2d=self.array,
+            resized_shape=new_shape,
+            pad_value=pad_value,
+        ).astype("bool")
+
+        return Mask2D(
+            mask=resized_mask,
+            pixel_scales=self.pixel_scales,
+            origin=self.origin,
+        )
+
+    @property
+    def is_circular(self) -> bool:
+        """
+        Returns whether the mask is circular or not.
+
+        This is performed by taking the central row and column of the mask (based on the mask centre) and counting
+        the number of unmasked pixels. If the number of unmasked pixels is the same, the mask is circular.
+
+        This function does not support rectangular masks and an exception will be raised if the pixel scales in each
+        direction are different.
+        """
+
+        if self.pixel_scales[0] != self.pixel_scales[1]:
+            raise exc.MaskException(
+                """
+                The is_circular function cannot be called for a mask with different pixel scales in each dimension
+                (e.g. it does not support rectangular masks.
+                """
+            )
+
+        pixel_coordinates_2d = self.geometry.pixel_coordinates_2d_from(
+            scaled_coordinates_2d=self.mask_centre
+        )
+
+        central_row_pixels = sum(np.invert(self[pixel_coordinates_2d[0], :]))
+        central_column_pixels = sum(np.invert(self[:, pixel_coordinates_2d[1]]))
+
+        return central_row_pixels == central_column_pixels
+
+    @cached_property
+    def circular_radius(self) -> float:
+        """
+        Returns the radius in scaled units of a circular mask.
+
+        This is performed by taking the central row of the mask (based on the mask centre) and counting the number of
+        unmasked pixels. The radius is then half the number of unmasked pixels times the pixel scale.
+
+        The mask is first checked that it is circular using the `is_circular` property, with an exception raised if
+        it is not.
+
+        This function does not support rectangular masks and an exception will be raised if the pixel scales in each
+        direction are different.
+
+        Returns
+        -------
+        The circular radius of the mask in scaled units.
+        """
+
+        if not self.is_circular:
+            raise exc.MaskException(
+                """
+                A circular radius can only be computed for a circular mask.
+                
+                The `is_circular` property of this mask has returned False, indicating the mask is not circular.
+                """
+            )
+
+        pixel_coordinates_2d = self.geometry.pixel_coordinates_2d_from(
+            scaled_coordinates_2d=self.mask_centre
+        )
+
+        central_row_pixels = sum(np.invert(self[pixel_coordinates_2d[0], :]))
+
+        return central_row_pixels * self.pixel_scales[0] / 2.0
