@@ -1,5 +1,6 @@
 from autoarray.dataset.mock.mock_dataset import MockDataset
 from autoarray.inversion.mock.mock_mesh import MockMesh
+from autoarray.inversion.mock.mock_image_mesh import MockImageMesh
 from autoarray.inversion.mock.mock_regularization import MockRegularization
 from autoarray.inversion.mock.mock_pixelization import MockPixelization
 from autoarray.inversion.mock.mock_mapper import MockMapper
@@ -14,10 +15,8 @@ from autoarray.inversion.mock.mock_inversion_interferometer import (
 from autoarray.fit.mock.mock_fit_imaging import MockFitImaging
 from autoarray.fit.mock.mock_fit_interferometer import MockFitInterferometer
 from autoarray.mask.mock.mock_mask import MockMask
-from autoarray.operators.mock.mock_convolver import MockConvolver
+from autoarray.operators.mock.mock_psf import MockPSF
 from autoarray.structures.mock.mock_grid import MockGrid2DMesh
 from autoarray.structures.mock.mock_grid import MockMeshGrid
-from autoarray.structures.mock.mock_structure_decorators import MockGridRadialMinimum
-from autoarray.structures.mock.mock_structure_decorators import MockGrid1DLikeObj
-from autoarray.structures.mock.mock_structure_decorators import MockGrid2DLikeObj
-from autoarray.structures.mock.mock_structure_decorators import MockGridLikeIteratorObj
+from autoarray.structures.mock.mock_decorators import MockGrid1DLikeObj
+from autoarray.structures.mock.mock_decorators import MockGrid2DLikeObj
