@@ -1,18 +1,15 @@
+from autoconf import conf
+
 from autoarray.plot.wrap.base.abstract import set_backend
 
 set_backend()
 
-import matplotlib.pyplot as plt
 from typing import Optional, Tuple
 
 from autoarray.plot.visuals.one_d import Visuals1D
 from autoarray.plot.visuals.two_d import Visuals2D
-from autoarray.plot.include.one_d import Include1D
-from autoarray.plot.include.two_d import Include2D
 from autoarray.plot.mat_plot.one_d import MatPlot1D
 from autoarray.plot.mat_plot.two_d import MatPlot2D
-from autoarray.plot.get_visuals.one_d import GetVisuals1D
-from autoarray.plot.get_visuals.two_d import GetVisuals2D
 
 
 class AbstractPlotter:
@@ -20,24 +17,18 @@ class AbstractPlotter:
         self,
         mat_plot_1d: MatPlot1D = None,
         visuals_1d: Visuals1D = None,
-        include_1d: Include1D = None,
         mat_plot_2d: MatPlot2D = None,
         visuals_2d: Visuals2D = None,
-        include_2d: Include2D = None,
     ):
+        self.visuals_1d = visuals_1d or Visuals1D()
+        self.mat_plot_1d = mat_plot_1d or MatPlot1D()
 
-        self.visuals_1d = visuals_1d
-        self.include_1d = include_1d
-        self.mat_plot_1d = mat_plot_1d
-
-        self.visuals_2d = visuals_2d
-        self.include_2d = include_2d
-        self.mat_plot_2d = mat_plot_2d
+        self.visuals_2d = visuals_2d or Visuals2D()
+        self.mat_plot_2d = mat_plot_2d or MatPlot2D()
 
         self.subplot_figsize = None
 
     def set_title(self, label):
-
         if self.mat_plot_1d is not None:
             self.mat_plot_1d.title.manual_label = label
 
@@ -45,7 +36,6 @@ class AbstractPlotter:
             self.mat_plot_2d.title.manual_label = label
 
     def set_filename(self, filename):
-
         if self.mat_plot_1d is not None:
             self.mat_plot_1d.output.filename = filename
 
@@ -53,17 +43,20 @@ class AbstractPlotter:
             self.mat_plot_2d.output.filename = filename
 
     def set_format(self, format):
-
         if self.mat_plot_1d is not None:
             self.mat_plot_1d.output._format = format
 
         if self.mat_plot_2d is not None:
             self.mat_plot_2d.output._format = format
 
-    def set_mat_plot_1d_for_multi_plot(self, is_for_multi_plot, color: str):
-
+    def set_mat_plot_1d_for_multi_plot(
+        self, is_for_multi_plot, color: str, xticks=None, yticks=None
+    ):
         self.mat_plot_1d.set_for_multi_plot(
-            is_for_multi_plot=is_for_multi_plot, color=color
+            is_for_multi_plot=is_for_multi_plot,
+            color=color,
+            xticks=xticks,
+            yticks=yticks,
         )
 
     def set_mat_plots_for_subplot(
@@ -82,7 +75,6 @@ class AbstractPlotter:
 
     @property
     def is_for_subplot(self):
-
         if self.mat_plot_1d is not None:
             if self.mat_plot_1d.is_for_subplot:
                 return True
@@ -98,8 +90,10 @@ class AbstractPlotter:
         number_subplots: int,
         subplot_shape: Optional[Tuple[int, int]] = None,
         subplot_figsize: Optional[Tuple[int, int]] = None,
+        subplot_title: Optional[str] = None,
     ):
-        """Setup a figure for plotting an image.
+        """
+        Setup a figure for plotting an image.
 
         Parameters
         ----------
@@ -109,6 +103,7 @@ class AbstractPlotter:
             If the figure is a subplot, the setup_figure function is omitted to ensure that each subplot does not create a \
             new figure and so that it can be output using the *output.output_figure(structure=None)* function.
         """
+        import matplotlib.pyplot as plt
 
         self.set_mat_plots_for_subplot(
             is_for_subplot=True,
@@ -120,9 +115,9 @@ class AbstractPlotter:
 
         figsize = self.get_subplot_figsize(number_subplots=number_subplots)
         plt.figure(figsize=figsize)
+        plt.suptitle(subplot_title, fontsize=40, y=0.93)
 
     def close_subplot_figure(self):
-
         try:
             self.mat_plot_2d.figure.close()
         except AttributeError:
@@ -131,7 +126,8 @@ class AbstractPlotter:
         self.subplot_figsize = None
 
     def get_subplot_figsize(self, number_subplots):
-        """Get the size of a sub plotter in (total_y_pixels, total_x_pixels), based on the number of subplots that are going to be plotted.
+        """
+        Get the size of a sub plotter in (total_y_pixels, total_x_pixels), based on the number of subplots that are going to be plotted.
 
         Parameters
         ----------
@@ -150,25 +146,28 @@ class AbstractPlotter:
             if self.mat_plot_2d.figure.config_dict["figsize"] is not None:
                 return self.mat_plot_2d.figure.config_dict["figsize"]
 
-        if number_subplots <= 2:
-            return (18, 8)
-        elif number_subplots <= 4:
-            return (13, 10)
-        elif number_subplots <= 6:
-            return (18, 12)
-        elif number_subplots <= 9:
-            return (25, 20)
-        elif number_subplots <= 12:
-            return (25, 20)
-        elif number_subplots <= 16:
-            return (25, 20)
-        elif number_subplots <= 20:
-            return (25, 20)
-        else:
-            return (25, 20)
+        try:
+            subplot_shape = self.mat_plot_1d.get_subplot_shape(
+                number_subplots=number_subplots
+            )
+        except AttributeError:
+            subplot_shape = self.mat_plot_2d.get_subplot_shape(
+                number_subplots=number_subplots
+            )
+
+        subplot_shape_to_figsize_factor = conf.instance["visualize"]["general"][
+            "subplot_shape_to_figsize_factor"
+        ]
+        subplot_shape_to_figsize_factor = tuple(
+            map(int, subplot_shape_to_figsize_factor[1:-1].split(","))
+        )
+
+        return (
+            subplot_shape[1] * subplot_shape_to_figsize_factor[1],
+            subplot_shape[0] * subplot_shape_to_figsize_factor[0],
+        )
 
     def _subplot_custom_plot(self, **kwargs):
-
         figures_dict = dict(
             (key, value) for key, value in kwargs.items() if value is True
         )
@@ -176,13 +175,22 @@ class AbstractPlotter:
         self.open_subplot_figure(number_subplots=len(figures_dict))
 
         for index, (key, value) in enumerate(figures_dict.items()):
-
             if value:
                 try:
                     self.figures_2d(**{key: True})
                 except AttributeError:
                     self.figures_1d(**{key: True})
 
+            try:
+                self.mat_plot_2d.subplot_index = max(
+                    self.mat_plot_1d.subplot_index, self.mat_plot_2d.subplot_index
+                )
+                self.mat_plot_1d.subplot_index = max(
+                    self.mat_plot_1d.subplot_index, self.mat_plot_2d.subplot_index
+                )
+            except AttributeError:
+                pass
+
         try:
             self.mat_plot_2d.output.subplot_to_figure(
                 auto_filename=kwargs["auto_labels"].filename
@@ -195,23 +203,11 @@ class AbstractPlotter:
         self.close_subplot_figure()
 
     def subplot_of_plotters_figure(self, plotter_list, name):
-
         self.open_subplot_figure(number_subplots=len(plotter_list))
 
         for i, plotter in enumerate(plotter_list):
-
             plotter.figures_2d(**{name: True})
 
         self.mat_plot_2d.output.subplot_to_figure(auto_filename=f"subplot_{name}")
 
         self.close_subplot_figure()
-
-
-class Plotter(AbstractPlotter):
-    @property
-    def get_1d(self):
-        return GetVisuals1D(visuals=self.visuals_1d, include=self.include_1d)
-
-    @property
-    def get_2d(self):
-        return GetVisuals2D(visuals=self.visuals_2d, include=self.include_2d)
