@@ -5,14 +5,16 @@ from typing import Optional, List, Union
 from autoconf import conf
 
 from autoarray.inversion.pixelization.mappers.rectangular import (
-    MapperRectangularNoInterp,
+    MapperRectangular,
 )
 from autoarray.inversion.pixelization.mappers.delaunay import MapperDelaunay
-from autoarray.inversion.pixelization.mappers.voronoi import MapperVoronoiNoInterp
+from autoarray.inversion.pixelization.mappers.voronoi import MapperVoronoi
+from autoarray.mask.derive.zoom_2d import Zoom2D
 from autoarray.plot.mat_plot.abstract import AbstractMatPlot
 from autoarray.plot.auto_labels import AutoLabels
 from autoarray.plot.visuals.two_d import Visuals2D
 from autoarray.structures.arrays.uniform_2d import Array2D
+from autoarray.structures.arrays.rgb import Array2DRGB
 
 from autoarray.structures.arrays import array_2d_util
 
@@ -41,22 +43,29 @@ class MatPlot2D(AbstractMatPlot):
         legend: Optional[wb.Legend] = None,
         output: Optional[wb.Output] = None,
         array_overlay: Optional[w2d.ArrayOverlay] = None,
+        fill: Optional[w2d.Fill] = None,
+        contour: Optional[w2d.Contour] = None,
         grid_scatter: Optional[w2d.GridScatter] = None,
         grid_plot: Optional[w2d.GridPlot] = None,
         grid_errorbar: Optional[w2d.GridErrorbar] = None,
         vector_yx_quiver: Optional[w2d.VectorYXQuiver] = None,
         patch_overlay: Optional[w2d.PatchOverlay] = None,
         interpolated_reconstruction: Optional[w2d.InterpolatedReconstruction] = None,
+        delaunay_drawer: Optional[w2d.DelaunayDrawer] = None,
         voronoi_drawer: Optional[w2d.VoronoiDrawer] = None,
         origin_scatter: Optional[w2d.OriginScatter] = None,
         mask_scatter: Optional[w2d.MaskScatter] = None,
         border_scatter: Optional[w2d.BorderScatter] = None,
         positions_scatter: Optional[w2d.PositionsScatter] = None,
         index_scatter: Optional[w2d.IndexScatter] = None,
+        index_plot: Optional[w2d.IndexPlot] = None,
         mesh_grid_scatter: Optional[w2d.MeshGridScatter] = None,
         parallel_overscan_plot: Optional[w2d.ParallelOverscanPlot] = None,
         serial_prescan_plot: Optional[w2d.SerialPrescanPlot] = None,
         serial_overscan_plot: Optional[w2d.SerialOverscanPlot] = None,
+        use_log10: bool = False,
+        plot_mask: bool = True,
+        quick_update: bool = False,
     ):
         """
         Visualizes 2D data structures (e.g an `Array2D`, `Grid2D`, `VectorField`, etc.) using Matplotlib.
@@ -72,7 +81,7 @@ class MatPlot2D(AbstractMatPlot):
         - `Line`: using `plt.plot`, `plt.semilogy`, `plt.loglog` or `plt.scatter`.
         - `VectorField`: using `plt.quiver`.
         - `RectangularMapper`: using `plt.imshow`.
-        - `MapperVoronoiNoInterp`: using `plt.fill`.
+        - `MapperVoronoi`: using `plt.fill`.
 
         Parameters
         ----------
@@ -115,6 +124,10 @@ class MatPlot2D(AbstractMatPlot):
             Sets if the figure is displayed on the user's screen or output to `.png` using `plt.show` and `plt.savefig`
         array_overlay
             Overlays an input `Array2D` over the figure using `plt.imshow`.
+        fill
+            Sets the fill of the figure using `plt.fill` and customizes its appearance, such as the color and alpha.
+        contour
+            Overlays contours of an input `Array2D` over the figure using `plt.contour`.
         grid_scatter
             Scatters a `Grid2D` of (y,x) coordinates over the figure using `plt.scatter`.
         grid_plot
@@ -123,6 +136,8 @@ class MatPlot2D(AbstractMatPlot):
             Plots a `VectorField` object using the matplotlib function `plt.quiver`.
         patch_overlay
             Overlays matplotlib `patches.Patch` objects over the figure, such as an `Ellipse`.
+        delaunay_drawer
+            Draws a colored Delaunay mesh of pixels using `plt.tripcolor`.
         voronoi_drawer
             Draws a colored Voronoi mesh of pixels using `plt.fill`.
         interpolated_reconstruction
@@ -145,6 +160,8 @@ class MatPlot2D(AbstractMatPlot):
             Plots the serial prescan on an `Array2D` data structure representing a CCD imaging via `plt.plot`.
         serial_overscan_plot
             Plots the serial overscan on an `Array2D` data structure representing a CCD imaging via `plt.plot`.
+        use_log10
+            If True, the plot has a log10 colormap, colorbar and contours showing the values.
         """
 
         super().__init__(
@@ -167,6 +184,9 @@ class MatPlot2D(AbstractMatPlot):
         )
 
         self.array_overlay = array_overlay or w2d.ArrayOverlay(is_default=True)
+        self.fill = fill or w2d.Fill(is_default=True)
+
+        self.contour = contour or w2d.Contour(is_default=True)
 
         self.grid_scatter = grid_scatter or w2d.GridScatter(is_default=True)
         self.grid_plot = grid_plot or w2d.GridPlot(is_default=True)
@@ -179,6 +199,7 @@ class MatPlot2D(AbstractMatPlot):
             interpolated_reconstruction
             or w2d.InterpolatedReconstruction(is_default=True)
         )
+        self.delaunay_drawer = delaunay_drawer or w2d.DelaunayDrawer(is_default=True)
         self.voronoi_drawer = voronoi_drawer or w2d.VoronoiDrawer(is_default=True)
 
         self.origin_scatter = origin_scatter or w2d.OriginScatter(is_default=True)
@@ -188,6 +209,7 @@ class MatPlot2D(AbstractMatPlot):
             is_default=True
         )
         self.index_scatter = index_scatter or w2d.IndexScatter(is_default=True)
+        self.index_plot = index_plot or w2d.IndexPlot(is_default=True)
         self.mesh_grid_scatter = mesh_grid_scatter or w2d.MeshGridScatter(
             is_default=True
         )
@@ -202,13 +224,18 @@ class MatPlot2D(AbstractMatPlot):
             is_default=True
         )
 
+        self.use_log10 = use_log10
+        self.plot_mask = plot_mask
+
         self.is_for_subplot = False
+        self.quick_update = quick_update
 
     def plot_array(
         self,
         array: Array2D,
         visuals_2d: Visuals2D,
         auto_labels: AutoLabels,
+        grid_indexes=None,
         bypass: bool = False,
     ):
         """
@@ -229,27 +256,24 @@ class MatPlot2D(AbstractMatPlot):
         if array is None or np.all(array == 0):
             return
 
+        if self.use_log10 and (np.all(array == array[0]) or np.all(array < 0)):
+            return
+
         if array.pixel_scales is None and self.units.use_scaled:
             raise exc.ArrayException(
                 "You cannot plot an array using its scaled unit_label if the input array does not have "
                 "a pixel scales attribute."
             )
 
-        array = array.binned
-
-        if array.mask.is_all_false:
-            buffer = 0
-        else:
-            buffer = 1
-
         if conf.instance["visualize"]["general"]["general"]["zoom_around_mask"]:
 
-            extent = array.extent_of_zoomed_array(buffer=buffer)
-            array = array.zoomed_around_mask(buffer=buffer)
+            zoom = Zoom2D(mask=array.mask)
 
-        else:
+            buffer = 0 if array.mask.is_all_false else 1
 
-            extent = array.geometry.extent
+            array = zoom.array_2d_from(array=array, buffer=buffer)
+
+        extent = array.geometry.extent
 
         ax = None
 
@@ -260,18 +284,30 @@ class MatPlot2D(AbstractMatPlot):
                 ax = self.setup_subplot()
 
         aspect = self.figure.aspect_from(shape_native=array.shape_native)
-        norm_scale = self.cmap.norm_from(array=array)
+
+        norm = self.cmap.norm_from(array=array.array, use_log10=self.use_log10)
 
         origin = conf.instance["visualize"]["general"]["general"]["imshow_origin"]
 
-        plt.imshow(
-            X=array.native,
-            aspect=aspect,
-            cmap=self.cmap.cmap,
-            norm=norm_scale,
-            extent=extent,
-            origin=origin,
-        )
+        if isinstance(array, Array2DRGB):
+
+            plt.imshow(
+                X=array.native.array,
+                aspect=aspect,
+                extent=extent,
+                origin=origin,
+            )
+
+        else:
+
+            plt.imshow(
+                X=array.native.array,
+                aspect=aspect,
+                cmap=self.cmap.cmap,
+                norm=norm,
+                extent=extent,
+                origin=origin,
+            )
 
         if visuals_2d.array_overlay is not None:
             self.array_overlay.overlay_array(
@@ -281,7 +317,6 @@ class MatPlot2D(AbstractMatPlot):
         extent_axis = self.axis.config_dict.get("extent")
 
         if extent_axis is None:
-
             extent_axis = extent
 
         self.axis.set(extent=extent_axis)
@@ -289,22 +324,27 @@ class MatPlot2D(AbstractMatPlot):
         self.tickparams.set()
 
         self.yticks.set(
-            array=array,
             min_value=extent_axis[2],
             max_value=extent_axis[3],
             units=self.units,
+            pixels=array.shape_native[0],
         )
 
         self.xticks.set(
-            array=array,
             min_value=extent_axis[0],
             max_value=extent_axis[1],
             units=self.units,
+            pixels=array.shape_native[1],
         )
 
-        self.title.set(auto_title=auto_labels.title)
-        self.ylabel.set(units=self.units, include_brackets=True)
-        self.xlabel.set(units=self.units, include_brackets=True)
+        if isinstance(array, Array2DRGB):
+            title = "RGB"
+        else:
+            title = auto_labels.title
+
+        self.title.set(auto_title=title, use_log10=self.use_log10)
+        self.ylabel.set()
+        self.xlabel.set()
 
         if not isinstance(self.text, list):
             self.text.set()
@@ -317,13 +357,26 @@ class MatPlot2D(AbstractMatPlot):
             [annotate.set() for annotate in self.annotate]
 
         if self.colorbar is not False:
-            cb = self.colorbar.set(ax=ax)
+            cb = self.colorbar.set(
+                units=self.units,
+                ax=ax,
+                norm=norm,
+                cb_unit=auto_labels.cb_unit,
+                use_log10=self.use_log10,
+            )
             self.colorbar_tickparams.set(cb=cb)
 
-        grid_indexes = None
+        if self.contour is not False:
+            try:
+                self.contour.set(array=array, extent=extent, use_log10=self.use_log10)
+            except ValueError:
+                pass
 
-        if visuals_2d.indexes is not None or visuals_2d.pix_indexes is not None:
-            grid_indexes = array.mask.derive_grid.unmasked
+        if self.plot_mask and visuals_2d.mask is None:
+
+            if not array.mask.is_all_false:
+
+                self.mask_scatter.scatter_grid(grid=array.mask.derive_grid.edge.array)
 
         visuals_2d.plot_via_plotter(plotter=self, grid_indexes=grid_indexes)
 
@@ -339,7 +392,9 @@ class MatPlot2D(AbstractMatPlot):
         color_array=None,
         y_errors=None,
         x_errors=None,
-        buffer=1.0,
+        plot_grid_lines=False,
+        plot_over_sampled_grid=False,
+        buffer=0.1,
     ):
         """Plot a grid of (y,x) Cartesian coordinates as a scatter plotter of points.
 
@@ -357,26 +412,29 @@ class MatPlot2D(AbstractMatPlot):
         else:
             ax = self.setup_subplot()
 
-        if color_array is None:
+        if plot_over_sampled_grid:
+            grid_plot = grid.over_sampled
+        else:
+            grid_plot = grid
 
+        if color_array is None:
             if y_errors is None and x_errors is None:
-                self.grid_scatter.scatter_grid(grid=grid)
+                self.grid_scatter.scatter_grid(grid=grid_plot.array)
             else:
                 self.grid_errorbar.errorbar_grid(
-                    grid=grid, y_errors=y_errors, x_errors=x_errors
+                    grid=grid_plot.array, y_errors=y_errors, x_errors=x_errors
                 )
 
         elif color_array is not None:
-
             cmap = plt.get_cmap(self.cmap.cmap)
 
             if y_errors is None and x_errors is None:
                 self.grid_scatter.scatter_grid_colored(
-                    grid=grid, color_array=color_array, cmap=cmap
+                    grid=grid.array, color_array=color_array, cmap=cmap
                 )
             else:
                 self.grid_errorbar.errorbar_grid_colored(
-                    grid=grid,
+                    grid=grid.array,
                     cmap=cmap,
                     color_array=color_array,
                     y_errors=y_errors,
@@ -386,14 +444,17 @@ class MatPlot2D(AbstractMatPlot):
             if self.colorbar is not None:
 
                 colorbar = self.colorbar.set_with_color_values(
-                    cmap=self.cmap.cmap, color_values=color_array, ax=ax
+                    units=self.units,
+                    cmap=self.cmap.cmap,
+                    color_values=color_array,
+                    ax=ax,
                 )
                 if colorbar is not None and self.colorbar_tickparams is not None:
                     self.colorbar_tickparams.set(cb=colorbar)
 
         self.title.set(auto_title=auto_labels.title)
-        self.ylabel.set(units=self.units, include_brackets=True)
-        self.xlabel.set(units=self.units, include_brackets=True)
+        self.ylabel.set()
+        self.xlabel.set()
 
         if not isinstance(self.text, list):
             self.text.set()
@@ -408,23 +469,26 @@ class MatPlot2D(AbstractMatPlot):
         extent = self.axis.config_dict.get("extent")
 
         if extent is None:
+            extent = grid.extent_with_buffer_from(buffer=buffer)
 
-            extent = np.asarray(grid.geometry.extent)
-            extent = extent + (buffer * extent)
+        if plot_grid_lines:
+            self.grid_plot.plot_rectangular_grid_lines(
+                extent=grid.geometry.extent,
+                shape_native=grid.shape_native,
+            )
 
         self.axis.set(extent=extent, grid=grid)
 
         self.tickparams.set()
 
         if not self.axis.symmetric_around_centre:
-            self.yticks.set(
-                array=None, min_value=extent[2], max_value=extent[3], units=self.units
-            )
-            self.xticks.set(
-                array=None, min_value=extent[0], max_value=extent[1], units=self.units
-            )
+            self.yticks.set(min_value=extent[2], max_value=extent[3], units=self.units)
+            self.xticks.set(min_value=extent[0], max_value=extent[1], units=self.units)
 
-        visuals_2d.plot_via_plotter(plotter=self, grid_indexes=grid)
+        if self.contour is not False:
+            self.contour.set(array=color_array, extent=extent, use_log10=self.use_log10)
+
+        visuals_2d.plot_via_plotter(plotter=self, grid_indexes=grid.array)
 
         if not self.is_for_subplot:
             self.output.to_figure(structure=grid, auto_filename=auto_labels.filename)
@@ -432,21 +496,20 @@ class MatPlot2D(AbstractMatPlot):
 
     def plot_mapper(
         self,
-        mapper: Union[MapperRectangularNoInterp, MapperVoronoiNoInterp],
+        mapper: MapperRectangular,
         visuals_2d: Visuals2D,
         auto_labels: AutoLabels,
-        interpolate_to_uniform: bool = True,
-        source_pixelization_values=None,
+        interpolate_to_uniform: bool = False,
+        pixel_values: np.ndarray = Optional[None],
+        zoom_to_brightest: bool = True,
     ):
-
-        if isinstance(mapper, MapperRectangularNoInterp):
-
+        if isinstance(mapper, MapperRectangular):
             self._plot_rectangular_mapper(
                 mapper=mapper,
                 visuals_2d=visuals_2d,
                 auto_labels=auto_labels,
-                interpolate_to_uniform=interpolate_to_uniform,
-                source_pixelization_values=source_pixelization_values,
+                pixel_values=pixel_values,
+                zoom_to_brightest=zoom_to_brightest,
             )
 
         elif isinstance(mapper, MapperDelaunay):
@@ -455,50 +518,46 @@ class MatPlot2D(AbstractMatPlot):
                 visuals_2d=visuals_2d,
                 auto_labels=auto_labels,
                 interpolate_to_uniform=interpolate_to_uniform,
-                source_pixelization_values=source_pixelization_values,
+                pixel_values=pixel_values,
+                zoom_to_brightest=zoom_to_brightest,
             )
         else:
-
             self._plot_voronoi_mapper(
                 mapper=mapper,
                 visuals_2d=visuals_2d,
                 auto_labels=auto_labels,
                 interpolate_to_uniform=interpolate_to_uniform,
-                source_pixelization_values=source_pixelization_values,
+                pixel_values=pixel_values,
+                zoom_to_brightest=zoom_to_brightest,
             )
 
     def _plot_rectangular_mapper(
         self,
-        mapper: MapperRectangularNoInterp,
+        mapper: MapperRectangular,
         visuals_2d: Visuals2D,
         auto_labels: AutoLabels,
-        interpolate_to_uniform: bool = True,
-        source_pixelization_values=None,
+        pixel_values: np.ndarray = Optional[None],
+        zoom_to_brightest: bool = True,
     ):
-
-        if source_pixelization_values is not None:
-
+        if pixel_values is not None:
             solution_array_2d = array_2d_util.array_2d_native_from(
-                array_2d_slim=source_pixelization_values,
+                array_2d_slim=pixel_values,
                 mask_2d=np.full(
                     fill_value=False, shape=mapper.source_plane_mesh_grid.shape_native
                 ),
-                sub_size=1,
             )
 
-            source_pixelization_values = Array2D.no_mask(
+            pixel_values = Array2D.no_mask(
                 values=solution_array_2d,
-                sub_size=1,
                 pixel_scales=mapper.source_plane_mesh_grid.pixel_scales,
                 origin=mapper.source_plane_mesh_grid.origin,
             )
 
         extent = self.axis.config_dict.get("extent")
-        extent = (
-            extent
-            if extent is not None
-            else mapper.source_plane_mesh_grid.geometry.extent
-        )
+        if extent is None:
+            extent = mapper.extent_from(
+                values=pixel_values, zoom_to_brightest=zoom_to_brightest
+            )
 
         aspect_inv = self.figure.aspect_for_subplot_from(extent=extent)
 
@@ -507,22 +566,79 @@ class MatPlot2D(AbstractMatPlot):
         else:
             ax = self.setup_subplot(aspect=aspect_inv)
 
-        if source_pixelization_values is not None:
-            self.plot_array(
-                array=source_pixelization_values,
-                visuals_2d=visuals_2d,
-                auto_labels=auto_labels,
-                bypass=True,
+        shape_native = mapper.source_plane_mesh_grid.shape_native
+
+        if pixel_values is not None:
+
+            from autoarray.inversion.pixelization.mappers.rectangular_uniform import (
+                MapperRectangularUniform,
+            )
+            from autoarray.inversion.pixelization.mappers.rectangular import (
+                MapperRectangular,
             )
 
-        self.axis.set(extent=extent, grid=mapper.source_plane_mesh_grid)
+            if isinstance(mapper, MapperRectangularUniform):
 
-        self.yticks.set(
-            array=None, min_value=extent[2], max_value=extent[3], units=self.units
-        )
-        self.xticks.set(
-            array=None, min_value=extent[0], max_value=extent[1], units=self.units
-        )
+                self.plot_array(
+                    array=pixel_values,
+                    visuals_2d=visuals_2d,
+                    auto_labels=auto_labels,
+                    bypass=True,
+                )
+
+            else:
+
+                norm = self.cmap.norm_from(
+                    array=pixel_values.array, use_log10=self.use_log10
+                )
+
+                edges_transformed = mapper.edges_transformed
+
+                edges_transformed_dense = np.moveaxis(
+                    np.stack(np.meshgrid(*edges_transformed.T)), 0, 2
+                )
+
+                plt.pcolormesh(
+                    edges_transformed_dense[..., 0],
+                    edges_transformed_dense[..., 1],
+                    pixel_values.array.reshape(shape_native),
+                    shading="flat",
+                    norm=norm,
+                    cmap=self.cmap.cmap,
+                )
+
+                if self.colorbar is not False:
+
+                    cb = self.colorbar.set(
+                        units=self.units,
+                        ax=ax,
+                        norm=norm,
+                        cb_unit=auto_labels.cb_unit,
+                        use_log10=self.use_log10,
+                    )
+                    self.colorbar_tickparams.set(cb=cb)
+
+                extent_axis = self.axis.config_dict.get("extent")
+
+                if extent_axis is None:
+                    extent_axis = extent
+
+                self.axis.set(extent=extent_axis)
+
+                self.tickparams.set()
+                self.yticks.set(
+                    min_value=extent_axis[2],
+                    max_value=extent_axis[3],
+                    units=self.units,
+                    pixels=shape_native[0],
+                )
+
+                self.xticks.set(
+                    min_value=extent_axis[0],
+                    max_value=extent_axis[1],
+                    units=self.units,
+                    pixels=shape_native[1],
+                )
 
         if not isinstance(self.text, list):
             self.text.set()
@@ -534,18 +650,17 @@ class MatPlot2D(AbstractMatPlot):
         else:
             [annotate.set() for annotate in self.annotate]
 
-        self.grid_plot.plot_rectangular_grid_lines(
-            extent=mapper.source_plane_mesh_grid.geometry.extent,
-            shape_native=mapper.shape_native,
-        )
+        # self.grid_plot.plot_rectangular_grid_lines(
+        #     extent=mapper.source_plane_mesh_grid.geometry.extent,
+        #     shape_native=mapper.shape_native,
+        # )
 
         self.title.set(auto_title=auto_labels.title)
-        self.tickparams.set()
-        self.ylabel.set(units=self.units, include_brackets=True)
-        self.xlabel.set(units=self.units, include_brackets=True)
+        self.ylabel.set()
+        self.xlabel.set()
 
         visuals_2d.plot_via_plotter(
-            plotter=self, grid_indexes=mapper.source_plane_data_grid, mapper=mapper
+            plotter=self, grid_indexes=mapper.source_plane_data_grid.over_sampled
         )
 
         if not self.is_for_subplot:
@@ -557,81 +672,15 @@ class MatPlot2D(AbstractMatPlot):
         mapper: MapperDelaunay,
         visuals_2d: Visuals2D,
         auto_labels: AutoLabels,
-        interpolate_to_uniform: bool = True,
-        source_pixelization_values=None,
+        interpolate_to_uniform: bool = False,
+        pixel_values: np.ndarray = Optional[None],
+        zoom_to_brightest: bool = True,
     ):
-
         extent = self.axis.config_dict.get("extent")
-        extent = (
-            extent
-            if extent is not None
-            else mapper.source_plane_mesh_grid.geometry.extent
-        )
-
-        aspect_inv = self.figure.aspect_for_subplot_from(extent=extent)
-
-        if not self.is_for_subplot:
-            fig, ax = self.figure.open()
-        else:
-            ax = self.setup_subplot(aspect=aspect_inv)
-
-        self.axis.set(extent=extent, grid=mapper.source_plane_mesh_grid)
-
-        self.tickparams.set()
-        self.yticks.set(
-            array=None, min_value=extent[2], max_value=extent[3], units=self.units
-        )
-        self.xticks.set(
-            array=None, min_value=extent[0], max_value=extent[1], units=self.units
-        )
-
-        if not isinstance(self.text, list):
-            self.text.set()
-        else:
-            [text.set() for text in self.text]
-
-        if not isinstance(self.annotate, list):
-            self.annotate.set()
-        else:
-            [annotate.set() for annotate in self.annotate]
-
-        self.interpolated_reconstruction.imshow_reconstruction(
-            mapper=mapper,
-            pixel_values=source_pixelization_values,
-            cmap=self.cmap,
-            colorbar=self.colorbar,
-            colorbar_tickparams=self.colorbar_tickparams,
-            aspect=aspect_inv,
-            ax=ax,
-        )
-
-        self.title.set(auto_title=auto_labels.title)
-        self.ylabel.set(units=self.units, include_brackets=True)
-        self.xlabel.set(units=self.units, include_brackets=True)
-
-        visuals_2d.plot_via_plotter(
-            plotter=self, grid_indexes=mapper.source_plane_data_grid, mapper=mapper
-        )
-
-        if not self.is_for_subplot:
-            self.output.to_figure(structure=None, auto_filename=auto_labels.filename)
-            self.figure.close()
-
-    def _plot_voronoi_mapper(
-        self,
-        mapper: MapperVoronoiNoInterp,
-        visuals_2d: Visuals2D,
-        auto_labels: AutoLabels,
-        interpolate_to_uniform: bool = True,
-        source_pixelization_values=None,
-    ):
-
-        extent = self.axis.config_dict.get("extent")
-        extent = (
-            extent
-            if extent is not None
-            else mapper.source_plane_mesh_grid.geometry.extent
-        )
+        if extent is None:
+            extent = mapper.extent_from(
+                values=pixel_values, zoom_to_brightest=zoom_to_brightest
+            )
 
         aspect_inv = self.figure.aspect_for_subplot_from(extent=extent)
 
@@ -645,13 +694,93 @@ class MatPlot2D(AbstractMatPlot):
         plt.gca().set_aspect(aspect_inv)
 
         self.tickparams.set()
-        self.yticks.set(
-            array=None, min_value=extent[2], max_value=extent[3], units=self.units
-        )
-        self.xticks.set(
-            array=None, min_value=extent[0], max_value=extent[1], units=self.units
+        self.yticks.set(min_value=extent[2], max_value=extent[3], units=self.units)
+        self.xticks.set(min_value=extent[0], max_value=extent[1], units=self.units)
+
+        if not isinstance(self.text, list):
+            self.text.set()
+        else:
+            [text.set() for text in self.text]
+
+        if not isinstance(self.annotate, list):
+            self.annotate.set()
+        else:
+            [annotate.set() for annotate in self.annotate]
+
+        interpolation_array = None
+
+        if interpolate_to_uniform:
+            interpolation_array = (
+                self.interpolated_reconstruction.imshow_reconstruction(
+                    mapper=mapper,
+                    pixel_values=pixel_values,
+                    units=self.units,
+                    cmap=self.cmap,
+                    colorbar=self.colorbar,
+                    colorbar_tickparams=self.colorbar_tickparams,
+                    aspect=aspect_inv,
+                    ax=ax,
+                    use_log10=self.use_log10,
+                )
+            )
+
+        else:
+            self.delaunay_drawer.draw_delaunay_pixels(
+                mapper=mapper,
+                pixel_values=pixel_values,
+                units=self.units,
+                cmap=self.cmap,
+                colorbar=self.colorbar,
+                colorbar_tickparams=self.colorbar_tickparams,
+                ax=ax,
+                use_log10=self.use_log10,
+            )
+
+        self.title.set(auto_title=auto_labels.title)
+        self.ylabel.set()
+        self.xlabel.set()
+
+        visuals_2d.plot_via_plotter(
+            plotter=self, grid_indexes=mapper.source_plane_data_grid.over_sampled
         )
 
+        if not self.is_for_subplot:
+            self.output.to_figure(
+                structure=interpolation_array, auto_filename=auto_labels.filename
+            )
+            self.figure.close()
+
+    def _plot_voronoi_mapper(
+        self,
+        mapper: MapperVoronoi,
+        visuals_2d: Visuals2D,
+        auto_labels: AutoLabels,
+        interpolate_to_uniform: bool = False,
+        pixel_values: np.ndarray = Optional[None],
+        zoom_to_brightest: bool = True,
+    ):
+        extent = self.axis.config_dict.get("extent")
+
+        if extent is None:
+            extent = mapper.extent_from(
+                values=pixel_values, zoom_to_brightest=zoom_to_brightest
+            )
+
+        aspect_inv = self.figure.aspect_for_subplot_from(extent=extent)
+
+        if not self.is_for_subplot:
+            fig, ax = self.figure.open()
+        else:
+            ax = self.setup_subplot(aspect=aspect_inv)
+
+        self.axis.set(extent=extent, grid=mapper.source_plane_mesh_grid)
+
+        plt.gca().set_aspect(aspect_inv)
+
+        self.tickparams.set()
+        self.yticks.set(min_value=extent[2], max_value=extent[3], units=self.units)
+        self.xticks.set(min_value=extent[0], max_value=extent[1], units=self.units)
+
         if not isinstance(self.text, list):
             self.text.set()
         else:
@@ -663,36 +792,45 @@ class MatPlot2D(AbstractMatPlot):
             [annotate.set() for annotate in self.annotate]
 
         if not interpolate_to_uniform:
-
             self.voronoi_drawer.draw_voronoi_pixels(
                 mapper=mapper,
-                pixel_values=source_pixelization_values,
+                units=self.units,
+                pixel_values=pixel_values,
                 cmap=self.cmap,
                 colorbar=self.colorbar,
                 colorbar_tickparams=self.colorbar_tickparams,
                 ax=ax,
+                use_log10=self.use_log10,
             )
 
         else:
-
             self.interpolated_reconstruction.imshow_reconstruction(
                 mapper=mapper,
-                pixel_values=source_pixelization_values,
+                pixel_values=pixel_values,
+                units=self.units,
                 cmap=self.cmap,
                 colorbar=self.colorbar,
                 colorbar_tickparams=self.colorbar_tickparams,
                 aspect=aspect_inv,
                 ax=ax,
+                use_log10=self.use_log10,
             )
 
         self.title.set(auto_title=auto_labels.title)
-        self.ylabel.set(units=self.units, include_brackets=True)
-        self.xlabel.set(units=self.units, include_brackets=True)
+        self.ylabel.set()
+        self.xlabel.set()
 
         visuals_2d.plot_via_plotter(
-            plotter=self, grid_indexes=mapper.source_plane_data_grid, mapper=mapper
+            plotter=self, grid_indexes=mapper.source_plane_data_grid.over_sampled
         )
 
+        if pixel_values is not None:
+            interpolation_array = mapper.interpolated_array_from(values=pixel_values)
+        else:
+            interpolation_array = None
+
         if not self.is_for_subplot:
-            self.output.to_figure(structure=None, auto_filename=auto_labels.filename)
+            self.output.to_figure(
+                structure=interpolation_array, auto_filename=auto_labels.filename
+            )
             self.figure.close()
