@@ -1,5 +1,3 @@
-import matplotlib.pyplot as plt
-
 from autoarray.plot.wrap.base.abstract import AbstractMatWrap
 
 
@@ -12,16 +10,17 @@ class Legend(AbstractMatWrap):
     - plt.legend: https://matplotlib.org/3.3.2/api/_as_gen/matplotlib.pyplot.legend.html
     """
 
-    def __init__(self, include=True, **kwargs):
-
+    def __init__(self, label=None, include=True, **kwargs):
         super().__init__(**kwargs)
 
+        self.label = label
         self.include = include
 
     def set(self):
 
-        if self.include:
+        import matplotlib.pyplot as plt
 
+        if self.include:
             config_dict = self.config_dict
             config_dict.pop("include") if "include" in config_dict else None
             config_dict.pop("include_2d") if "include_2d" in config_dict else None
