@@ -1,4 +1,6 @@
 from .array_overlay import ArrayOverlay
+from .contour import Contour
+from .fill import Fill
 from .grid_scatter import GridScatter
 from .grid_plot import GridPlot
 from .grid_errorbar import GridErrorbar
@@ -7,12 +9,14 @@ from .patch_overlay import PatchOverlay
 from .interpolated_reconstruction import (
     InterpolatedReconstruction,
 )
+from .delaunay_drawer import DelaunayDrawer
 from .voronoi_drawer import VoronoiDrawer
 from .origin_scatter import OriginScatter
 from .mask_scatter import MaskScatter
 from .border_scatter import BorderScatter
 from .positions_scatter import PositionsScatter
 from .index_scatter import IndexScatter
+from .index_plot import IndexPlot
 from .mesh_grid_scatter import MeshGridScatter
 from .parallel_overscan_plot import ParallelOverscanPlot
 from .serial_prescan_plot import SerialPrescanPlot
