@@ -1,8 +1,7 @@
 from __future__ import annotations
 from abc import ABC
 from abc import abstractmethod
-import numpy as np
-from typing import TYPE_CHECKING, List, Tuple, Union
+from typing import TYPE_CHECKING, Dict, Tuple, Union
 
 if TYPE_CHECKING:
     from autoarray.structures.grids.uniform_1d import Grid1D
@@ -18,7 +17,6 @@ from autoarray import exc
 
 class Structure(AbstractNDArray, ABC):
     def __array_finalize__(self, obj):
-
         if hasattr(obj, "mask"):
             self.mask = obj.mask
 
@@ -49,18 +47,10 @@ class Structure(AbstractNDArray, ABC):
     def shape_slim(self) -> int:
         return self.mask.shape_slim
 
-    @property
-    def sub_shape_slim(self) -> int:
-        return self.mask.sub_shape_slim
-
     @property
     def shape_native(self) -> Tuple[int, ...]:
         return self.mask.shape
 
-    @property
-    def sub_shape_native(self) -> Tuple[int, ...]:
-        return self.mask.sub_shape_native
-
     @property
     def pixel_scales(self) -> Tuple[float, ...]:
         return self.mask.pixel_scales
@@ -69,11 +59,13 @@ class Structure(AbstractNDArray, ABC):
     def pixel_scale(self) -> float:
         return self.mask.pixel_scale
 
+    @property
+    def header_dict(self) -> Dict:
+        return self.mask.header_dict
+
     @property
     def pixel_area(self):
-
         if len(self.pixel_scales) != 2:
-
             raise exc.GridException("Cannot compute area of structure which is not 2D.")
 
         return self.pixel_scales[0] * self.pixel_scales[1]
@@ -86,23 +78,13 @@ class Structure(AbstractNDArray, ABC):
     def origin(self) -> Tuple[int, ...]:
         return self.mask.origin
 
-    @property
-    def sub_size(self) -> int:
-        return self.mask.sub_size
-
     @property
     def unmasked_grid(self) -> Union[Grid1D, Grid2D]:
-        return self.mask.derive_grid.all_false_sub_1
+        return self.mask.derive_grid.all_false
 
     @property
     def total_pixels(self) -> int:
         return self.shape[0]
 
-    def structure_2d_list_from(self, result_list: list) -> List["Structure"]:
-        raise NotImplementedError
-
-    def structure_2d_from(self, result: np.ndarray) -> "Structure":
-        raise NotImplementedError
-
     def trimmed_after_convolution_from(self, kernel_shape) -> "Structure":
         raise NotImplementedError
