@@ -1,6 +1,9 @@
 import numpy as np
+from pathlib import Path
 from typing import Optional, Union, Tuple, List
 
+from autoconf.fitsable import ndarray_via_fits_from, header_obj_from
+
 from autoarray.structures.header import Header
 
 from autoarray.structures.abstract_structure import Structure
@@ -14,32 +17,30 @@ from autoarray import type as ty
 
 
 class Array1D(Structure):
-    def __new__(
-        cls,
+    def __init__(
+        self,
         values: Union[np.ndarray, List],
         mask: Mask1D,
         header: Optional[Header] = None,
         store_native: bool = False,
-        *args,
-        **kwargs
     ):
 
         values = array_1d_util.convert_array_1d(
-            array_1d=values, mask_1d=mask, store_native=store_native
+            array_1d=values,
+            mask_1d=mask,
+            store_native=store_native,
         )
 
-        obj = values.view(cls)
-        obj.mask = mask
-        obj.header = header
+        self.mask = mask
+        self.header = header
 
-        return obj
+        super().__init__(values)
 
     @classmethod
     def no_mask(
         cls,
         values: Union[np.ndarray, Tuple[float], List[float]],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float] = (0.0,),
         header: Optional[Header] = None,
     ) -> "Array1D":
@@ -49,11 +50,9 @@ class Array1D(Structure):
         Parameters
         ----------
         values
-            The values of the array input as an ndarray of shape [total_unmasked_pixels*sub_size] or a list.
+            The values of the array input as an ndarray of shape [total_unmasked_pixels] or a list.
         pixel_scales
             The scaled units to pixel units conversion factor of the array data coordinates (e.g. the x-axis).
-        sub_size
-            The size of each unmasked pixels sub-grid.
         origin
             The origin of the 1D array's mask.
 
@@ -84,13 +83,12 @@ class Array1D(Structure):
         pixel_scales = geometry_util.convert_pixel_scales_1d(pixel_scales=pixel_scales)
 
         mask = Mask1D.all_false(
-            shape_slim=values.shape[0] // sub_size,
+            shape_slim=values.shape[0],
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
         )
 
-        return Array1D(values=values, mask=mask, header=header)
+        return Array1D(values=np.array(values), mask=mask, header=header)
 
     @classmethod
     def full(
@@ -98,7 +96,6 @@ class Array1D(Structure):
         fill_value: float,
         shape_native: Union[int, Tuple[int]],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float] = (0.0,),
         header: Optional[Header] = None,
     ) -> "Array1D":
@@ -118,20 +115,14 @@ class Array1D(Structure):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float,) structure.
-        sub_size
-            The size (sub_size) of each unmasked pixels sub-array.
         origin : (float,)
             The (x) scaled units origin of the mask's coordinate system.
         """
         shape_native = geometry_util.convert_shape_native_1d(shape_native=shape_native)
 
-        if sub_size is not None:
-            shape_native = (shape_native[0] * sub_size,)
-
         return cls.no_mask(
             values=np.full(fill_value=fill_value, shape=shape_native[0]),
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=header,
         )
@@ -141,7 +132,6 @@ class Array1D(Structure):
         cls,
         shape_native: Union[int, Tuple[int]],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float] = (0.0,),
         header: Optional[Header] = None,
     ) -> "Array1D":
@@ -159,8 +149,6 @@ class Array1D(Structure):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float,) structure.
-        sub_size
-            The size (sub_size) of each unmasked pixels sub-array.
         origin : (float,)
             The (x) scaled units origin of the mask's coordinate system.
         """
@@ -168,7 +156,6 @@ class Array1D(Structure):
             fill_value=0.0,
             shape_native=shape_native,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=header,
         )
@@ -178,7 +165,6 @@ class Array1D(Structure):
         cls,
         shape_native: Union[int, Tuple[int]],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float] = (0.0,),
         header: Optional[Header] = None,
     ) -> "Array1D":
@@ -196,8 +182,6 @@ class Array1D(Structure):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float,) structure.
-        sub_size
-            The size (sub_size) of each unmasked pixels sub-array.
         origin : (float,)
             The (x) scaled units origin of the mask's coordinate system.
         """
@@ -205,7 +189,6 @@ class Array1D(Structure):
             fill_value=1.0,
             shape_native=shape_native,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=header,
         )
@@ -213,10 +196,9 @@ class Array1D(Structure):
     @classmethod
     def from_fits(
         cls,
-        file_path: str,
+        file_path: Union[Path, str],
         pixel_scales: ty.PixelScales,
         hdu: int = 0,
-        sub_size: int = 1,
         origin: Tuple[float] = (0.0, 0.0),
     ) -> "Array1D":
         """
@@ -232,24 +214,19 @@ class Array1D(Structure):
         pixel_scales
             The (x,) scaled units to pixel units conversion factors of every pixel. If this is input as a float,
             it is converted to a (float,) structure.
-        sub_size
-            The sub-size of each unmasked pixels sub-array.
         origin
             The (x,) scaled units origin of the coordinate system.
         """
-        array_1d = array_1d_util.numpy_array_1d_via_fits_from(
-            file_path=file_path, hdu=hdu
-        )
+        array_1d = ndarray_via_fits_from(file_path=file_path, hdu=hdu)
 
-        header_sci_obj = array_2d_util.header_obj_from(file_path=file_path, hdu=0)
-        header_hdu_obj = array_2d_util.header_obj_from(file_path=file_path, hdu=hdu)
+        header_sci_obj = header_obj_from(file_path=file_path, hdu=0)
+        header_hdu_obj = header_obj_from(file_path=file_path, hdu=hdu)
 
         return cls.no_mask(
             values=array_1d.astype(
                 "float64"
             ),  # Have to do this due to typing issues in 1D with astorpy fits.
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=Header(header_sci_obj=header_sci_obj, header_hdu_obj=header_hdu_obj),
         )
@@ -258,7 +235,7 @@ class Array1D(Structure):
     def slim(self) -> "Array1D":
         """
         Return an `Array1D` where the data is stored its `slim` representation, which is an ndarray of shape
-        [total_unmasked_pixels * sub_size].
+        [total_unmasked_pixels].
 
         If it is already stored in its `slim` representation  it is returned as it is. If not, it is  mapped from
         `native` to `slim` and returned as a new `Array1D`.
@@ -269,7 +246,7 @@ class Array1D(Structure):
     def native(self) -> "Array1D":
         """
         Return an `Array1D` where the data is stored in its `native` representation, which is an ndarray of shape
-        [total_pixels * sub_size].
+        [total_pixels].
 
         If it is already stored in its `native` representation it is return as it is. If not, it is mapped from
         `slim` to `native` and returned as a new `Array1D`.
@@ -288,20 +265,4 @@ class Array1D(Structure):
         return Grid1D.uniform_from_zero(
             shape_native=self.shape_native,
             pixel_scales=self.pixel_scales,
-            sub_size=self.sub_size,
-        )
-
-    def output_to_fits(self, file_path: str, overwrite: bool = False):
-        """
-        Output the array to a .fits file.
-
-        Parameters
-        ----------
-        file_path
-            The output path of the file, including the filename and the `.fits` extension e.g. '/path/to/filename.fits'
-        overwrite
-            If a file already exists at the path, if overwrite=True it is overwritten else an error is raised.
-        """
-        array_1d_util.numpy_array_1d_to_fits(
-            array_1d=self.native, file_path=file_path, overwrite=overwrite
         )
