@@ -1,8 +1,13 @@
 import logging
 import numpy as np
+from pathlib import Path
 from typing import List, Optional, Tuple, Union
 
+from autoconf import conf
+from autoconf.fitsable import ndarray_via_fits_from, header_obj_from
+
 from autoarray.mask.mask_2d import Mask2D
+from autoarray.mask.derive.zoom_2d import Zoom2D
 from autoarray.structures.abstract_structure import Structure
 from autoarray.structures.header import Header
 from autoarray.structures.arrays.uniform_1d import Array1D
@@ -14,14 +19,15 @@ from autoarray.structures.arrays import array_2d_util
 from autoarray.geometry import geometry_util
 from autoarray.layout import layout_util
 
+
 logging.basicConfig()
 logger = logging.getLogger(__name__)
 
 
 class AbstractArray2D(Structure):
-    def __new__(
-        cls,
-        values: Union[np.ndarray, List],
+    def __init__(
+        self,
+        values: Union[np.ndarray, List, "AbstractArray2D"],
         mask: Mask2D,
         header: Header = None,
         store_native: bool = False,
@@ -30,7 +36,7 @@ class AbstractArray2D(Structure):
         **kwargs,
     ):
         """
-        A uniform 2D array of values, which are paired with a 2D mask of pixels which may be split into sub-pixels.
+        A uniform 2D array of values, which are paired with a 2D mask of pixels.
 
         The ``Array2D`, like all data structures (e.g. ``Grid2D``, ``VectorYX2D``) has in-built functionality which:
 
@@ -42,16 +48,16 @@ class AbstractArray2D(Structure):
         - Associates Cartesian ``Grid2D`` objects of (y,x) coordinates with the data structure (e.g.
           a (y,x) grid of all unmasked pixels).
 
-        - Associates sub-grids with the data structure, which perform calculations higher resolutions which are then
+        - Associates grids with the data structure, which perform calculations higher resolutions which are then
           binned up.
 
-        Each entry of an ``Array2D`` corresponds to a value at the centre of a sub-pixel in its
+        Each entry of an ``Array2D`` corresponds to a value at the centre of a pixel in its
         corresponding ``Mask2D``.  It is ordered such that pixels begin from the top-row of the corresponding mask
         and go right and down. The positive y-axis is upwards and positive x-axis to the right.
 
         A detailed description of the data structure API is provided below.
 
-        **SLIM DATA REPRESENTATION (sub-size=1)**
+        __slim__
 
         Below is a visual illustration of an ``Array2D``'s 2D mask, where a total of 10 pixels are unmasked and are
         included in the array.
@@ -101,7 +107,7 @@ class AbstractArray2D(Structure):
         by ``array_2d.derive_grid.masked.slim``.
 
 
-        **NATIVE DATA REPRESENTATION (sub_size=1)**
+        __native__
 
         The ``Array2D`` above, but represented as an an ``ndarray`` of shape [total_y_values, total_x_values], where
         all masked entries have values of 0.0.
@@ -169,91 +175,9 @@ class AbstractArray2D(Structure):
             slim_to_native[8] = [5,5]
             slim_to_native[9] = [5,6]
 
-        **SUB GRIDDING**
-
-        If the ``Mask2D`` ``sub_size`` is > 1, the array has entries corresponding to the values at the centre of
-        every sub-pixel of each unmasked pixel.
-
-        The sub-array indexes are ordered such that pixels begin from the first (top-left) sub-pixel in the first
-        unmasked pixel. Indexes then go over the sub-pixels in each unmasked pixel, for every unmasked pixel.
-
-        Therefore, the shapes of the sub-array are as follows:
-
-        - ``slim`` representation: an ``ndarray`` of shape [total_unmasked_pixels*sub_size**2].
-        - ``native`` representation: an ``ndarray`` of shape [total_y_values*sub_size, total_x_values*sub_size].
-
-        Below is a visual illustration of a sub array. Indexing of each sub-pixel goes from the top-left corner. In
-        contrast to the array above, our illustration below restricts the mask to just 2 pixels, to keep the
-        illustration brief.
-
-        ::
-
-             x x x x x x x x x x
-             x x x x x x x x x x     This is an example ``Mask2D``, where:
-             x x x x x x x x x x
-             x x x x x x x x x x     x = `True` (Pixel is masked and excluded from lens)
-             x 0 0 x x x x x x x     O = `False` (Pixel is not masked and included in lens)
-             x x x x x x x x x x
-             x x x x x x x x x x
-             x x x x x x x x x x
-             x x x x x x x x x x
-             x x x x x x x x x x
-
-        If ``sub_size=2``, each unmasked pixel has 4 (2x2) sub-pixel values. For the example above, pixels 0 and 1
-        each have 4 values which map to the ``array_2d``'s ``slim`` representation as follows:
-
-        ::
-
-            Pixel 0 - (2x2):
-
-                   array_2d.slim[0] = value of first sub-pixel in pixel 0.
-             0 1   array_2d.slim[1] = value of first sub-pixel in pixel 1.
-             2 3   array_2d.slim[2] = value of first sub-pixel in pixel 2.
-                   array_2d.slim[3] = value of first sub-pixel in pixel 3.
-
-            Pixel 1 - (2x2):
-
-                   array_2d.slim[4] = value of first sub-pixel in pixel 0.
-             4 5   array_2d.slim[5] = value of first sub-pixel in pixel 1.
-             6 7   array_2d.slim[6] = value of first sub-pixel in pixel 2.
-                   array_2d.slim[7] = value of first sub-pixel in pixel 3.
-
-        For the ``native`` data representation we get the following mappings:
-
-        ::
-
-            Pixel 0 - (2x2):
-
-                   array_2d.native[8, 2] = value of first sub-pixel in pixel 0.
-             0 1   array_2d.native[8, 3] = value of first sub-pixel in pixel 1.
-             2 3   array_2d.native[9, 2] = value of first sub-pixel in pixel 2.
-                   array_2d.native[9, 3] = value of first sub-pixel in pixel 3.
-
-            Pixel 1 - (2x2):
-
-                   array_2d.native[10, 4] = value of first sub-pixel in pixel 0.
-             4 5   array_2d.native[10, 5] = value of first sub-pixel in pixel 1.
-             6 7   array_2d.native[11, 4] = value of first sub-pixel in pixel 2.
-                   array_2d.native[11, 5] = value of first sub-pixel in pixel 3.
-
-            Other entries (all masked sub-pixels are zero):
-
-                   array_2d.native[0, 0] = 0.0 (it is masked, thus zero)
-                   array_2d.native[15, 12] = 0.0 (it is masked, thus zero)
-
-        If we used a sub_size of 3, for pixel 0 we we would create a 3x3 sub-array:
-
-        ::
-
-                     array_2d.slim[0] = value of first sub-pixel in pixel 0.
-                     array_2d.slim[1] = value of first sub-pixel in pixel 1.
-                     array_2d.slim[2] = value of first sub-pixel in pixel 2.
-             0 1 2   array_2d.slim[3] = value of first sub-pixel in pixel 3.
-             3 4 5   array_2d.slim[4] = value of first sub-pixel in pixel 4.
-             6 7 8   array_2d.slim[5] = value of first sub-pixel in pixel 5.
-                     array_2d.slim[6] = value of first sub-pixel in pixel 6.
-                     array_2d.slim[7] = value of first sub-pixel in pixel 7.
-                     array_2d.slim[8] = value of first sub-pixel in pixel 8.
+        In **PyAutoCTI** all `Array2D` objects are used in their `native` representation. Significant memory can be
+        saved by only store this format, thus the `native_only` config override can force this behaviour.
+        It is recommended users do not use this option to avoid unexpected behaviour.
 
         Parameters
         ----------
@@ -269,7 +193,7 @@ class AbstractArray2D(Structure):
         Examples
         --------
 
-        This example uses the ``Array2D.no_mask`` method to create the the ``Array2D``.
+        This example uses the ``Array2D.no_mask`` method to create the ``Array2D``.
 
         Different methods using different inputs are available and documented throughout this webpage.
 
@@ -277,60 +201,38 @@ class AbstractArray2D(Structure):
 
             import autoarray as aa
 
-            # Make Array2D from input np.ndarray with sub_size 1.
-
             array_2d = aa.Array2D.no_mask(
                 values=np.array([1.0, 2.0, 3.0, 4.0]),
                 shape_native=(2, 2),
                 pixel_scales=1.0,
-                sub_size=1
-            )
-
-            # Make Array2D from input list with different shape_native and sub_size 1.
-
-            array_2d = aa.Array2D.no_mask(
-                values=[1.0, 2.0, 3.0, 4.0, 5.0, 6.0],
-                shape_native=(2, 3),
-                pixel_scales=1.0,
-                sub_size=1
             )
 
         .. code-block:: python
 
             import autoarray as aa
 
-            # Make Array2D with sub_size 2.
-
             array_2d = aa.Array2D.no_mask(
-                values=[1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0],
+                values=[1.0, 2.0, 3.0, 4.0],
                 shape_native=(2, 1),
                 pixel_scales=1.0,
-                sub_size=2,
             )
 
-            # Apply 2D mask to Array2D with sub_size 2, where the
-            # True value masks entries (5.0, 6.0, 7.0, 8.0).
-
             mask = aa.Mask2D(
-                mask=[[False], [True]],
+                mask=[[False, False], [True, True]],
                 pixel_scales=2.0,
-                sub_size=2
             )
 
             array_2d = array_2d.apply_mask(mask=mask)
 
             # Print certain array attributes.
 
-            print(array_2d.slim) # masked 1D data representation on sub-grid.
-            print(array_2d.native) # masked 2D data representation on sub-grid.
-            print(array_2d.slim.binned) # masked 1D data representation binned up from sub-grid.
-            print(array_2d.native.binned) # masked 2D data representation binned up from sub-grid.
-
-            # Output array to .fits file.
-
-            array_2d.output_to_fits(file_path="/path/for/output")
+            print(array_2d.slim) # masked 1D data representation.
+            print(array_2d.native) # masked 2D data representation.
         """
 
+        if conf.instance["general"]["structures"]["native_binned_only"]:
+            store_native = True
+
         values = array_2d_util.convert_array_2d(
             array_2d=values,
             mask_2d=mask,
@@ -338,14 +240,15 @@ class AbstractArray2D(Structure):
             skip_mask=skip_mask,
         )
 
-        obj = values.view(cls)
-        obj.mask = mask
-        obj.header = header
+        super().__init__(values)
+        self.mask = mask
+        self.header = header
 
-        return obj
+    @property
+    def values(self):
+        return self._array
 
     def __array_finalize__(self, obj):
-
         if hasattr(obj, "mask"):
             self.mask = obj.mask
 
@@ -356,9 +259,7 @@ class AbstractArray2D(Structure):
 
     @property
     def store_native(self):
-        if len(self.shape) == 1:
-            return False
-        return True
+        return len(self.shape) != 1
 
     def apply_mask(self, mask: Mask2D) -> "Array2D":
         return Array2D(values=self.native, mask=mask, header=self.header)
@@ -367,7 +268,7 @@ class AbstractArray2D(Structure):
     def slim(self) -> "Array2D":
         """
         Return an `Array2D` where the data is stored its `slim` representation, which is an ``ndarray`` of shape
-        [total_unmasked_pixels * sub_size**2].
+        [total_unmasked_pixels].
 
         If it is already stored in its `slim` representation it is returned as it is. If not, it is mapped from
         `native` to `slim` and returned as a new `Array2D`.
@@ -378,7 +279,7 @@ class AbstractArray2D(Structure):
     def native(self) -> "Array2D":
         """
         Return a `Array2D` where the data is stored in its `native` representation, which is an ``ndarray`` of shape
-        [sub_size*total_y_pixels, sub_size*total_x_pixels].
+        [total_y_pixels, total_x_pixels].
 
         If it is already stored in its `native` representation it is return as it is. If not, it is mapped from
         `slim` to `native` and returned as a new `Array2D`.
@@ -387,11 +288,36 @@ class AbstractArray2D(Structure):
             values=self, mask=self.mask, header=self.header, store_native=True
         )
 
+    @property
+    def native_for_fits(self) -> "Array2D":
+        """
+        Return a `Array2D` for output to a .fits file, where the data is stored in its `native` representation,
+        which is an ``ndarray`` of shape [total_y_pixels, total_x_pixels].
+
+        Depending on configuration files, this array could be zoomed in on such that only the unmasked region
+        of the image is included in the .fits file, to save hard-disk space. Alternatively, the original `shape_native`
+        of the data can be retained.
+
+        If it is already stored in its `native` representation it is return as it is. If not, it is mapped from
+        `slim` to `native` and returned as a new `Array2D`.
+        """
+        if conf.instance["visualize"]["plots"]["fits_are_zoomed"]:
+
+            zoom = Zoom2D(mask=self.mask)
+
+            buffer = 0 if self.mask.is_all_false else 1
+
+            return zoom.array_2d_from(array=self, buffer=buffer)
+
+        return Array2D(
+            values=self, mask=self.mask, header=self.header, store_native=True
+        )
+
     @property
     def native_skip_mask(self) -> "Array2D":
         """
         Return a `Array2D` where the data is stored in its `native` representation, which is an ``ndarray`` of shape
-        [sub_size*total_y_pixels, sub_size*total_x_pixels].
+        [total_y_pixels, total_x_pixels].
 
         If it is already stored in its `native` representation it is return as it is. If not, it is mapped from
         `slim` to `native` and returned as a new `Array2D`.
@@ -404,32 +330,6 @@ class AbstractArray2D(Structure):
             skip_mask=True,
         )
 
-    @property
-    def binned(self) -> "Array2D":
-        """
-        Convenience method to access the binned-up array in its 1D representation, which is a Grid2D stored as an
-        ``ndarray`` of shape [total_unmasked_pixels, 2].
-
-        The binning up process converts a array from (y,x) values where each value is a coordinate on the sub-array to
-        (y,x) values where each coordinate is at the centre of its mask (e.g. a array with a sub_size of 1). This is
-        performed by taking the mean of all (y,x) values in each sub pixel.
-
-        If the array is stored in 1D it is return as is. If it is stored in 2D, it must first be mapped from 2D to 1D.
-        """
-
-        array_2d_slim = self.slim
-
-        binned_array_1d = np.multiply(
-            self.mask.sub_fraction,
-            array_2d_slim.reshape(-1, self.mask.sub_length).sum(axis=1),
-        )
-
-        return Array2D(
-            values=binned_array_1d,
-            mask=self.mask.derive_mask.sub_1,
-            header=self.header,
-        )
-
     @property
     def in_counts(self) -> "Array2D":
         return self.header.array_eps_to_counts(array_eps=self)
@@ -455,77 +355,122 @@ class AbstractArray2D(Structure):
 
     @property
     def binned_across_rows(self) -> Array1D:
-        binned_array = np.mean(np.ma.masked_array(self.native, self.mask), axis=0)
+        """
+        Bins the 2D array up to a 1D array, where each value is the mean of all unmasked values in each row.
+        """
+        binned_array = np.mean(self.native.array, axis=0, where=~self.mask)
+
+        # binned_array = (self.native * np.invert(self.mask)).sum(axis=0) / np.invert(
+        #     self.mask
+        # ).sum(axis=0)
         return Array1D.no_mask(values=binned_array, pixel_scales=self.pixel_scale)
 
     @property
     def binned_across_columns(self) -> Array1D:
-        binned_array = np.mean(np.ma.masked_array(self.native, self.mask), axis=1)
+        """
+        Bins the 2D array up to a 1D array, where each value is the mean of all unmasked values in each column.
+        """
+        binned_array = np.mean(self.native.array, axis=1, where=~self.mask)
+
+        # binned_array = (self.native*np.invert(self.mask)).sum(axis=1)/np.invert(self.mask).sum(axis=1)
         return Array1D.no_mask(values=binned_array, pixel_scales=self.pixel_scale)
 
-    def zoomed_around_mask(self, buffer: int = 1) -> "Array2D":
+    def brightest_coordinate_in_region_from(
+        self, region: Optional[Tuple[float, float, float, float]]
+    ) -> Tuple[float, float]:
         """
-        Extract the 2D region of an array corresponding to the rectangle encompassing all unmasked values.
+        Returns the brightest pixel in the array inside an input region of form (y0, y1, x0, x1) where
+        the region is in scaled coordinates.
 
-        This is used to extract and visualize only the region of an image that is used in an analysis.
+        For example, if the input region is `region=(-0.15, 0.25, 0.35, 0.55)` the code finds all pixels inside of
+        this region in scaled units, finds the brightest pixel of those pixels and then returns the scaled
+        coordinate location of that brightest pixel.
+
+        The centre of the brightest pixel is returned, the function `brightest_sub_pixel_coordinate_in_region_from`
+        performs a sub pixel calculation to return the brightest sub pixel coordinate.
 
         Parameters
         ----------
-        buffer
-            The number pixels around the extracted array used as a buffer.
+        region
+            The (y0, y1, x0, x1) region in scaled coordinates over which the brightest coordinate is located.
+
+        Returns
+        -------
+        The coordinates of the brightest pixel in scaled units (converted from pixels units).
         """
 
-        extracted_array_2d = array_2d_util.extracted_array_2d_from(
-            array_2d=self.native,
-            y0=self.mask.zoom_region[0] - buffer,
-            y1=self.mask.zoom_region[1] + buffer,
-            x0=self.mask.zoom_region[2] - buffer,
-            x1=self.mask.zoom_region[3] + buffer,
+        y1, y0 = self.geometry.pixel_coordinates_2d_from(
+            scaled_coordinates_2d=(
+                region[0] - self.pixel_scales[0] / 2.0,
+                region[2] + self.pixel_scales[0] / 2.0,
+            )
+        )
+        x0, x1 = self.geometry.pixel_coordinates_2d_from(
+            scaled_coordinates_2d=(
+                region[1] - self.pixel_scales[1] / 2.0,
+                region[3] + self.pixel_scales[1] / 2.0,
+            )
         )
 
-        mask = Mask2D.all_false(
-            shape_native=extracted_array_2d.shape,
-            pixel_scales=self.pixel_scales,
-            sub_size=self.sub_size,
-            origin=self.mask.mask_centre,
+        extracted_region = self.native[y0:y1, x0:x1]
+
+        brightest_pixel_value = np.max(extracted_region)
+        extracted_pixels = np.argwhere(extracted_region == brightest_pixel_value)[0]
+        pixel_coordinates_2d = (y0 + extracted_pixels[0], x0 + extracted_pixels[1])
+
+        return self.geometry.scaled_coordinates_2d_from(
+            pixel_coordinates_2d=pixel_coordinates_2d
         )
 
-        array = array_2d_util.convert_array_2d(
-            array_2d=extracted_array_2d, mask_2d=mask
-        )
-
-        return Array2D(values=array, mask=mask, header=self.header)
-
-    def extent_of_zoomed_array(self, buffer: int = 1) -> np.ndarray:
+    def brightest_sub_pixel_coordinate_in_region_from(
+        self, region: Optional[Tuple[float, float, float, float]], box_size: int = 1
+    ) -> Tuple[float, float]:
         """
-        For an extracted zoomed array computed from the method *zoomed_around_mask* compute its extent in scaled
-        coordinates.
+        Returns the brightest sub-pixel in the array inside an input region of form (y0, y1, x0, x1) where
+        the region is in scaled coordinates.
 
-        The extent of the grid in scaled units returned as an ``ndarray`` of the form [x_min, x_max, y_min, y_max].
+        For example, if the input region is `region=(-0.15, 0.25, 0.35, 0.55)` the code finds all pixels inside of
+        this region in scaled units, finds the brightest pixel of those pixels, and then on a 3x3 grid surrounding
+        this pixel determines the locaiton of the brightest sub pixel using a weighted centre calculation.
 
-        This is used visualize zoomed and extracted arrays via the imshow() method.
+        The centre of the brightest pixel is returned, the function `brightest_sub_pixel_coordinate_in_region_from`
+        performs a sub pixel calculation to return the brightest sub pixel coordinate.
 
         Parameters
         ----------
-        buffer
-            The number pixels around the extracted array used as a buffer.
+        region
+            The (y0, y1, x0, x1) region in scaled coordinates over which the brightest coordinate is located.
+
+        Returns
+        -------
+        The coordinates of the brightest pixel in scaled units (converted from pixels units).
         """
-        extracted_array_2d = array_2d_util.extracted_array_2d_from(
-            array_2d=self.native,
-            y0=self.mask.zoom_region[0] - buffer,
-            y1=self.mask.zoom_region[1] + buffer,
-            x0=self.mask.zoom_region[2] - buffer,
-            x1=self.mask.zoom_region[3] + buffer,
+        brightest_pixel = self.brightest_coordinate_in_region_from(region=region)
+
+        y, x = self.geometry.pixel_coordinates_2d_from(
+            scaled_coordinates_2d=brightest_pixel
+        )
+        region_start_y, region_end_y = max(0, y - box_size), min(
+            self.shape_native[0], y + box_size + 1
+        )
+        region_start_x, region_end_x = max(0, x - box_size), min(
+            self.shape_native[1], x + box_size + 1
+        )
+        region = self.native[region_start_y:region_end_y, region_start_x:region_end_x]
+
+        y_indices, x_indices = np.meshgrid(
+            range(region_start_y, region_end_y),
+            range(region_start_x, region_end_x),
+            indexing="ij",
         )
 
-        mask = Mask2D.all_false(
-            shape_native=extracted_array_2d.shape,
-            pixel_scales=self.pixel_scales,
-            sub_size=self.sub_size,
-            origin=self.mask.mask_centre,
-        )
+        weights = region.flatten()
+        subpixel_y = np.sum(weights * y_indices.flatten()) / np.sum(weights)
+        subpixel_x = np.sum(weights * x_indices.flatten()) / np.sum(weights)
 
-        return mask.geometry.extent
+        return self.geometry.scaled_coordinates_2d_from(
+            pixel_coordinates_2d=(subpixel_y, subpixel_x)
+        )
 
     def resized_from(
         self, new_shape: Tuple[int, int], mask_pad_value: int = 0.0
@@ -546,10 +491,10 @@ class AbstractArray2D(Structure):
         """
 
         resized_array_2d = array_2d_util.resized_array_2d_from(
-            array_2d=self.native, resized_shape=new_shape
+            array_2d=self.native.array, resized_shape=new_shape
         )
 
-        resized_mask = self.mask.derive_mask.resized_from(
+        resized_mask = self.mask.resized_from(
             new_shape=new_shape, pad_value=mask_pad_value
         )
 
@@ -606,13 +551,11 @@ class AbstractArray2D(Structure):
         psf_cut_x = int(np.ceil(kernel_shape[1] / 2)) - 1
         array_y = int(self.mask.shape[0])
         array_x = int(self.mask.shape[1])
-        trimmed_array_2d = self.native[
+        trimmed_array_2d = self.native.array[
             psf_cut_y : array_y - psf_cut_y, psf_cut_x : array_x - psf_cut_x
         ]
 
-        resized_mask = self.mask.derive_mask.resized_from(
-            new_shape=trimmed_array_2d.shape
-        )
+        resized_mask = self.mask.resized_from(new_shape=trimmed_array_2d.shape)
 
         array = array_2d_util.convert_array_2d(
             array_2d=trimmed_array_2d, mask_2d=resized_mask
@@ -625,30 +568,14 @@ class AbstractArray2D(Structure):
             store_native=self.store_native,
         )
 
-    def output_to_fits(self, file_path: str, overwrite: bool = False):
-        """
-        Output the array to a .fits file.
-
-        Parameters
-        ----------
-        file_path
-            The output path of the file, including the filename and the `.fits` extension e.g. '/path/to/filename.fits'
-        overwrite
-            If a file already exists at the path, if overwrite=True it is overwritten else an error is raised.
-        """
-        array_2d_util.numpy_array_2d_to_fits(
-            array_2d=self.native, file_path=file_path, overwrite=overwrite
-        )
-
 
 class Array2D(AbstractArray2D):
     @classmethod
     def no_mask(
         cls,
-        values: Union[np.ndarray, List],
+        values: Union[np.ndarray, List, AbstractArray2D],
         pixel_scales: ty.PixelScales,
         shape_native: Tuple[int, int] = None,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         header: Optional[Header] = None,
     ) -> "Array2D":
@@ -666,16 +593,14 @@ class Array2D(AbstractArray2D):
         Parameters
         ----------
         values
-            The values of the array input with shape [total_unmasked_pixels*(sub_size**2)] or
-            shape [total_y_pixels*sub_size, total_x_pixel*sub_size].
+            The values of the array input with shape [total_unmasked_pixels] or
+            shape [total_y_pixels, total_x_pixels].
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
         shape_native
             The 2D shape of the array in its ``native`` format, and its 2D mask (only required if input shape is
             in ``slim`` format).
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
 
@@ -686,26 +611,12 @@ class Array2D(AbstractArray2D):
 
             import autoarray as aa
 
-            # Make Array2D from input list, native format with sub_size 1
+            # Make Array2D from input list, native format
             # (This array has shape_native=(2,2)).
 
             array_2d = aa.Array2D.manual(
                 array=np.array([[1.0, 2.0], [3.0, 4.0]]),
                 pixel_scales=1.0.
-                sub_size=1
-            )
-
-        .. code-block:: python
-
-            import autoarray as aa
-
-            # Make Array2D from input list, slim format with sub_size 2.
-
-            array_2d = aa.Array2D.no_mask(
-                values=[1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0],
-                shape_native=(2, 1),
-                pixel_scales=1.0,
-                sub_size=2,
             )
         """
 
@@ -714,7 +625,6 @@ class Array2D(AbstractArray2D):
         values = array_2d_util.convert_array(array=values)
 
         if len(values.shape) == 1:
-
             if shape_native is None:
                 raise exc.ArrayException(
                     f"""
@@ -735,16 +645,14 @@ class Array2D(AbstractArray2D):
                 )
 
         else:
-
             shape_native = (
-                int(values.shape[0] / sub_size),
-                int(values.shape[1] / sub_size),
+                int(values.shape[0]),
+                int(values.shape[1]),
             )
 
         mask = Mask2D.all_false(
             shape_native=shape_native,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
         )
 
@@ -756,7 +664,6 @@ class Array2D(AbstractArray2D):
         fill_value: float,
         shape_native: Tuple[int, int],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         header: Optional[Header] = None,
     ) -> "Array2D":
@@ -780,8 +687,6 @@ class Array2D(AbstractArray2D):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
 
@@ -792,36 +697,15 @@ class Array2D(AbstractArray2D):
 
             import autoarray as aa
 
-            # Make Array2D with sub_size 1.
-
             array_2d = aa.Array2D.full(
                 fill_value=2.0,
                 shape_native=(2, 2),
                 pixel_scales=1.0,
-                sub_size=1
             )
-
-        .. code-block:: python
-
-            import autoarray as aa
-
-            # Make Array2D with sub_size 2.
-
-            array_2d = aa.Array2D.full(
-                fill_value=2.0,
-                shape_native=(2, 2),
-                pixel_scales=1.0,
-                sub_size=2
-            )
-
         """
-        if sub_size is not None:
-            shape_native = (shape_native[0] * sub_size, shape_native[1] * sub_size)
-
         return cls.no_mask(
             values=np.full(fill_value=fill_value, shape=shape_native),
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=header,
         )
@@ -831,7 +715,6 @@ class Array2D(AbstractArray2D):
         cls,
         shape_native: Tuple[int, int],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         header: Header = None,
     ) -> "Array2D":
@@ -853,8 +736,6 @@ class Array2D(AbstractArray2D):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
 
@@ -865,31 +746,15 @@ class Array2D(AbstractArray2D):
 
             import autoarray as aa
 
-            # Make Array2D with sub_size 1.
-
             array_2d = aa.Array2D.ones(
                 shape_native=(2, 2),
                 pixel_scales=1.0,
-                sub_size=1
-            )
-
-        .. code-block:: python
-
-            import autoarray as aa
-
-            # Make Array2D with sub_size 2.
-
-            array_2d = aa.Array2D.ones(
-                shape_native=(2, 2),
-                pixel_scales=1.0,
-                sub_size=2
             )
         """
         return cls.full(
             fill_value=1.0,
             shape_native=shape_native,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=header,
         )
@@ -899,7 +764,6 @@ class Array2D(AbstractArray2D):
         cls,
         shape_native: Tuple[int, int],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
         header: Header = None,
     ) -> "Array2D":
@@ -921,8 +785,6 @@ class Array2D(AbstractArray2D):
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the mask's coordinate system.
 
@@ -933,31 +795,15 @@ class Array2D(AbstractArray2D):
 
             import autoarray as aa
 
-            # Make Array2D with sub_size 1.
-
             array_2d = aa.Array2D.zeros(
                 shape_native=(2, 2),
                 pixel_scales=1.0,
-                sub_size=1
-            )
-
-        .. code-block:: python
-
-            import autoarray as aa
-
-            # Make Array2D with sub_size 2.
-
-            array_2d = aa.Array2D.zeros(
-                shape_native=(2, 2),
-                pixel_scales=1.0,
-                sub_size=2
             )
         """
         return cls.full(
             fill_value=0.0,
             shape_native=shape_native,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=header,
         )
@@ -965,10 +811,9 @@ class Array2D(AbstractArray2D):
     @classmethod
     def from_fits(
         cls,
-        file_path: str,
-        pixel_scales: ty.PixelScales,
+        file_path: Union[Path, str],
+        pixel_scales: Optional[ty.PixelScales],
         hdu: int = 0,
-        sub_size: int = 1,
         origin: Tuple[float, float] = (0.0, 0.0),
     ) -> "Array2D":
         """
@@ -988,8 +833,6 @@ class Array2D(AbstractArray2D):
             it is converted to a (float, float) structure.
         hdu
             The Header-Data Unit of the .fits file the array data is loaded from.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-array.
         origin
             The (y,x) scaled units origin of the coordinate system.
 
@@ -1000,41 +843,20 @@ class Array2D(AbstractArray2D):
 
             import autoarray as aa
 
-            # Make Array2D with sub_size 1.
-
             array_2d = aa.Array2D.from_fits(
                 file_path="path/to/file.fits",
                 hdu=0,
                 pixel_scales=1.0,
-                sub_size=1
-            )
-
-        .. code-block:: python
-
-            import autoarray as aa
-
-            # Make Array2D with sub_size 2.
-            # (It is uncommon that a sub-gridded array would be loaded from
-            # a .fits, but the API support its).
-
-            array_2d = aa.Array2D.from_fits(
-                file_path="path/to/file.fits",
-                hdu=0,
-                pixel_scales=1.0,
-                sub_size=2
             )
         """
-        array_2d = array_2d_util.numpy_array_2d_via_fits_from(
-            file_path=file_path, hdu=hdu
-        )
+        array_2d = ndarray_via_fits_from(file_path=file_path, hdu=hdu)
 
-        header_sci_obj = array_2d_util.header_obj_from(file_path=file_path, hdu=0)
-        header_hdu_obj = array_2d_util.header_obj_from(file_path=file_path, hdu=hdu)
+        header_sci_obj = header_obj_from(file_path=file_path, hdu=0)
+        header_hdu_obj = header_obj_from(file_path=file_path, hdu=hdu)
 
         return cls.no_mask(
             values=array_2d,
             pixel_scales=pixel_scales,
-            sub_size=sub_size,
             origin=origin,
             header=Header(header_sci_obj=header_sci_obj, header_hdu_obj=header_hdu_obj),
         )
@@ -1047,11 +869,10 @@ class Array2D(AbstractArray2D):
         values: Union[np.ndarray, List],
         shape_native: Tuple[int, int],
         pixel_scales: ty.PixelScales,
-        sub_size: int = 1,
         header: Header = None,
     ) -> "Array2D":
         """
-        Returns an ``Array2D`` by by inputting the y and x pixel values where the array is filled and the values that
+        Returns an ``Array2D`` by inputting the y and x pixel values where the array is filled and the values that
         fill it.
 
         For a full description of ``Array2D`` objects, including a description of the ``slim`` and ``native`` attribute
@@ -1061,18 +882,16 @@ class Array2D(AbstractArray2D):
         Parameters
         ----------
         y
-            The y pixel indexes where value are input, with shape [total_unmasked_pixels*sub_size].
+            The y pixel indexes where value are input, with shape [total_unmasked_pixels].
         x
-            The x pixel indexes where value are input, with shape [total_unmasked_pixels*sub_size].
+            The x pixel indexes where value are input, with shape [total_unmasked_pixels].
         values or list
-            The values which are used to fill in the array, with shape [total_unmasked_pixels*sub_size].
+            The values which are used to fill in the array, with shape [total_unmasked_pixel].
         shape_native
             The 2D shape of the array in its ``native`` format, and its 2D mask.
         pixel_scales
             The (y,x) scaled units to pixel units conversion factors of every pixel. If this is input as a `float`,
             it is converted to a (float, float) structure.
-        sub_size
-            The size (sub_size x sub_size) of each unmasked pixels sub-grid.
         origin
             The origin of the grid's mask.
 
@@ -1082,30 +901,12 @@ class Array2D(AbstractArray2D):
 
             import autoarray as aa
 
-            # Make Array2D with sub_size 1.
-
             array_2d = aa.Array2D.from_yx_and_values(
                 y=np.array([0.5, 0.5, -0.5, -0.5]),
                 x=np.array([-0.5, 0.5, -0.5, 0.5]),
                 values=np.array([1.0, 2.0, 3.0, 4.0]),
                 shape_native=(2, 2),
                 pixel_scales=1.0,
-                sub_size=1,
-            )
-
-        .. code-block:: python
-
-            import autoarray as aa
-
-            # Make Array2D with sub_size 2.
-
-            array_2d = aa.Array2D.from_yx_and_values(
-                y=np.array([1.0, 1.0. 0.5, 0.5, -0.5, -0.5, -1.0, -1.0]),
-                x=np.array([-0.5, 0.5, -0.5, 0.5, -0.5, 0.5, -0.5, 0.5]),
-                values=np.array([1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0]),
-                shape_native=(2, 1),
-                pixel_scales=1.0,
-                sub_size=2,
             )
         """
         pixel_scales = geometry_util.convert_pixel_scales_2d(pixel_scales=pixel_scales)
@@ -1113,7 +914,10 @@ class Array2D(AbstractArray2D):
         from autoarray.structures.grids.uniform_2d import Grid2D
 
         grid = Grid2D.from_yx_1d(
-            y=y, x=x, shape_native=shape_native, pixel_scales=pixel_scales, sub_size=1
+            y=y,
+            x=x,
+            shape_native=shape_native,
+            pixel_scales=pixel_scales,
         )
 
         grid_pixels = geometry_util.grid_pixel_indexes_2d_slim_from(
@@ -1122,15 +926,13 @@ class Array2D(AbstractArray2D):
             pixel_scales=pixel_scales,
         )
 
-        array_1d = np.zeros(shape=shape_native[0] * shape_native[1])
-
-        for i in range(grid_pixels.shape[0]):
-            array_1d[i] = values[int(grid_pixels[i])]
+        array_1d = np.array(
+            [values[int(grid_pixels[i])] for i in range(grid_pixels.shape[0])]
+        )
 
         return cls.no_mask(
             values=array_1d,
             pixel_scales=pixel_scales,
             shape_native=shape_native,
-            sub_size=sub_size,
             header=header,
         )
