@@ -1,5 +1,7 @@
 from __future__ import annotations
 import numpy as np
+import jax.numpy as jnp
+
 from typing import TYPE_CHECKING, List, Optional, Tuple, Union
 
 if TYPE_CHECKING:
@@ -8,13 +10,23 @@ if TYPE_CHECKING:
 from autoarray import exc
 from autoarray.structures.arrays import array_2d_util
 from autoarray.geometry import geometry_util
-from autoarray import numba_util
-from autoarray.mask import mask_2d_util
 from autoarray import type as ty
 
 
-def check_grid_slim(grid, shape_native):
+def convert_grid(grid: Union[np.ndarray, List]) -> np.ndarray:
 
+    try:
+        grid = grid.array
+    except AttributeError:
+        pass
+
+    if isinstance(grid, list):
+        grid = np.asarray(grid)
+
+    return grid
+
+
+def check_grid_slim(grid, shape_native):
     if shape_native is None:
         raise exc.GridException(
             f"""
@@ -35,14 +47,6 @@ def check_grid_slim(grid, shape_native):
         )
 
 
-def convert_grid(grid: Union[np.ndarray, List]) -> np.ndarray:
-
-    if type(grid) is list:
-        grid = np.asarray(grid)
-
-    return grid
-
-
 def check_grid_2d(grid_2d: np.ndarray):
     if grid_2d.shape[-1] != 2:
         raise exc.GridException(
@@ -54,33 +58,28 @@ def check_grid_2d(grid_2d: np.ndarray):
 
 
 def check_grid_2d_and_mask_2d(grid_2d: np.ndarray, mask_2d: Mask2D):
-
     if len(grid_2d.shape) == 2:
-
-        if grid_2d.shape[0] != mask_2d.sub_pixels_in_mask:
+        if grid_2d.shape[0] != mask_2d.pixels_in_mask:
             raise exc.GridException(
                 f"""
-                The input 2D grid does not have the same number of values as sub-pixels in
+                The input 2D grid does not have the same number of values as pixels in
                 the mask.
-                
+
                 The shape of the input grid_2d is {grid_2d.shape}.
                 The mask shape_native is {mask_2d.shape_native}.
-                The mask number of sub-pixels is {mask_2d.sub_pixels_in_mask}. 
-                The mask sub size is {mask_2d.sub_size}).
+                The mask number of pixels is {mask_2d.pixels_in_mask}. 
                 """
             )
 
     elif len(grid_2d.shape) == 3:
-
-        if (grid_2d.shape[0], grid_2d.shape[1]) != mask_2d.sub_shape_native:
+        if (grid_2d.shape[0], grid_2d.shape[1]) != mask_2d.shape_native:
             raise exc.GridException(
                 f"""
-                The input 2D grid is not the same dimensions as the sub-mask
-                (e.g. the mask 2D shape multipled by its sub size.)
+                The input 2D grid is not the same dimensions as the mask
+                (e.g. the mask 2D shape.)
 
                 The shape of the input grid_2d is {grid_2d.shape}.
-                The sub_shape_native of the mask is {mask_2d.sub_shape_native} (the 
-                mask sub size is {mask_2d.sub_size}).
+                The mask shape_native is {mask_2d.shape_native}.
                 """
             )
 
@@ -94,7 +93,7 @@ def convert_grid_2d(
     This function performs the following and checks and conversions on the input:
 
     1: If the input is a list, convert it to an ndarray.
-    2: Check that the number of sub-pixels in the array is identical to that of the mask.
+    2: Check that the number of coordinates in the grid is identical to that of the mask.
     3) Map the input ndarray to its `slim` representation.
 
     For a Grid2D, `slim` refers to a 2D NumPy array of shape [total_coordinates, 2] and `native` a 3D NumPy array of
@@ -103,7 +102,7 @@ def convert_grid_2d(
     Parameters
     ----------
     grid_2d
-        The input (y,x) grid of coordinates which is converted to an ndarray if it is a list.
+        The input (y,x) grid of coordinates which is converted to a ndarray if it is a list.
     mask_2d
         The mask of the output Array2D.
     store_native
@@ -113,23 +112,33 @@ def convert_grid_2d(
 
     grid_2d = convert_grid(grid=grid_2d)
 
+    is_numpy = True if isinstance(grid_2d, np.ndarray) else False
+
     check_grid_2d_and_mask_2d(grid_2d=grid_2d, mask_2d=mask_2d)
 
     is_native = len(grid_2d.shape) == 3
 
     if is_native:
-        grid_2d[:, :, 0] *= np.invert(mask_2d.derive_mask.sub)
-        grid_2d[:, :, 1] *= np.invert(mask_2d.derive_mask.sub)
+        if not is_numpy:
+            grid_2d = grid_2d.at[:, :, 0].multiply(~mask_2d)
+            grid_2d = grid_2d.at[:, :, 1].multiply(~mask_2d)
+        else:
+            grid_2d[:, :, 0] *= ~mask_2d
+            grid_2d[:, :, 1] *= ~mask_2d
 
     if is_native == store_native:
-        return grid_2d
+        grid_2d = grid_2d
     elif not store_native:
-        return grid_2d_slim_from(
-            grid_2d_native=grid_2d, mask=mask_2d, sub_size=mask_2d.sub_size
+        grid_2d = grid_2d_slim_from(
+            grid_2d_native=grid_2d,
+            mask=mask_2d,
         )
-    return grid_2d_native_from(
-        grid_2d_slim=grid_2d, mask_2d=mask_2d, sub_size=mask_2d.sub_size
-    )
+    else:
+        grid_2d = grid_2d_native_from(
+            grid_2d_slim=grid_2d,
+            mask_2d=mask_2d,
+        )
+    return np.array(grid_2d) if is_numpy else jnp.array(grid_2d)
 
 
 def convert_grid_2d_to_slim(
@@ -152,7 +161,8 @@ def convert_grid_2d_to_slim(
     if len(grid_2d.shape) == 2:
         return grid_2d
     return grid_2d_slim_from(
-        grid_2d_native=grid_2d, mask=mask_2d, sub_size=mask_2d.sub_size
+        grid_2d_native=grid_2d,
+        mask=mask_2d,
     )
 
 
@@ -176,11 +186,11 @@ def convert_grid_2d_to_native(
     if len(grid_2d.shape) == 3:
         return grid_2d
     return grid_2d_native_from(
-        grid_2d_slim=grid_2d, mask_2d=mask_2d, sub_size=mask_2d.sub_size
+        grid_2d_slim=grid_2d,
+        mask_2d=mask_2d,
     )
 
 
-@numba_util.jit()
 def grid_2d_centre_from(grid_2d_slim: np.ndarray) -> Tuple[float, float]:
     """
     Returns the centre of a grid from a 1D grid.
@@ -200,190 +210,161 @@ def grid_2d_centre_from(grid_2d_slim: np.ndarray) -> Tuple[float, float]:
     return centre_y, centre_x
 
 
-@numba_util.jit()
 def grid_2d_slim_via_mask_from(
     mask_2d: np.ndarray,
     pixel_scales: ty.PixelScales,
-    sub_size: int,
     origin: Tuple[float, float] = (0.0, 0.0),
 ) -> np.ndarray:
     """
-    For a sub-grid, every unmasked pixel of its 2D mask with shape (total_y_pixels, total_x_pixels) is divided into
-    a finer uniform grid of shape (total_y_pixels*sub_size, total_x_pixels*sub_size). This routine computes the (y,x)
-    scaled coordinates a the centre of every sub-pixel defined by this 2D mask array.
+    For a grid, every unmasked pixel is on a 2D mask with shape (total_y_pixels, total_x_pixels). This routine
+    computes the (y,x) scaled coordinates a the centre of every pixel defined by this 2D mask array.
 
-    The sub-grid is returned on an array of shape (total_unmasked_pixels*sub_size**2, 2). y coordinates are
-    stored in the 0 index of the second dimension, x coordinates in the 1 index. Masked coordinates are therefore
-    removed and not included in the slimmed grid.
+    The grid is returned on an array of shape (total_unmasked_pixels, 2). y coordinates are stored in the 0 index of
+    the second dimension, x coordinates in the 1 index. Masked coordinates are therefore removed and not included in
+    the slimmed grid.
 
-    Grid2D are defined from the top-left corner, where the first unmasked sub-pixel corresponds to index 0.
-    Sub-pixels that are part of the same mask array pixel are indexed next to one another, such that the second
-    sub-pixel in the first pixel has index 1, its next sub-pixel has index 2, and so forth.
+    Grid2D are defined from the top-left corner, where the first unmasked pixel corresponds to index 0.
 
     Parameters
     ----------
     mask_2d
         A 2D array of bools, where `False` values are unmasked and therefore included as part of the calculated
-        sub-grid.
+        grid.
     pixel_scales
         The (y,x) scaled units to pixel units conversion factor of the 2D mask array.
-    sub_size
-        The size of the sub-grid that each pixel of the 2D mask array is divided into.
     origin
-        The (y,x) origin of the 2D array, which the sub-grid is shifted around.
+        The (y,x) origin of the 2D array, which the grid is shifted around.
 
     Returns
     -------
     ndarray
-        A slimmed sub grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
-        array. The sub grid array has dimensions (total_unmasked_pixels*sub_size**2, 2).
+        A slimmed grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
+        array. The grid array has dimensions (total_unmasked_pixels, 2).
 
     Examples
     --------
     mask = np.array([[True, False, True],
                      [False, False, False]
                      [True, False, True]])
-    grid_slim = grid_2d_slim_via_mask_from(mask=mask, pixel_scales=(0.5, 0.5), sub_size=1, origin=(0.0, 0.0))
+    grid_slim = grid_2d_slim_via_mask_from(mask=mask, pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
     """
 
-    total_sub_pixels = mask_2d_util.total_sub_pixels_2d_from(mask_2d, sub_size)
-
-    grid_slim = np.zeros(shape=(total_sub_pixels, 2))
-
     centres_scaled = geometry_util.central_scaled_coordinate_2d_from(
         shape_native=mask_2d.shape, pixel_scales=pixel_scales, origin=origin
     )
 
-    sub_index = 0
+    # JAX branch
+    if isinstance(mask_2d, jnp.ndarray):
+        centres_scaled = jnp.asarray(centres_scaled)
+        pixel_scales = jnp.asarray(pixel_scales)
+        sign = jnp.array([-1.0, 1.0])
 
-    y_sub_half = pixel_scales[0] / 2
-    y_sub_step = pixel_scales[0] / (sub_size)
+        # use jnp.where instead of jnp.nonzero
+        rows, cols = jnp.where(~mask_2d.astype(bool))
+        indices = jnp.stack([rows, cols], axis=1)  # shape (N_unmasked, 2)
 
-    x_sub_half = pixel_scales[1] / 2
-    x_sub_step = pixel_scales[1] / (sub_size)
+        # (indices - centre) -> pixel offsets; apply sign and scale to get physical coords
+        return (indices - centres_scaled) * sign * pixel_scales
 
-    for y in range(mask_2d.shape[0]):
-        for x in range(mask_2d.shape[1]):
+    # NumPy branch (kept consistent)
+    centres_scaled = np.asarray(centres_scaled)
+    pixel_scales = np.asarray(pixel_scales)
+    sign = np.array([-1.0, 1.0])
 
-            if not mask_2d[y, x]:
+    rows, cols = np.where(~mask_2d.astype(bool))
+    indices = np.stack([rows, cols], axis=1)
 
-                y_scaled = (y - centres_scaled[0]) * pixel_scales[0]
-                x_scaled = (x - centres_scaled[1]) * pixel_scales[1]
-
-                for y1 in range(sub_size):
-                    for x1 in range(sub_size):
-
-                        grid_slim[sub_index, 0] = -(
-                            y_scaled - y_sub_half + y1 * y_sub_step + (y_sub_step / 2.0)
-                        )
-                        grid_slim[sub_index, 1] = (
-                            x_scaled - x_sub_half + x1 * x_sub_step + (x_sub_step / 2.0)
-                        )
-                        sub_index += 1
-
-    return grid_slim
+    return (indices - centres_scaled) * sign * pixel_scales
 
 
 def grid_2d_via_mask_from(
     mask_2d: np.ndarray,
     pixel_scales: ty.PixelScales,
-    sub_size: int,
     origin: Tuple[float, float] = (0.0, 0.0),
 ) -> np.ndarray:
     """
-    For a sub-grid, every unmasked pixel of its 2D mask with shape (total_y_pixels, total_x_pixels) is divided into a
-    finer uniform grid of shape (total_y_pixels*sub_size, total_x_pixels*sub_size). This routine computes the (y,x)
-    scaled coordinates at the centre of every sub-pixel defined by this 2D mask array.
+    For a grid, every unmasked pixel is on a 2D mask with shape (total_y_pixels, total_x_pixels). This routine computes
+    the (y,x) scaled coordinates at the centre of every pixel defined by this 2D mask array.
 
-    The sub-grid is returned in its native dimensions with shape (total_y_pixels*sub_size, total_x_pixels*sub_size).
-    y coordinates are stored in the 0 index of the second dimension, x coordinates in the 1 index. Masked pixels are
-    given values (0.0, 0.0).
+    The grid is returned in its native dimensions with shape (total_y_pixels, total_x_pixels). y coordinates are
+    stored in the 0 index of the second dimension, x coordinates in the 1 index. Masked pixels are given
+    values (0.0, 0.0).
 
-    Grids are defined from the top-left corner, where the first unmasked sub-pixel corresponds to index 0.
-    Sub-pixels that are part of the same mask array pixel are indexed next to one another, such that the second
-    sub-pixel in the first pixel has index 1, its next sub-pixel has index 2, and so forth.
+    Grids are defined from the top-left corner, where the first unmasked pixel corresponds to index 0.
 
     Parameters
     ----------
     mask_2d
         A 2D array of bools, where `False` values are unmasked and therefore included as part of the calculated
-        sub-grid.
+        grid.
     pixel_scales
         The (y,x) scaled units to pixel units conversion factor of the 2D mask array.
-    sub_size
-        The size of the sub-grid that each pixel of the 2D mask array is divided into.
     origin
-        The (y,x) origin of the 2D array, which the sub-grid is shifted around.
+        The (y,x) origin of the 2D array, which the grid is shifted around.
 
     Returns
     -------
     ndarray
-        A sub grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
-        array. The sub grid array has dimensions (total_y_pixels*sub_size, total_x_pixels*sub_size).
+        A grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
+        array. The grid array has dimensions (total_y_pixels, total_x_pixels).
 
     Examples
     --------
     mask = np.array([[True, False, True],
                      [False, False, False]
                      [True, False, True]])
-    grid_2d = grid_2d_via_mask_from(mask=mask, pixel_scales=(0.5, 0.5), sub_size=1, origin=(0.0, 0.0))
+    grid_2d = grid_2d_via_mask_from(mask=mask, pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
     """
 
     grid_2d_slim = grid_2d_slim_via_mask_from(
-        mask_2d=mask_2d, pixel_scales=pixel_scales, sub_size=sub_size, origin=origin
+        mask_2d=mask_2d, pixel_scales=pixel_scales, origin=origin
     )
 
     return grid_2d_native_from(
-        grid_2d_slim=grid_2d_slim, mask_2d=mask_2d, sub_size=sub_size
+        grid_2d_slim=grid_2d_slim,
+        mask_2d=mask_2d,
     )
 
 
 def grid_2d_slim_via_shape_native_from(
     shape_native: Tuple[int, int],
     pixel_scales: ty.PixelScales,
-    sub_size: int,
     origin: Tuple[float, float] = (0.0, 0.0),
 ) -> np.ndarray:
     """
-    For a sub-grid, every unmasked pixel of its 2D mask with shape (total_y_pixels, total_x_pixels) is divided into a
-    finer uniform grid of shape (total_y_pixels*sub_size, total_x_pixels*sub_size). This routine computes the (y,x)
-    scaled coordinates at the centre of every sub-pixel defined by this 2D mask array.
+    For a grid, every unmasked pixel is in a 2D mask with shape (total_y_pixels, total_x_pixels). This routine computes
+    the (y,x) scaled coordinates at the centre of every pixel defined by this 2D mask array.
 
-    The sub-grid is returned in its slimmed dimensions with shape (total_pixels**2*sub_size**2, 2). y coordinates are
+    The grid is returned in its slimmed dimensions with shape (total_pixels, 2). y coordinates are
     stored in the 0 index of the second dimension, x coordinates in the 1 index.
 
-    Grid2D are defined from the top-left corner, where the first sub-pixel corresponds to index [0,0].
-    Sub-pixels that are part of the same mask array pixel are indexed next to one another, such that the second
-    sub-pixel in the first pixel has index 1, its next sub-pixel has index 2, and so forth.
+    Grid2D are defined from the top-left corner, where the first pixel corresponds to index [0,0].
 
     Parameters
     ----------
     shape_native
-        The (y,x) shape of the 2D array the sub-grid of coordinates is computed for.
+        The (y,x) shape of the 2D array the grid of coordinates is computed for.
     pixel_scales
         The (y,x) scaled units to pixel units conversion factor of the 2D mask array.
-    sub_size
-        The size of the sub-grid that each pixel of the 2D mask array is divided into.
     origin
-        The (y,x) origin of the 2D array, which the sub-grid is shifted around.
+        The (y,x) origin of the 2D array, which the grid is shifted around.
 
     Returns
     -------
     ndarray
-        A sub grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
-        array. The sub grid is slimmed and has dimensions (total_unmasked_pixels*sub_size**2, 2).
+        A grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
+        array. The grid is slimmed and has dimensions (total_unmasked_pixels, 2).
 
     Examples
     --------
     mask = np.array([[True, False, True],
                      [False, False, False]
                      [True, False, True]])
-    grid_2d_slim = grid_2d_slim_via_shape_native_from(shape_native=(3,3), pixel_scales=(0.5, 0.5), sub_size=2, origin=(0.0, 0.0))
+    grid_2d_slim = grid_2d_slim_via_shape_native_from(shape_native=(3,3), pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
     """
     return grid_2d_slim_via_mask_from(
         mask_2d=np.full(fill_value=False, shape=shape_native),
         pixel_scales=pixel_scales,
-        sub_size=sub_size,
         origin=origin,
     )
 
@@ -391,56 +372,47 @@ def grid_2d_slim_via_shape_native_from(
 def grid_2d_via_shape_native_from(
     shape_native: Tuple[int, int],
     pixel_scales: ty.PixelScales,
-    sub_size: int,
     origin: Tuple[float, float] = (0.0, 0.0),
 ) -> np.ndarray:
     """
-    For a sub-grid, every unmasked pixel of its 2D mask with shape (total_y_pixels, total_x_pixels) is divided
-    into a finer uniform grid of shape (total_y_pixels*sub_size, total_x_pixels*sub_size). This routine computes
-    the (y,x) scaled coordinates at the centre of every sub-pixel defined by this 2D mask array.
+    For a grid, every unmasked pixel is in a 2D mask with shape (total_y_pixels, total_x_pixels). This routine computes
+    the (y,x) scaled coordinates at the centre of every pixel defined by this 2D mask array.
 
-    The sub-grid is returned in its native dimensions with shape (total_y_pixels*sub_size, total_x_pixels*sub_size).
+    The grid is returned in its native dimensions with shape (total_y_pixels, total_x_pixels).
     y coordinates are stored in the 0 index of the second dimension, x coordinates in the 1 index.
 
-    Grids are defined from the top-left corner, where the first sub-pixel corresponds to index [0,0].
-    Sub-pixels that are part of the same mask array pixel are indexed next to one another, such that the second
-    sub-pixel in the first pixel has index 1, its next sub-pixel has index 2, and so forth.
+    Grids are defined from the top-left corner, where the first pixel corresponds to index [0,0].
 
     Parameters
     ----------
     shape_native
-        The (y,x) shape of the 2D array the sub-grid of coordinates is computed for.
+        The (y,x) shape of the 2D array the grid of coordinates is computed for.
     pixel_scales
         The (y,x) scaled units to pixel units conversion factor of the 2D mask array.
-    sub_size
-        The size of the sub-grid that each pixel of the 2D mask array is divided into.
     origin
-        The (y,x) origin of the 2D array, which the sub-grid is shifted around.
+        The (y,x) origin of the 2D array, which the grid is shifted around.
 
     Returns
     -------
     ndarray
-        A sub grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
-        array. The sub grid array has dimensions (total_y_pixels*sub_size, total_x_pixels*sub_size).
+        A grid of (y,x) scaled coordinates at the centre of every pixel unmasked pixel on the 2D mask
+        array. The grid array has dimensions (total_y_pixels, total_x_pixels).
 
     Examples
     --------
-    grid_2d = grid_2d_via_shape_native_from(shape_native=(3, 3), pixel_scales=(1.0, 1.0), sub_size=2, origin=(0.0, 0.0))
+    grid_2d = grid_2d_via_shape_native_from(shape_native=(3, 3), pixel_scales=(1.0, 1.0), origin=(0.0, 0.0))
     """
     return grid_2d_via_mask_from(
         mask_2d=np.full(fill_value=False, shape=shape_native),
         pixel_scales=pixel_scales,
-        sub_size=sub_size,
         origin=origin,
     )
 
 
-@numba_util.jit()
 def _radial_projected_shape_slim_from(
     extent: np.ndarray,
     centre: Tuple[float, float],
     pixel_scales: ty.PixelScales,
-    sub_size: int,
 ) -> int:
     """
     The function `grid_scaled_2d_slim_radial_projected_from()` determines a projected radial grid of points from a 2D
@@ -479,8 +451,6 @@ def _radial_projected_shape_slim_from(
         The (y,x) central coordinate which the radial grid is traced outwards from.
     pixel_scales
         The (y,x) scaled units to pixel units conversion factor of the 2D mask array.
-    sub_size
-        The size of the sub-grid that each pixel of the 2D mask array is divided into.
 
     Returns
     -------
@@ -510,15 +480,13 @@ def _radial_projected_shape_slim_from(
     else:
         pixel_scale = pixel_scales[1]
 
-    return sub_size * int((scaled_distance / pixel_scale)) + 1
+    return int((scaled_distance / pixel_scale)) + 1
 
 
-@numba_util.jit()
 def grid_scaled_2d_slim_radial_projected_from(
     extent: np.ndarray,
     centre: Tuple[float, float],
     pixel_scales: ty.PixelScales,
-    sub_size: int,
     shape_slim: Optional[int] = 0,
 ) -> np.ndarray:
     """
@@ -564,11 +532,9 @@ def grid_scaled_2d_slim_radial_projected_from(
         The (y,x) central coordinate which the radial grid is traced outwards from.
     pixel_scales
         The (y,x) scaled units to pixel units conversion factor of the 2D mask array.
-    sub_size
-        The size of the sub-grid that each pixel of the 2D mask array is divided into.
     shape_slim
         Manually choose the shape of the 1D projected grid that is returned. If 0, the border based on the 2D grid is
-        used (due to numba None cannot be used as a default value).
+        used.
 
     Returns
     -------
@@ -599,7 +565,7 @@ def grid_scaled_2d_slim_radial_projected_from(
         pixel_scale = pixel_scales[1]
 
     if shape_slim == 0:
-        shape_slim = sub_size * int((scaled_distance / pixel_scale)) + 1
+        shape_slim = int((scaled_distance / pixel_scale)) + 1
 
     grid_scaled_2d_slim_radii = np.zeros((shape_slim, 2))
 
@@ -607,107 +573,18 @@ def grid_scaled_2d_slim_radial_projected_from(
 
     radii = centre[1]
 
-    for slim_index in range(shape_slim):
+    # Create an array of radii values spaced by pixel_scale
+    radii_array = radii + pixel_scale * np.arange(shape_slim)
 
-        grid_scaled_2d_slim_radii[slim_index, 1] = radii
-        radii += pixel_scale / sub_size
+    # Assign all values at once to the second column (index 1)
+    grid_scaled_2d_slim_radii[:, 1] = radii_array
 
-    return grid_scaled_2d_slim_radii
-
-
-@numba_util.jit()
-def relocated_grid_via_jit_from(grid, border_grid):
-    """
-    Relocate the coordinates of a grid to its border if they are outside the border, where the border is
-    defined as all pixels at the edge of the grid's mask (see *mask._border_1d_indexes*).
-
-    This is performed as follows:
-
-    1: Use the mean value of the grid's y and x coordinates to determine the origin of the grid.
-    2: Compute the radial distance of every grid coordinate from the origin.
-    3: For every coordinate, find its nearest pixel in the border.
-    4: Determine if it is outside the border, by comparing its radial distance from the origin to its paired
-    border pixel's radial distance.
-    5: If its radial distance is larger, use the ratio of radial distances to move the coordinate to the
-    border (if its inside the border, do nothing).
-
-    The method can be used on uniform or irregular grids, however for irregular grids the border of the
-    'image-plane' mask is used to define border pixels.
-
-    Parameters
-    ----------
-    grid : Grid2D
-        The grid (uniform or irregular) whose pixels are to be relocated to the border edge if outside it.
-    border_grid : Grid2D
-        The grid of border (y,x) coordinates.
-    """
-
-    grid_relocated = np.zeros(grid.shape)
-    grid_relocated[:, :] = grid[:, :]
-
-    border_origin = np.zeros(2)
-    border_origin[0] = np.mean(border_grid[:, 0])
-    border_origin[1] = np.mean(border_grid[:, 1])
-    border_grid_radii = np.sqrt(
-        np.add(
-            np.square(np.subtract(border_grid[:, 0], border_origin[0])),
-            np.square(np.subtract(border_grid[:, 1], border_origin[1])),
-        )
-    )
-    border_min_radii = np.min(border_grid_radii)
-
-    grid_radii = np.sqrt(
-        np.add(
-            np.square(np.subtract(grid[:, 0], border_origin[0])),
-            np.square(np.subtract(grid[:, 1], border_origin[1])),
-        )
-    )
-
-    for pixel_index in range(grid.shape[0]):
-
-        if grid_radii[pixel_index] > border_min_radii:
-
-            closest_pixel_index = np.argmin(
-                np.square(grid[pixel_index, 0] - border_grid[:, 0])
-                + np.square(grid[pixel_index, 1] - border_grid[:, 1])
-            )
-
-            move_factor = (
-                border_grid_radii[closest_pixel_index] / grid_radii[pixel_index]
-            )
-
-            if move_factor < 1.0:
-
-                grid_relocated[pixel_index, :] = (
-                    move_factor * (grid[pixel_index, :] - border_origin[:])
-                    + border_origin[:]
-                )
-
-    return grid_relocated
-
-
-@numba_util.jit()
-def furthest_grid_2d_slim_index_from(
-    grid_2d_slim: np.ndarray, slim_indexes: np.ndarray, coordinate: Tuple[float, float]
-) -> int:
-
-    distance_to_centre = 0.0
-
-    for slim_index in slim_indexes:
-
-        y = grid_2d_slim[slim_index, 0]
-        x = grid_2d_slim[slim_index, 1]
-        distance_to_centre_new = (x - coordinate[1]) ** 2 + (y - coordinate[0]) ** 2
-
-        if distance_to_centre_new >= distance_to_centre:
-            distance_to_centre = distance_to_centre_new
-            furthest_grid_2d_slim_index = slim_index
-
-    return furthest_grid_2d_slim_index
+    return grid_scaled_2d_slim_radii + 1e-6
 
 
 def grid_2d_slim_from(
-    grid_2d_native: np.ndarray, mask: np.ndarray, sub_size: int
+    grid_2d_native: np.ndarray,
+    mask: np.ndarray,
 ) -> np.ndarray:
     """
     For a native 2D grid and mask of shape [total_y_pixels, total_x_pixels, 2], map the values of all unmasked
@@ -726,8 +603,6 @@ def grid_2d_slim_from(
         The native grid of (y,x) values which are mapped to the slimmed grid.
     mask_2d
         A 2D array of bools, where `False` values mean unmasked and are included in the mapping.
-    sub_size
-        The size (sub_size x sub_size) of each unmasked pixels sub-array.
 
     Returns
     -------
@@ -736,18 +611,22 @@ def grid_2d_slim_from(
     """
 
     grid_1d_slim_y = array_2d_util.array_2d_slim_from(
-        array_2d_native=grid_2d_native[:, :, 0], mask_2d=mask, sub_size=sub_size
+        array_2d_native=grid_2d_native[:, :, 0],
+        mask_2d=mask,
     )
 
     grid_1d_slim_x = array_2d_util.array_2d_slim_from(
-        array_2d_native=grid_2d_native[:, :, 1], mask_2d=mask, sub_size=sub_size
+        array_2d_native=grid_2d_native[:, :, 1],
+        mask_2d=mask,
     )
-
-    return np.stack((grid_1d_slim_y, grid_1d_slim_x), axis=-1)
+    if isinstance(grid_2d_native, np.ndarray):
+        return np.stack((grid_1d_slim_y, grid_1d_slim_x), axis=-1)
+    return jnp.stack((grid_1d_slim_y, grid_1d_slim_x), axis=-1)
 
 
 def grid_2d_native_from(
-    grid_2d_slim: np.ndarray, mask_2d: np.ndarray, sub_size: int
+    grid_2d_slim: np.ndarray,
+    mask_2d: np.ndarray,
 ) -> np.ndarray:
     """
     For a slimmed 2D grid of shape [total_unmasked_pixels, 2], that was computed by extracting the unmasked values
@@ -767,8 +646,6 @@ def grid_2d_native_from(
         The (y,x) values of the slimmed 2D grid which are mapped to the native 2D grid.
     mask_2d
         A 2D array of bools, where `False` values mean unmasked and are included in the mapping.
-    sub_size
-        The size (sub_size x sub_size) of each unmasked pixels sub-array.
 
     Returns
     -------
@@ -778,70 +655,18 @@ def grid_2d_native_from(
     """
 
     grid_2d_native_y = array_2d_util.array_2d_native_from(
-        array_2d_slim=grid_2d_slim[:, 0], mask_2d=mask_2d, sub_size=sub_size
+        array_2d_slim=grid_2d_slim[:, 0],
+        mask_2d=mask_2d,
     )
 
     grid_2d_native_x = array_2d_util.array_2d_native_from(
-        array_2d_slim=grid_2d_slim[:, 1], mask_2d=mask_2d, sub_size=sub_size
+        array_2d_slim=grid_2d_slim[:, 1],
+        mask_2d=mask_2d,
     )
 
-    return np.stack((grid_2d_native_y, grid_2d_native_x), axis=-1)
-
-
-@numba_util.jit()
-def grid_2d_slim_upscaled_from(
-    grid_slim: np.ndarray, upscale_factor: int, pixel_scales: ty.PixelScales
-) -> np.ndarray:
-    """
-    From an input slimmed 2D grid, return an upscaled slimmed 2D grid where (y,x) coordinates are added at an
-    upscaled resolution to each grid coordinate, analogous to a sub-grid.
-
-    Parameters
-    ----------
-    grid_slim
-        The slimmed grid of (y,x) coordinates over which a square uniform grid is overlaid.
-    upscale_factor
-        The upscaled resolution at which the new grid coordinates are computed.
-    pixel_scales
-        The pixel scale of the uniform grid that laid over the irregular grid of (y,x) coordinates.
-    """
-
-    grid_2d_slim_upscaled = np.zeros(
-        shape=(grid_slim.shape[0] * upscale_factor**2, 2)
-    )
-
-    upscale_index = 0
-
-    y_upscale_half = pixel_scales[0] / 2
-    y_upscale_step = pixel_scales[0] / upscale_factor
-
-    x_upscale_half = pixel_scales[1] / 2
-    x_upscale_step = pixel_scales[1] / upscale_factor
-
-    for slim_index in range(grid_slim.shape[0]):
-
-        y_grid = grid_slim[slim_index, 0]
-        x_grid = grid_slim[slim_index, 1]
-
-        for y in range(upscale_factor):
-            for x in range(upscale_factor):
-
-                grid_2d_slim_upscaled[upscale_index, 0] = (
-                    y_grid
-                    + y_upscale_half
-                    - y * y_upscale_step
-                    - (y_upscale_step / 2.0)
-                )
-                grid_2d_slim_upscaled[upscale_index, 1] = (
-                    x_grid
-                    - x_upscale_half
-                    + x * x_upscale_step
-                    + (x_upscale_step / 2.0)
-                )
-
-                upscale_index += 1
-
-    return grid_2d_slim_upscaled
+    if isinstance(grid_2d_slim, np.ndarray):
+        return np.stack((grid_2d_native_y, grid_2d_native_x), axis=-1)
+    return jnp.stack((grid_2d_native_y, grid_2d_native_x), axis=-1)
 
 
 def grid_2d_of_points_within_radius(
@@ -861,8 +686,99 @@ def grid_2d_of_points_within_radius(
 
 
 def compute_polygon_area(points):
-
     x = points[:, 1]
     y = points[:, 0]
 
     return 0.5 * np.abs(np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1)))
+
+
+def grid_pixels_in_mask_pixels_from(
+    grid, shape_native, pixel_scales, origin
+) -> np.ndarray:
+    """
+    Returns an array containing the number of pixels of one grid in every pixel of another masked grid.
+
+    For example, image-mesh adaption may be performed on a 3.0" circular mask of data. The high weight pixels
+    may have 3 or more mesh pixels per image pixel, whereas low weight regions may have zero pixels. The array
+    returned by this function gives the integer number of pixels in each data pixel.
+
+    Parameters
+    ----------
+    grid_pixel_centres
+        The 2D integer index of every image pixel that each image-mesh pixel falls within.
+    shape_native
+        The 2D shape of the data's mask, which the number of image-mesh pixels that fall within eac pixel is counted.
+
+    Returns
+    -------
+    An array containing the integer number of image-mesh pixels that fall without each of the data's mask.
+    """
+    grid_pixel_centres = geometry_util.grid_pixel_centres_2d_slim_from(
+        grid_scaled_2d_slim=grid,
+        shape_native=shape_native,
+        pixel_scales=pixel_scales,
+        origin=origin,
+    ).astype("int")
+
+    mesh_pixels_per_image_pixel = np.zeros(shape=shape_native)
+
+    # Assuming grid_pixel_centres is a 2D array where each row contains (y, x) indices.
+    y_indices = grid_pixel_centres[:, 0]
+    x_indices = grid_pixel_centres[:, 1]
+
+    # Use np.add.at to increment the specific indices in a safe and efficient manner
+    np.add.at(mesh_pixels_per_image_pixel, (y_indices, x_indices), 1)
+
+    return mesh_pixels_per_image_pixel
+
+
+def grid_2d_slim_via_shape_native_not_mask_from(
+    shape_native: Tuple[int, int],
+    pixel_scales: Tuple[float, float],
+    origin: Tuple[float, float] = (0.0, 0.0),
+) -> np.ndarray:
+    """
+    Build the slim (flattened) grid of all (y, x) pixel centres for a rectangular grid
+    of shape `shape_native`, scaled by `pixel_scales` and shifted by `origin`.
+
+    This is equivalent to taking an unmasked mask of shape `shape_native` and calling
+    grid_2d_slim_via_mask_from on it.
+
+    Parameters
+    ----------
+    shape_native
+        A pair (Ny, Nx) giving the number of pixels in y and x.
+    pixel_scales
+        A pair (sy, sx) giving the physical size of each pixel in y and x.
+    origin
+        A 2-tuple (y0, x0) around which the grid is centred.
+
+    Returns
+    -------
+    grid_slim : ndarray, shape (Ny*Nx, 2)
+        Each row is the (y, x) coordinate of one pixel centre, in row-major order,
+        shifted so that `origin` ↔ physical pixel-centre average, and scaled by
+        `pixel_scales`, with y increasing “up” and x increasing “right”.
+    """
+    Ny, Nx = shape_native
+    sy, sx = pixel_scales
+    y0, x0 = origin
+
+    # compute the integer pixel‐centre coordinates in array index space
+    # row indices 0..Ny-1, col indices 0..Nx-1
+    arange = jnp.arange
+    meshy, meshx = jnp.meshgrid(arange(Ny), arange(Nx), indexing="ij")
+    coords = jnp.stack([meshy, meshx], axis=-1).reshape(-1, 2)
+
+    # convert to physical coordinates: subtract array‐centre, flip y, scale, then add origin
+    # array‐centre in index space is at ((Ny-1)/2, (Nx-1)/2)
+    cy, cx = (Ny - 1) / 2.0, (Nx - 1) / 2.0
+    # row index i → physical y =  (cy - i) * sy + y0
+    # col index j → physical x =  (j  - cx) * sx + x0
+    idx_y = coords[:, 0]
+    idx_x = coords[:, 1]
+
+    phys_y = (cy - idx_y) * sy + y0
+    phys_x = (idx_x - cx) * sx + x0
+
+    return jnp.stack([phys_y, phys_x], axis=1)
