@@ -3,12 +3,12 @@ from typing import Tuple
 
 from autoarray.geometry.abstract_2d import AbstractGeometry2D
 from autoarray.inversion.linear_obj.neighbors import Neighbors
+from autoarray.structures.abstract_structure import Structure
 from autoarray.structures.mesh.abstract_2d import Abstract2DMesh
 
 
 class MockGeometry(AbstractGeometry2D):
     def __init__(self, extent):
-
         self._extent = extent
 
     @property
@@ -17,7 +17,21 @@ class MockGeometry(AbstractGeometry2D):
 
 
 class MockGrid2DMesh(Abstract2DMesh):
-    def __new__(cls, grid: np.ndarray = None, extent: Tuple[int, int, int, int] = None):
+    @property
+    def pixels(self) -> int:
+        raise NotImplementedError()
+
+    @property
+    def slim(self) -> "Structure":
+        raise NotImplementedError()
+
+    @property
+    def native(self) -> Structure:
+        raise NotImplementedError()
+
+    def __init__(
+        self, grid: np.ndarray = None, extent: Tuple[int, int, int, int] = None
+    ):
         """
         A grid of (y,x) coordinates which represent a uniform rectangular pixelization.
 
@@ -44,17 +58,14 @@ class MockGrid2DMesh(Abstract2DMesh):
             it is converted to a (float, float) structure.
         origin
             The (y,x) origin of the pixelization.
-        nearest_pixelization_index_for_slim_index
-            A 1D array that maps every grid pixel to its nearest pixelization-grid pixel.
         """
 
         if grid is None:
             grid = np.ones(shape=(1, 2))
 
-        obj = grid.view(cls)
-        obj._extent = extent
+        self._extent = extent
 
-        return obj
+        super().__init__(grid)
 
     @property
     def geometry(self):
@@ -67,6 +78,5 @@ class MockGrid2DMesh(Abstract2DMesh):
 
 class MockMeshGrid:
     def __init__(self, neighbors=None, neighbors_sizes=None):
-
         self.neighbors = Neighbors(arr=neighbors, sizes=neighbors_sizes)
         self.shape = (len(self.neighbors.sizes),)
