@@ -1,5 +1,6 @@
 import logging
 import numpy as np
+import jax.numpy as jnp
 from typing import List, Tuple, Union
 
 from autoarray.structures.vectors.abstract import AbstractVectorYX2D
@@ -13,8 +14,8 @@ logger = logging.getLogger(__name__)
 
 
 class VectorYX2DIrregular(AbstractVectorYX2D):
-    def __new__(
-        cls,
+    def __init__(
+        self,
         values: Union[
             np.ndarray, List[np.ndarray], List[List], List[Tuple[float, float]]
         ],
@@ -43,20 +44,14 @@ class VectorYX2DIrregular(AbstractVectorYX2D):
         grid
             The irregular grid of (y,x) coordinates where each vector is located.
         """
-
-        if len(values) == 0:
-            return []
-
         if type(values) is list:
             values = np.asarray(values)
 
-        obj = values.view(cls)
-        obj.grid = Grid2DIrregular(values=grid)
+        self.grid = Grid2DIrregular(values=grid)
 
-        return obj
+        super().__init__(values)
 
     def __array_finalize__(self, obj):
-
         if hasattr(obj, "grid"):
             self.grid = obj.grid
 
@@ -67,6 +62,10 @@ class VectorYX2DIrregular(AbstractVectorYX2D):
         """
         return self
 
+    @property
+    def native(self) -> np.ndarray:
+        return self
+
     @property
     def in_list(self) -> List[Tuple]:
         """
@@ -121,13 +120,13 @@ class VectorYX2DIrregular(AbstractVectorYX2D):
         squared_distances = self.grid.distances_to_coordinate_from(coordinate=centre)
         mask = squared_distances < radius
 
-        if np.all(mask == False):
+        if jnp.all(mask == False):
             raise exc.VectorYXException(
                 "The input radius removed all vectors / points on the grid."
             )
 
         return VectorYX2DIrregular(
-            values=self[mask], grid=Grid2DIrregular(self.grid[mask])
+            values=jnp.array(self.array)[mask], grid=Grid2DIrregular(self.grid[mask])
         )
 
     def vectors_within_annulus(
