@@ -1,22 +1,23 @@
+from abc import ABC
+
 import logging
 import numpy as np
+from pathlib import Path
 from typing import List, Tuple, Union
 
 from autoconf import cached_property
+from autoconf.fitsable import ndarray_via_fits_from, output_to_fits
 
 from autoarray.structures.abstract_structure import Structure
 from autoarray.structures.grids.irregular_2d import Grid2DIrregular
 
-from autoarray.structures.arrays import array_2d_util
-
 logging.basicConfig()
 logger = logging.getLogger(__name__)
 
 
-class AbstractVisibilities(Structure):
-
+class AbstractVisibilities(Structure, ABC):
     # noinspection PyUnusedLocal
-    def __new__(cls, visibilities: Union[np.ndarray, List[complex]], *args, **kwargs):
+    def __init__(self, visibilities: Union[np.ndarray, List[complex]]):
         """
         A collection of (real, imag) visibilities which are used to represent the data in an `Interferometer` dataset.
 
@@ -49,30 +50,23 @@ class AbstractVisibilities(Structure):
                     .ravel()
                 )
 
-        obj = visibilities.view(cls)
-
-        obj.ordered_1d = np.concatenate(
-            (np.real(visibilities), np.imag(visibilities)), axis=0
-        )
-
-        return obj
-
-    def __array_finalize__(self, obj):
-
-        if hasattr(obj, "ordered_1d"):
-            self.ordered_1d = obj.ordered_1d
+        super().__init__(array=visibilities)
 
     @property
     def slim(self) -> "AbstractVisibilities":
         return self
 
+    @property
+    def native(self) -> Structure:
+        return self
+
     @property
     def in_array(self) -> np.ndarray:
         """
         Returns the 1D complex NumPy array of values with shape [total_visibilities] as a NumPy float array of
         shape [total_visibilities, 2].
         """
-        return np.stack((np.real(self), np.imag(self)), axis=-1)
+        return np.stack((np.real(self.array), np.imag(self.array)), axis=-1)
 
     @property
     def in_grid(self) -> Grid2DIrregular:
@@ -91,13 +85,13 @@ class AbstractVisibilities(Structure):
 
     @cached_property
     def amplitudes(self) -> np.ndarray:
-        return np.sqrt(np.square(self.real) + np.square(self.imag))
+        return np.sqrt(np.square(self.array.real) + np.square(self.array.imag))
 
     @cached_property
     def phases(self) -> np.ndarray:
-        return np.arctan2(self.imag, self.real)
+        return np.arctan2(self.array.imag, self.array.real)
 
-    def output_to_fits(self, file_path: str, overwrite: bool = False):
+    def output_to_fits(self, file_path: Union[Path, str], overwrite: bool = False):
         """
         Output the visibilities to a .fits file.
 
@@ -112,23 +106,21 @@ class AbstractVisibilities(Structure):
         overwrite
             If a file already exists at the path, if overwrite=True it is overwritten else an error is raised.
         """
-        array_2d_util.numpy_array_2d_to_fits(
-            array_2d=self.in_array, file_path=file_path, overwrite=overwrite
-        )
+        output_to_fits(values=self.in_array, file_path=file_path, overwrite=overwrite)
 
     @property
     def scaled_maxima(self) -> Tuple[float, float]:
         """
         The maximum values of the visibilities if they are treated as a 2D grid in the complex plane.
         """
-        return np.max(self.real), np.max(self.imag)
+        return np.max(self.array.real), np.max(self.array.imag)
 
     @property
     def scaled_minima(self) -> Tuple[float, float]:
         """
         The minimum values of the visibilities if they are treated as a 2D grid in the complex plane.
         """
-        return np.min(self.real), np.min(self.imag)
+        return np.min(self.array.real), np.min(self.array.imag)
 
 
 class Visibilities(AbstractVisibilities):
@@ -187,7 +179,7 @@ class Visibilities(AbstractVisibilities):
         return cls.full(fill_value=0.0, shape_slim=shape_slim)
 
     @classmethod
-    def from_fits(cls, file_path: str, hdu: int) -> "Visibilities":
+    def from_fits(cls, file_path: Union[Path, str], hdu: int) -> "Visibilities":
         """
         Create `Visibilities` (see `AbstractVisibilities.__new__`) by loading the(real, imag) values from a .fits file.
 
@@ -202,23 +194,18 @@ class Visibilities(AbstractVisibilities):
         hdu
             The Header-Data Unit of the .fits file the visibilitiy data is loaded from.
         """
-        visibilities_1d = array_2d_util.numpy_array_2d_via_fits_from(
-            file_path=file_path, hdu=hdu
-        )
+        visibilities_1d = ndarray_via_fits_from(file_path=file_path, hdu=hdu)
         return cls(visibilities=visibilities_1d)
 
 
 class VisibilitiesNoiseMap(Visibilities):
-
     # noinspection PyUnusedLocal
-    def __new__(cls, visibilities: Union[np.ndarray, List[complex]], *args, **kwargs):
+    def __init__(self, visibilities: Union[np.ndarray, List[complex]], *args, **kwargs):
         """
         A collection of (real, imag) visibilities noise-map values which are used to represent the noise-map in
         an `Interferometer` dataset.
 
-        This data structure behaves the same as the `Visibilities` structure (see `AbstractVisibilities.__new__`). The
-        only difference is that it includes a `WeightOperator` used by `Inversion`'s which use `LinearOperators` and
-        the library `PyLops` to fit `Interferometer` data.
+        This data structure behaves the same as the `Visibilities` structure (see `AbstractVisibilities.__new__`).
 
         Parameters
         ----------
@@ -237,26 +224,4 @@ class VisibilitiesNoiseMap(Visibilities):
                     .ravel()
                 )
 
-        obj = super(VisibilitiesNoiseMap, cls).__new__(
-            cls=cls, visibilities=visibilities
-        )
-
-        obj.ordered_1d = np.concatenate(
-            (np.real(visibilities), np.imag(visibilities)), axis=0
-        )
-
-        weight_list = 1.0 / obj.in_array**2.0
-
-        obj.weight_list_ordered_1d = np.concatenate(
-            (weight_list[:, 0], weight_list[:, 1]), axis=0
-        )
-
-        return obj
-
-    def __array_finalize__(self, obj):
-
-        if hasattr(obj, "ordered_1d"):
-            self.ordered_1d = obj.ordered_1d
-
-        if hasattr(obj, "weight_list_ordered_1d"):
-            self.weight_list_ordered_1d = obj.weight_list_ordered_1d
+        super().__init__(visibilities=visibilities)
