@@ -2,6 +2,7 @@ from autoarray import numba_util as numba
 from autoarray.geometry import geometry_util as geometry
 from autoarray.mask import mask_1d_util as mask_1d
 from autoarray.mask import mask_2d_util as mask_2d
+from autoarray.operators.over_sampling import over_sample_util as over_sample
 from autoarray.structures.arrays import array_1d_util as array_1d
 from autoarray.structures.arrays import array_2d_util as array_2d
 from autoarray.structures.grids import grid_1d_util as grid_1d
@@ -10,12 +11,20 @@ from autoarray.structures.grids import sparse_2d_util as sparse
 from autoarray.layout import layout_util as layout
 from autoarray.fit import fit_util as fit
 from autoarray.inversion.pixelization.mesh import mesh_util as mesh
+from autoarray.inversion.pixelization.mesh import mesh_numba_util as mesh_numba
 from autoarray.inversion.pixelization.mappers import mapper_util as mapper
+from autoarray.inversion.pixelization.mappers import mapper_numba_util as mapper_numba
 from autoarray.inversion.regularization import regularization_util as regularization
 from autoarray.inversion.inversion import inversion_util as inversion
 from autoarray.inversion.inversion.imaging import (
     inversion_imaging_util as inversion_imaging,
 )
+from autoarray.inversion.inversion.imaging import (
+    inversion_imaging_util as inversion_imaging,
+)
+from autoarray.inversion.inversion.imaging import (
+    inversion_imaging_numba_util as inversion_imaging_numba,
+)
 from autoarray.inversion.inversion.interferometer import (
     inversion_interferometer_util as inversion_interferometer,
 )
